#HOTFIX_VERSION:cpi-6.0.1.1100
package EDRu;

# verify an entered NIC is in lettersnumbers format
sub isnic {
    my ($nic) = @_;
    # e3465678, change the judgement condition of entered NIC format
    return 1 if ($nic=~/^\w+[\w\.:-]*$/mx);
    return 0;
}

package Padv::SunOS;

sub get_updatelevel_by_patchid {
    my ($arch, $osver, $patchid) = @_;
 
    # Get the following kernel patchid and update level matrix from:
    #   http://blogs.oracle.com/patch/entry/solaris_10_kernel_patchid_progression
   
    my $patch_matrix = << 'EOF';
# arch    os_version   update  patchid                 description
# ---------------------------------------------------------------------------------------------------
sparc,    5.10,        11,     147147-26,              Solaris 10 Update 11 Kernel PatchID
sparc,    5.10,        10,     144500-[0-9][0-9],      Solaris 10 Update 10 Kernel PatchID
sparc,    5.10,        9,      142909-17,              Solaris 10 9/10 (Update 9) Kernel PatchID
sparc,    5.10,        8,      141444-09,              Solaris 10 10/09 (Update 8) Kernel PatchID
sparc,    5.10,        7,      139555-08,              Solaris 10 5/09 (Update 7) Kernel PatchID
sparc,    5.10,        6,      137137-09,              Solaris 10 10/08 (Update 6) Kernel PatchID
sparc,    5.10,        5,      127127-11,              Solaris 10 5/08 (Update 5) Kernel PatchID
sparc,    5.10,        4,      120011-14,              Solaris 10 8/07 (Update 4) Kernel PatchID
sparc,    5.10,        3,      118833-33,              Solaris 10 11/06 (Update 3)
sparc,    5.10,        2,      118833-17,              Solaris 10 6/06 (Update 2)
sparc,    5.10,        1,      118822-25,              Solaris 10 1/06 (Update 1)

sparc,    5.10,        11,     148888-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 1/13 (Update 11)
sparc,    5.10,        10,     147440-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 8/11 (Update 10)
sparc,    5.10,        9,      144488-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 9/10 (Update 9)
sparc,    5.10,        8,      142900-(0[1-9]|1[0-5]), Kernel Bug Fixes post Solaris 10 10/09 (Update 8)
sparc,    5.10,        7,      141414-(0[1-9]|10),     Kernel Bug Fixes post Solaris 10 5/09 (Update 7)
sparc,    5.10,        6,      138888-0[1-8],          Kernel Bug Fixes post Solaris 10 10/08 (Update 6)
sparc,    5.10,        5,      137111-0[1-8],          Kernel Bug Fixes post Solaris 10 5/08 (Update 5)
sparc,    5.10,        4,      127111-(0[1-9]|1[0-1]), Kernel Bug Fixes post Solaris 10 8/07 (Update 4)
sparc,    5.10,        3,      125100-(0[4-9]|10),     Kernel Bug Fixes post Solaris 10 8/07 (Update 4)

i386,     5.10,        11,     147148-26,              Solaris 10 Update 11 Kernel PatchID
i386,     5.10,        10,     144501-[0-9][0-9],      Solaris 10 Update 10 Kernel PatchID
i386,     5.10,        9,      142910-17,              Solaris 10 9/10 (Update 9) Kernel PatchID
i386,     5.10,        8,      141445-09,              Solaris 10 10/09 (Update 8) Kernel PatchID
i386,     5.10,        7,      139556-08,              Solaris 10 5/09 (Update 7) Kernel PatchID
i386,     5.10,        6,      137138-09,              Solaris 10 10/08 (Update 6) Kernel PatchID
i386,     5.10,        5,      127128-11,              Solaris 10 5/08 (Update 5) Kernel PatchID
i386,     5.10,        4,      120012-14,              Solaris 10 8/07 (Update 4) Kernel PatchID
i386,     5.10,        3,      118855-33,              Solaris 10 11/06 (Update 3)
i386,     5.10,        2,      118855-14,              Solaris 10 6/06 (Update 2)
i386,     5.10,        1,      118844-26,              Solaris 10 1/06 (Update 1)

i386,     5.10,        11,     148889-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 1/13 (Update 11)
i386,     5.10,        10,     147441-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 8/11 (Update 10)
i386,     5.10,        9,      144489-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 9/10 (Update 9)
i386,     5.10,        8,      142901-(0[1-9]|1[0-5]), Kernel Bug Fixes post Solaris 10 10/09 (Update 8)
i386,     5.10,        7,      141415-(0[1-9]|10),     Kernel Bug Fixes post Solaris 10 5/09 (Update 7)
i386,     5.10,        6,      138889-0[1-8],          Kernel Bug Fixes post Solaris 10 10/08 (Update 6)
i386,     5.10,        5,      137112-0[1-8],          Kernel Bug Fixes post Solaris 10 5/08 (Update 5)
i386,     5.10,        4,      127112-(0[1-9]|1[0-1]), Kernel Bug Fixes post Solaris 10 8/07 (Update 4)
i386,     5.10,        3,      125101-(0[1-9]|10),     Kernel Bug Fixes post Solaris 10 8/07 (Update 4)
 
EOF
 
    foreach my $line (split(/\n+/, $patch_matrix)) {
        next if ($line=~/^#/);
        my @fields = split(/,\s*/, $line);
        next if (@fields < 5);
 
        for my $i (0 .. $#fields) {
            $fields[$i] =~ s/^\s*//;
            $fields[$i] =~ s/\s*$//;
        }
 
        if ($arch eq $fields[0] && $osver eq $fields[1]) {
            if ($patchid =~ /^$fields[3]$/) {
                return $fields[2];
            }
        }
    }
    return "";
}

package Padv::Linux;

sub detect_linux_by_dmidecode_sys {
    my ($padv, $sys) = @_;

    my $out = $sys->cmd("_cmd_dmidecode 2>/dev/null");

    my ($flag, $bios_vendor, $mfgr, $product) = ("", "", "", "");
    foreach (split(/\n+/, $out)) {
        if (/^\S/) {
            if (/^BIOS Information/) {
                $flag = "bios_info";
            } elsif (/^System Information/) {
                $flag = "system_info";
            } else {
                $flag = "other";
            }
            next;
        }

        if ($flag eq "bios_info") {
            $bios_vendor = $1 if (/Vendor:\s*(.*)/i);
        }
        if ($flag eq "system_info") {
            $mfgr = $1 if (/Manufacturer:\s*(.*)/i);
            $product = $1 if (/Product Name:\s*(.*)/i);
        }
    }

    $sys->{virtual_type} = $padv->VIRT_VMWARE if ($mfgr =~ /VMWare/i);
    $sys->{virtual_type} = $padv->VIRT_KVM . " or " . $padv->VIRT_QEMU if ($bios_vendor =~ /QEMU/i);
    $sys->{virtual_type} = $padv->VIRT_VIRTUALPC if ($mfgr =~ /microsoft/i && $product =~ /virtual machine/i);
    return;
}

package Rel::UXRT601::SunOS;

sub init_plat {
    my ($rel) = @_;

    $rel->{driververs}='6.0.1';

    $rel->{prods}=EDRu::arrdel($rel->{prods}, 'SFCFSRAC60');
    $rel->{menuprods}=EDRu::arrdel($rel->{menuprods}, 'VCS60') if ($rel->{oem});
    $rel->{upgradeprods} = [ qw(SFRAC60 SVS60 SFCFSHA60 SFHA60 SF60 VCS60 VM60 DMP60 FS60) ];

    $rel->{obsoleted_but_still_support_pkgs} = [qw(VRTSat50)];
    $rel->{'commands_need_be_checked'}=[ qw(adddrv fstyp getconf hostid isainfo kstat ldd mdb ndd netstat nm nohup pkgadd pkgchk pkginfo pkgrm prtconf prtdiag rmdrv route swap) ];
    $rel->{'commands_need_be_checked_for_5.9'}=[ qw(patchadd patchrm showrev) ];
    $rel->{'commands_need_be_checked_for_5.10'}=[ qw(bootadm patchadd patchrm showrev zoneadm) ];
    $rel->{'commands_need_be_checked_for_patchupgrade'}=[ qw(gunzip) ];
    $rel->{'commands_need_be_checked_for_patchupgrade_5.10'}=[ qw(patchadd) ];

    return;
}

package EDR;

sub rsh_sys {
    my ($edr,$sys) = @_;
    my ($msg,$echo,$rsh_deny,$test,$ssh_deny,$rsh_failure,$mkdir,$ssh_failure);
    return '' if ($sys->{stop_checks});
    $sys->{rsh}=Cfg::opt('rsh') ? 'rsh' : 'ssh';
    $sys->{rcp}=Cfg::opt('rsh') ? 'rcp' : 'scp';
    $ssh_failure=$rsh_failure='';
    $sys->set_value('com_deny', 0);
    $sys->set_value('sshrsh_failure', 0);
    # When connecting to a host using ssh for the first time and the host name
    # doesn't exist in $HOME/.ssh/known_hosts, ssh will print an extra line
    # of warning message and add the host name
    # After the host name is added, this warning message will not appear again
    # Run $sys->cmd("echo") twice to ensure banner doesn't include extra words
    #to avoid ssh prompts for password when ssh is not configured
    unless (Cfg::opt('rsh')) {
        $test=$sys->cmd('echo');
        # chomp the last return character caused by "echo" command
        chomp ($test);
        if ($edr->{cmdexit}) {
            #print out the message when ssh is not configured.
            if ($test=~/Permission denied/m) {
                $ssh_failure .= Msg::new("ssh permission was denied on $sys->{sys}\n")->{msg};
                $ssh_deny = 1;
            } else {
                $ssh_failure .= Msg::new("ssh exited $edr->{cmdexit} on $sys->{sys}\n")->{msg};
            }
            Msg::log($ssh_failure);
            $sys->{rsh}='rsh';
            $sys->{rcp}='rcp';
        } else {
            $sys->{banner}=$sys->cmd('echo');
            chomp ($sys->{banner});
            # chomp the last return character caused by "echo" command
            $sys->set_value('banner', $sys->{banner});
        }
    }
    $sys->set_value('rsh', $sys->{rsh});
    $sys->set_value('rcp', $sys->{rcp});
    if ($sys->{rsh} eq 'rsh') {
        $test=$sys->cmd('echo');
        # chomp the last return character caused by "echo" command
        chomp ($test);
        Msg::log("Using $sys->{rsh} for communication with $sys->{sys}");
        if ($edr->{cmdexit}) {
            if ($test=~/permission denied|permission is denied|Host name for your address unknown|login incorrect/mi) {
                $rsh_failure .= Msg::new("rsh permission was denied on $sys->{sys}\n")->{msg};
                $rsh_deny = 1;
            } else {
                $rsh_failure .= Msg::new("rsh exited $edr->{cmdexit} on $sys->{sys}\n")->{msg};
            }
        }  else {
            $sys->{banner}=$sys->cmd('echo');
            chomp ($sys->{banner});
            $sys->set_value('banner', $sys->{banner});
        }
    }
 
    if ($rsh_failure) {
        if (Cfg::opt('rsh')) {
            $msg=Msg::new("${rsh_failure}rsh is needed to be setup between the local node and $sys->{sys} for communication");
        } else {
            $msg=Msg::new("${ssh_failure}${rsh_failure}either ssh or rsh is needed to be setup between the local node and $sys->{sys} for communication");
        }
        if ($rsh_deny || $ssh_deny) {
            $sys->set_value('com_deny', 1);
        } else {
            $sys->set_value('sshrsh_failure', 1);
        }
        $sys->push_stop_checks_error($msg);
        return '';
    }
    return 1;
}

package Prod::VCS60::Common;

sub check_cluster_members_sys {
    my ($prod,$sys) = @_;
    my ($m,$cprod,$msg,$sys0,$n,$syslist,$conf);
    $syslist=CPIC::get('systems');
    $cprod=CPIC::get('prod');
    $conf = $prod->get_config_sys($sys);
    if ($conf) {
        if ($sys->system1) {
            $m = scalar(@{$conf->{systems}});
            $n = scalar(@$syslist);
            return 1 if($m == $n);
            if ( $m < $n) {
                $msg = Msg::new("The entered systems belong to different clusters. Only the members of the same cluster can be upgraded as a whole.");
                $sys->push_error($msg);
            } elsif ($m > $n) {
                return 1 unless ($cprod eq 'VCS60' || $cprod eq 'SFHA60');
                $msg = Msg::new("Not all systems of the cluster, $conf->{clustername}, have been entered to be upgraded. It is recommended that all cluster systems are upgraded together, except when you plan to perform phased upgrade on the set of systems.");
                $sys->push_warning($msg) unless(Cfg::opt("upgrade_kernelpkgs"));
            }
        } else {
            $sys0 = $$syslist[0];
            $conf = $prod->get_config_sys($sys0);
            return 0 unless ($conf);
            return 1 if(EDRu::inarr($sys->{sys},@{$conf->{systems}}));
            my $sysname = $sys->{vcs_sysname};
            return 1 if(EDRu::inarr($sysname, @{$conf->{systems}}));
            $msg = Msg::new("$sys->{sys} does not belong to the cluster, $conf->{clustername}. Only the members of the same cluster can be upgraded as a whole.");
            $sys->push_error($msg);
        }
    } else {
        $msg = Msg::new("Cannot find valid $prod->{abbr} configuration information on $sys->{sys}");
        $sys->push_warning($msg);
    }
    return 0;
}


sub dynupgrade_pre_upgrade_sys {
    my ($prod,$sys) = @_;
    my ($rootpath,$msg,$uuid,$upgradepath,$sourcefilebase,$line,$rtn,$sourcefile,$maincf);

    $rootpath=Cfg::opt('rootpath');
    $upgradepath= EDR::tmpdir().'/upgrade';
    $uuid=EDR::get('uuid');
    # below steps will ensure the old config files to be available

    # Pre Step 1- backup old cf files
    $sys->cmd("_cmd_rmr $upgradepath");

    $rtn = $prod->backup_cf_files_sys($sys, "$rootpath$prod->{configdir}", "$upgradepath/old_config");
    return 0 unless ($rtn);

    $rtn = $prod->backup_cf_files_sys($sys, "$rootpath$prod->{confdir}", "$upgradepath/old_conf");
    return 0 unless ($rtn);

    $msg = Msg::new("\nFailed to upgrade because the system configuration files are not available\n");

    # Pre Step 2- create cmd files for old user config
    if ($rootpath) {
        $maincf=$sys->readfile("$rootpath$prod->{configdir}/main.cf");
        $sys->cmd("_cmd_mv $rootpath$prod->{configdir}/main.cf $rootpath$prod->{configdir}/main.cf.$uuid");
        $maincf=~s/\s*include\s+\"(\/\S+)\"/\ninclude \"$rootpath$1\"/g;
        $sys->writefile($maincf,"$rootpath$prod->{configdir}/main.cf");
    }
    # Check if main.cf exist, if not exist, print warning and do not translate any configuration.
    unless ($sys->exists($prod->{maincf})) {
        $msg = Msg::new("main.cf does not exist on $sys->{sys}");
        $sys->push_warning($msg);
        return 0;
    }
    # Verify if main.cf is valid before upgrade.
    $rtn= $sys->cmd("$prod->{cmd_hacf} -verify $rootpath$prod->{configdir}");
    if (EDR::cmdexit() != 0) {
        $msg = Msg::new("main.cf is not valid:\n$rtn\nFix the errors, and verify the main.cf file before upgrade by running \'$prod->{cmd_hacf} -verify $rootpath$prod->{configdir}\'");
        $sys->push_error($msg);
        return 0;
    }

    $rtn = $prod->translate_file_cf2cmd_sys( $sys, "$rootpath$prod->{configdir}", 1);
    $sys->cmd("_cmd_mv $rootpath$prod->{configdir}/main.cf.$uuid $rootpath$prod->{configdir}/main.cf") if ($rootpath);
    unless($rtn) {
        $sys->push_error($msg);
        return 0;
    }

    # Fix 2026573.
    $prod->padv_mend_old_cf_sys($sys, "$upgradepath/old_config");
    $prod->padv_mend_old_cf_sys($sys, "$upgradepath/old_conf");

    # Pre Step 3- create cmd files for old sys conf
    $rtn = $prod->translate_file_cf2cmd_sys( $sys, "$upgradepath/old_conf");
    unless($rtn) {
        $sys->push_error($msg);
        return 0;
    }

    return 1;
}

sub ru_prestop_sys {
    my ($prod,$sys)=@_;
    my (@procs,$proc,$procobj,$cprod,$msg);
    if (Cfg::opt('upgrade_kernelpkgs')) {
        @procs=qw(gab60 vxfen60);
        for my $proc (@procs) {
             $procobj=$sys->proc($proc);
             $procobj->prestop_sys($sys);
        }
        $cprod=CPIC::get('prod');
        if($cprod=~/^SFRAC\d+/mx){
            $procobj=$sys->proc('vcsmm60');
            $procobj->prestop_sys($sys) if($procobj);
        }
        # Check if had is stopped
        $proc='had60';
        $procobj=$sys->proc($proc);
        if ($procobj->check_sys($sys,'stop')) {
            # Still running, execute hastop command
            $sys->cmd('_cmd_hastop -local -evacuate 2>/dev/null');
            if (EDR::cmdexit() != 0) {
                $msg=Msg::new("Failed to stop VCS on $sys->{sys}. Contact Symantec technical support or refer to the installation guide for further troubleshooting.");
                # Stop had failed for some reason
                $sys->push_error($msg);
                return 1;
            }
            sleep 2;
        }
    }
    return 0;
}

package Prod::VCS60::SunOS;

sub padv_maincf_upgrade_precheck_sys {
    my ($prod,$sys,$treeref,$vcsvers) = @_;
    my ($rootpath,@grps,@grp_syslist,@reses,$res_type,$consistent,$msg);
    my ($grp_container_type,$grp_container_names,$res_container_type,$res_container_names,$sys_container,$compvers_rtn,$zone_num,$multi_zone,$vcspkg);

    $rootpath=Cfg::opt('rootpath') || '';

    #return unless ($sys->exists($prod->{maincf}));

    # Check containerization setting in main.cf
    $vcspkg=$prod->pkg('VRTSvcs60');
    if (defined $vcspkg) {
        $vcsvers=$vcspkg->version_sys($sys);
    }
    if ($vcsvers && (EDRu::compvers($vcsvers,'5.1') == 2) && $sys->{zone}) {
        Msg::log("Checking the zone setting informations of main.cf on $sys->{sys}");

        $compvers_rtn = EDRu::compvers($prod->{vers},'6.0.100',3);
        # Retrieve Service Group and Resource hierarchy informations from main.cf of 5.0 version
        # get group list
        @grps = Prod::VCS60::Common::dynupgrade_get_grplist($treeref);
        for my $grp_name (@grps) {
            next unless ($grp_name);

            # get resources list
            @reses = Prod::VCS60::Common::dynupgrade_get_res_from_grp($treeref, $grp_name);
            next unless (@reses);
            $zone_num=0;
            for my $res_name (@reses) {
                next unless ($res_name);
                $res_type = Prod::VCS60::Common::dynupgrade_get_restype_of_resname($treeref,$res_name);

                # Currently only support 'Zone' type
                if ($res_type eq 'Zone') {
                    $zone_num++;
                }
            }
            $sys->set_value("maincf_grp_multi_zone,$grp_name", 1) if ($zone_num > 1);
        }

        for my $grp_name (@grps) {
            next unless ($grp_name);

            # get group system list
            @grp_syslist = Prod::VCS60::Common::dynupgrade_get_systemlist_of_grpname($treeref,$grp_name);
            next unless (@grp_syslist);

            # get resources list
            @reses = Prod::VCS60::Common::dynupgrade_get_res_from_grp($treeref, $grp_name);
            next unless (@reses);

            # get group container settings
            $grp_container_type=undef;
            $grp_container_names=undef;
            $consistent=1;
            $multi_zone=defined $sys->{maincf_grp_multi_zone}{$grp_name} ? 1 : 0;

            for my $res_name (@reses) {
                next unless ($res_name);

                $res_type = Prod::VCS60::Common::dynupgrade_get_restype_of_resname($treeref,$res_name);

                # Currently only support 'Zone' type
                if ($res_type eq 'Zone') {
                    $res_container_type='Zone';
                    $res_container_names = Prod::VCS60::Common::dynupgrade_get_attrvalue_of_resname_array($treeref, $res_name, 'ZoneName');
                } else {
                    $res_container_type = Prod::VCS60::Common::dynupgrade_get_attrvalue_of_typename($treeref,$res_type,'ContainerType');
                    if ($res_container_type) {
                        $res_container_names = Prod::VCS60::Common::dynupgrade_get_attrvalue_of_resname_array($treeref, $res_name, 'ContainerName');
                    }
                }
                next unless ($res_container_type && $res_container_names && @{$res_container_names});

                # Check if container type are consistent across the group.
                # Applicable for upgrading from version before 5.1 to versions do not 
                # support ResContainerInfo attribute(Before 6.0.1)
                if ($compvers_rtn == 2 || !$multi_zone) {
                    if ($grp_container_type) {
                        if ($grp_container_type ne $res_container_type) {
                            Msg::log("ContainerType value '$res_container_type' for resource '$res_name' in group '$grp_name' is not consistent with other resources");
                            $consistent=0;
                            next;
                        }
                    } else {
                        $grp_container_type=$res_container_type;
                    }
                }

                # get container name for each system
                my %sys_containers=();
                my $allsys_container='';
                for my $res_container (@{$res_container_names}) {
                    if ($res_container=~/^(\S+)\s+-sys\s+(\S+)/) {
                        $sys_containers{$2}=$1;
                        if ($compvers_rtn != 2 && $multi_zone) {
                            $sys->set_value("maincf_res_container_name,$grp_name,$res_name,$2", $1);
                        }
                    } elsif ($res_container=~/^\s*(\S+)\s*$/) {
                        $allsys_container=$1;
                        if ($compvers_rtn != 2 && $multi_zone) {
                            $sys->set_value("maincf_res_container_name,$grp_name,$res_name", $1);
                        }
                    }
                }

                if ($allsys_container || keys %sys_containers) {
                    $sys->set_value("maincf_res_container_type,$grp_name,$res_name", $res_container_type);
                    next if ($compvers_rtn != 2 && $multi_zone);
                    for my $sysi (@grp_syslist) {
                        $sys_container= $sys_containers{$sysi} || $allsys_container || '';
                        if (defined $grp_container_names &&
                            defined $grp_container_names->{$sysi} &&
                            $sys_container ne $grp_container_names->{$sysi}) {
                            Msg::log("ContainerName value '$sys_container' for resource '$res_name' in group '$grp_name' is not consistent with other resources");
                            $consistent=0;
                        } else {
                            $grp_container_names->{$sysi}=$sys_container;
                        }
                    }
                }
            }

            if (!$consistent) {
                $msg=Msg::new("In the file '$rootpath$prod->{maincf}' on $sys->{sys}, the ContainerName and ContainerType value of all resources in group '$grp_name' are not consistent");
                $sys->push_error($msg);
            } elsif ($grp_container_names && ($compvers_rtn == 2 || !$multi_zone)) {
                $sys->set_value("maincf_grp_container_type,$grp_name", $grp_container_type) if ($grp_container_type);
                for my $sysi (@grp_syslist) {
                    $sys_container=$grp_container_names->{$sysi};
                    $sys->set_value("maincf_grp_container_name,$grp_name,$sysi", $sys_container) if ($sys_container);
                }
            }
        }
    }

    return 1;
}

sub padv_maincf_upgrade_sys {
    my ($prod,$sys,$treeref_result,$treeref_newsys) = @_;
    my ($grp_name,$grp_container_name,$grp_container_type,@grp_syslist,$sysi,@grps);
    my (@reses,$res_name,$res_type,$res_container_type,$res_container_name,$container_opts);
    my ($container_settings,@container_settings);

    # Service groups with multiple zones
    for my $grp_name (sort keys %{$sys->{maincf_res_container_name}}) {
        # get group system list
        @grp_syslist = Prod::VCS60::Common::dynupgrade_get_systemlist_of_grpname($treeref_result,$grp_name);
        next unless (@grp_syslist);

        # get resources list
        @reses = Prod::VCS60::Common::dynupgrade_get_res_from_grp($treeref_result, $grp_name);
        next unless (@reses);
        for my $res_name (@reses) {
            $res_container_type=$sys->{maincf_res_container_type}{$grp_name}{$res_name};
            if($res_container_type && ($res_container_type eq 'Zone')) {
                if (ref($sys->{maincf_res_container_name}{$grp_name}{$res_name})) {
                    for my $sysi (@grp_syslist) {
                        $res_container_name = $sys->{maincf_res_container_name}{$grp_name}{$res_name}{$sysi};
                        if ($res_container_name) {
                            $container_settings.="hares -modify $res_name ResContainerInfo Name $res_container_name Type $res_container_type Enabled 1 -sys $sysi\n";
                        }
                    }
                } else {
                    $res_container_name = $sys->{maincf_res_container_name}{$grp_name}{$res_name};
                    if ($res_container_name) {
                        $container_settings.="hares -modify $res_name ResContainerInfo Name $res_container_name Type $res_container_type Enabled 1\n";
                    }
                }
            }
            $res_type = Prod::VCS60::Common::dynupgrade_get_restype_of_resname($treeref_result,$res_name);
            next if ($res_type eq 'Zone');

            $container_opts = Prod::VCS60::Common::dynupgrade_get_attrvalue_of_typename($treeref_newsys,$res_type,'ContainerOpts');
            if($res_container_type) {
                if ($res_container_type eq 'Zone') {
                    $container_settings.="hares -override $res_name ContainerOpts\nhares -modify $res_name ContainerOpts RunInContainer 1 PassCInfo 0\n"
                        if ($container_opts !~ /RunInContainer\s+1\s+PassCInfo\s+0/mx);
                } else {
                    $container_settings.="hares -override $res_name ContainerOpts\nhares -modify $res_name ContainerOpts RunInContainer 0 PassCInfo 1\n"
                        if ($container_opts !~ /RunInContainer\s+0\s+PassCInfo\s+1/mx);
                }
            } else {
                $container_settings.="hares -override $res_name ContainerOpts\nhares -modify $res_name ContainerOpts RunInContainer 0 PassCInfo 0\n"
                    if ($container_opts =~ /RunInContainer\s+\d+\s+PassCInfo\s+\d+/mx);
            }
        }
    }
    # Service groups with only 1 zone
    for my $grp_name (sort keys %{$sys->{maincf_grp_container_name}}) {
        next unless ($sys->{maincf_grp_container_name}{$grp_name});
        next unless Prod::VCS60::Common::dynupgrade_get_node_from_tree($treeref_result,'GRP',$grp_name);
        @grp_syslist = Prod::VCS60::Common::dynupgrade_get_systemlist_of_grpname($treeref_result,$grp_name);
        next unless (@grp_syslist);

        for my $sysi (@grp_syslist) {
            $grp_container_name = $sys->{maincf_grp_container_name}{$grp_name}{$sysi};
            $grp_container_type = $sys->{maincf_grp_container_type}{$grp_name};

            next unless ($grp_container_type && $grp_container_name);

            my $tmp_cmds = "hagrp -modify $grp_name ContainerInfo Type $grp_container_type Name $grp_container_name Enabled 1 -sys $sysi\n";
            # if os upgraded, CPI will use hacf of PBE, and following new attr will make the old hacf fail
            # so the cmds will be print to user
            if (Cfg::opt('rootpath') && $sys->{osupgraded}) {
                $sys->{dyn_manual_cmd_lu} .= $tmp_cmds;
            } else {
                $container_settings.=$tmp_cmds;
            }
        }

        # get resources list
        @reses = Prod::VCS60::Common::dynupgrade_get_res_from_grp($treeref_result, $grp_name);
        for my $res_name (@reses) {
            $res_type = Prod::VCS60::Common::dynupgrade_get_restype_of_resname($treeref_result,$res_name);
            next if ($res_type eq 'Zone');

            $container_opts = Prod::VCS60::Common::dynupgrade_get_attrvalue_of_typename($treeref_newsys,$res_type,'ContainerOpts');
            $res_container_type=$sys->{maincf_res_container_type}{$grp_name}{$res_name};
            if($res_container_type) {
                if ($res_container_type eq 'Zone') {
                    $container_settings.="hares -override $res_name ContainerOpts\nhares -modify $res_name ContainerOpts RunInContainer 1 PassCInfo 0\n"
                        if ($container_opts !~ /RunInContainer\s+1\s+PassCInfo\s+0/mx);
                } else {
                    $container_settings.="hares -override $res_name ContainerOpts\nhares -modify $res_name ContainerOpts RunInContainer 0 PassCInfo 1\n"
                        if ($container_opts !~ /RunInContainer\s+0\s+PassCInfo\s+1/mx);
                }
            } else {
                $container_settings.="hares -override $res_name ContainerOpts\nhares -modify $res_name ContainerOpts RunInContainer 0 PassCInfo 0\n"
                    if ($container_opts =~ /RunInContainer\s+\d+\s+PassCInfo\s+\d+/mx);
            }
        }
    }
    Msg::log("Upgrade main.cmd for user configuration: $container_settings\n");
    @container_settings = split (/\n+/,$container_settings);
    $treeref_result = Prod::VCS60::Common::dynupgrade_import_lines2tree(\@container_settings, $treeref_result);
    return $treeref_result;
}


package Prod::SFCFSHA60::Common;
@Prod::SFCFSHA60::Common::ISA = qw(Prod);

sub ru_prestop_sys {
    my ($prod,$sys) = @_;
    my ($ver1,$ver2,$ver3,$output);
    my ($msg,$edr,$vcs);
    $vcs=$sys->prod('VCS60');

    if (Cfg::opt('upgrade_kernelpkgs')) {
        $output=$sys->cmd('_cmd_vxdctl protocolversion 2>&1');
        $ver1=$1 if($output=~/Cluster\s+running\s+at\s+protocol\s+(\d+)/mx);
        $ver2=$sys->cmd("_cmd_fsclustadm protoversion 2>&1 | _cmd_grep 'local' | awk '{print \$3}'");
        $ver3=$sys->cmd("_cmd_odmclustadm protoversion 2>&1 | _cmd_grep 'local' | awk '{print \$3}'");

        Msg::log("Starting Stopping CVM in rolling upgrade on $sys->{sys}");
        if ($prod->stop_agents_sys($sys)) {
            # Print Error Message Here
            # Directly exit CPI
            $msg=Msg::new("Failed to stop CVM/CFS agents, contact Symantec technical support or refer to the installation guide for further troubleshooting.");
            $sys->push_error($msg);
            return 1;
        }
        Msg::log("Finished Stopping CVM in rolling upgrade on $sys->{sys}");
        # if ru_prestop_sys failed, return immediately.
        return 1 if ($vcs->ru_prestop_sys($sys));
        #set fs/vm/odm version
        $sys->cmd("_cmd_vxdctl setversion $ver1 2>&1") if($ver1);
        $sys->cmd("_cmd_fsclustadm protoset $ver2 2>&1") if($ver2);
        $sys->cmd("_cmd_odmclustadm protoset $ver3 2>&1") if($ver3);
    }
    return;
}

sub upgrade_prestop_sys {
    my ($prod,$sys) = @_;

    my $vcs=$prod->prod('VCS60');
    my $vm=$prod->prod('VM60');
    my $sfha=$prod->prod('SFHA60');

    $vm->upgrade_prestop_sys($sys) if ($vm->can('upgrade_prestop_sys'));
    $sfha->upgrade_rolling_post_sys($sys) if($sys->system1 &&  Cfg::opt('upgrade_nonkernelpkgs'));
    if(Cfg::opt('upgrade_kernelpkgs')) {
        return if ($prod->ru_prestop_sys($sys));
    }
    if (!Cfg::opt('rootpath')) {
        $prod->stop_agents_sys($sys) unless(Cfg::opt(qw(upgrade_kernelpkgs upgrade_nonkernelpkgs)));
    }
    $vcs->upgrade_prestop_sys($sys) if ($vcs->can('upgrade_prestop_sys'));
    return;
}

sub stop_agents_sys {
    my ($prod,$sys) = @_;
    my ($cfscluster,$fsclustadm,$failure);

    $cfscluster = $sys->cmd_bin('cfscluster');
    $fsclustadm = $sys->cmd_bin('fsclustadm');
    unless ($sys->exists($cfscluster) && $sys->exists($fsclustadm)){
        if ($sys->system1) {
            if ((Cfg::opt('upgrade'))) {
                EDRu::create_flag('stop_cvm_cfs_agents_done');
            }
            EDRu::create_flag('sfcfs_stop_agents_all_done');
        }
        return 0;
    }

    return 1 if ($prod->stop_odm_sys($sys));

    # If upgrade, do not stop VCS.
    # Just stop the CVM and CFS Agents. (Offline group cvm)
    if ((Cfg::opt('upgrade'))) {
        if ($sys->system1) {
            $failure=$prod->stop_cvm_cfs_agents($sys) if ($sys->{prod_running});
            EDRu::create_flag('stop_cvm_cfs_agents_done');
        } else {
            EDRu::wait_for_flag('stop_cvm_cfs_agents_done');
        }
        return 1 if ($failure);
        return 0;
    }

    if ($sys->system1) {
        $prod->sfcfs_stop_agents_all($sys);
        EDRu::create_flag('sfcfs_stop_agents_all_done');
    } else {
        EDRu::wait_for_flag('sfcfs_stop_agents_all_done');
    }
    return 0;
}

# stop CVM and CFS Agents
sub stop_cvm_cfs_agents {
    my ($prod,$sys) = @_;
    my ($padv,$port_f,$vcs,@deps,@cvmdeps,$syslist,$port_q,$offlinefailuregroups,$offlinefailuregrouplist);
    $syslist=CPIC::get('systems');
    $vcs = $prod->prod('VCS60');
    $padv=$sys->padv;

    Msg::log('Stopping CVM and CFS Agents');
    @cvmdeps = $prod->all_groups_depend_on_group($sys, 'cvm');
    if (@cvmdeps) {
        push(@deps, @cvmdeps);
    }
    push(@deps, 'cvm');
    $offlinefailuregroups=$prod->offline_groups($sys,\@deps);
    if (@{$offlinefailuregroups}) {
        # If there're groups depends on CVM and failed to offline
        # No need to go further and attempt to stop CVM/CFS related agents.
        $offlinefailuregrouplist=join(',',@{$offlinefailuregroups});
        Msg::log("The following service groups failed to offline: $offlinefailuregrouplist");
        return 1;
    }
   
    for my $sys (@$syslist) {
        $port_f = $sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port f gen' ");
        my $sysname = $sys->{vcs_sysname};
        $sys->cmd("$vcs->{bindir}/hares -offline vxfsckd -sys $sysname")
            if (($port_f >=1) && ($sys->proc_pids('vxfsckd')));
        $sys->cmd('_cmd_fsclustadm cfsdeinit 2>/dev/null');
        return 1 if (EDR::cmdexit());
        # Stop Qlog for SunOS
        if ($sys->sunos()) {
            $port_q = $sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port q gen' ");
            $padv->unload_driver_sys($sys,'qlog') if ($port_q >= 1);
            $port_q = $sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port q gen' ");
            if ($port_q >= 1) {
                Msg::log("Failed to unload driver Qlog");
                return 1;
            }
        }
    }
    Msg::log('CVM and CFS Agents are stopped');
    return 0;
}

sub offline_groups {
    my ($prod,$sys,$deps) = @_;
    my ($grpstate,$sysi,$grp,$sys_master,$vxdctl_mode,$state,$vcs,$count,$syslist,@fields,@master_nodes,$temp_sys);
    my (@offlinefailuregroups);
    $syslist=CPIC::get('systems');
    $vcs = $prod->prod('VCS60');

    for my $dep (@{$deps}) {
        $grpstate = $sys->cmd("$vcs->{bindir}/hagrp -display $dep -attribute State");
        @master_nodes = ();
        for my $line (split(/\n/,$grpstate)) {
            next if ($line=~/^#Group/);

            @fields = split(/\s+/, $line);
            $grp=$fields[0];
            $sysi=$fields[2];
            $state=$fields[3];

            # Only Offline on nodes passed to CPI.
            # This is done to handle phased upgrades.
            $count=0;
            for my $obj_sysi (@$syslist) {
                my $sysname = $obj_sysi->{vcs_sysname};
                if ($sysname && ($sysi eq $sysname)) {
                    $temp_sys=$obj_sysi;
                    last;
                }
                $count ++;
            }
            next if ($count == scalar(@$syslist));
            # Offline Slave nodes before Master node.
            if ($grp eq 'cvm') {
                $vxdctl_mode = $temp_sys->cmd("_cmd_vxdctl -c mode 2>/dev/null | _cmd_grep 'mode:'");
                if ($vxdctl_mode=~/MASTER/m) {
                    push(@master_nodes,$sysi);
                } else {
                    push (@offlinefailuregroups,$grp) if ($prod->offline_group_sys($sys,$sysi,$grp,$state));
                }
            } else {
                push (@offlinefailuregroups,$grp) if ($prod->offline_group_sys($sys,$sysi,$grp,$state));
            }
        }
        for my $sys_master (@master_nodes) {
            push (@offlinefailuregroups,$grp) if ($prod->offline_group_sys($sys,$sys_master,$grp,$state));
        }
    }
    return \@offlinefailuregroups;
}

sub offline_group_sys {
    my ($prod,$sys,$sysi,$grp,$state) = @_;
    my ($vcs);
    $vcs = $prod->prod('VCS60');
    if ($state =~ /FAULTED/m) {
        $sys->cmd("$vcs->{bindir}/hagrp -clear $grp -sys $sysi");
    }
    if ($state =~ /ONLINE|PARTIAL/mx) {
        $sys->cmd("$vcs->{bindir}/hagrp -offline $grp -sys $sysi");
        if (!EDR::cmdexit()) {
            $sys->cmd("$vcs->{bindir}/hagrp -wait $grp State OFFLINE -sys $sysi -time 300");
        } else {
            Msg::log("Unable to offline $grp on $sysi");
            return 1;
        }
    }
    return 0;
}

#installer should stop odm before stopping VCS because odm register on gab,
#if odm isn't stopped, gab can't be stopped also
sub stop_odm_sys {
    my ($prod,$sys) = @_;
    my ($odmver,$odmpkg,$odm,$gmspkg,$vxgms,$gmsver);
    $odmpkg=$sys->pkg('VRTSodm60');
    if (defined $odmpkg) {
        $odm=$sys->proc('odm60');
        $odmver=$odmpkg->version_sys($sys);
        if ($odmver && ($odm->check_sys($sys,'stop'))) {
            $odm->stop_sys($sys);
            return 1 if ($odm->check_sys($sys,'stop'));
        }
    }
    $gmspkg=$sys->pkg('VRTSgms60');
    if (defined $gmspkg) {
        $vxgms=$sys->proc('vxgms60');
        $gmsver=$gmspkg->version_sys($sys);
        if ($gmsver && ($vxgms->check_sys($sys,'stop'))) {
            $vxgms->stop_sys($sys);
            return 1 if ($vxgms->check_sys($sys,'stop'));
        }
    }
    return 0;
}

package Prod::VM60::Common;
@Prod::VM60::Common::ISA = qw(Prod);

sub upgrade_precheck_sys {
    my ($prod,$sys)= @_;
    my ($pkg,$cprod);

    $prod->large_lun_check_sys($sys);

    #fix e1518138 : uninstall AT pkgs when upgrade VM/VVR
    $cprod=CPIC::get('prod');
    if (EDRu::inarr($cprod,qw/VM60 DMP60/)) {
        $prod->obsoleted_bundles_sys($sys);
        $prod->obsoleted_bundled_pkgs_sys($sys);
    }
    $prod->obsoleted_thirdparty_pkgs_sys($sys);
    # check if dg has latest backup before upgrade
    $prod->dg_conf_upgrade_precheck_sys($sys) unless (Cfg::opt('upgrade_nonkernelpkgs'));
    if ($prod->is_encapsulated_bootdisk_sys($sys)){
        return '' unless ($prod->encap_upgrade_precheck_sys($sys));
        # set donotstop and donotremove flag here.
        $prod->set_flag_encap_upgrade_sys($sys);
    }
    $prod->open_volumes_sys($sys, 'upgrade') unless (Cfg::opt(qw(upgrade_kernelpkgs upgrade_nonkernelpkgs)));
    $prod->vvr_upgrade_precheck_sys($sys) if ($prod->vr_licensed_sys($sys));
    return;
}

sub large_lun_check_sys {
    my ($prod,$sys)=(@_);
    my ($tmpdir,$ivers,$disks,@disks,@disks_with_issue,$mode,$pkg,$msg);
    my ($cnt,$devcnt,$lpath,$nmp,$tmpfile,$tbnum,$tmpfile2,$vxscsi);
    my ($cap,$cyl,$ncyl,$ncyl_lmt,$nhead,$nsect,$rsize,$tb_mkr);
    my ($alt_cyl,$cyl_msb,$cyl_lsb,$cylsize,$hd,$hd_msb,$hd_lsb,$sec,$sec_msb,$sec_lsb,$tcyl);
    my ($disk_s,$disk_s2,$disk_size1,$disk_size2,$vxdisk);
    my ($label,$maxcap,$secoff,$skip_value,$tdisk_size);

    # VM have large LUN disk issue on 5.1SP1, 5.1SP1RP1, 5.1SP1RP2, 5.1SP1RP3
    # For more information, visit tech note:
    #   http://www.symantec.com/business/support/index?page=content&id=TECH155921
    $ivers=$prod->version_sys($sys);
    if ($ivers !~ /5\.1\.1\d[0123]/) {
        return 1;
    }

    # Need check large LUN if 5.1SP1, 5.1SP1RP1, 5.1SP1RP2, 5.1SP1RP3
    Msg::log("Checking large LUN: VM version: $ivers");

    # check vxvm is running
    $mode=$sys->cmd("_cmd_vxdctl mode 2>/dev/null");
    if ($mode!~/mode: enabled/) {
        # vxconfigd not running, installer will not re-start 5.1SP1 vxconfigd to gather data
        $sys->set_value('donot_restart_vxconfigd', 1);
        return 0;
    }
 
    # Check if any disk size > 1T, if yes, do not re-start 5.1SP1 vxconfigd.
    my $sol_disklmt=2147483647;
 
    $disks=$sys->cmd("_cmd_vxdisk -q list 2>/dev/null | _cmd_grep ':cdsdisk' | _cmd_grep -v error | _cmd_awk '{print \$1}' | _cmd_grep -v 'was:'");
    return 0 unless $disks;
 
    @disks=split(/\s+/,$disks);
    @disks_with_issue=();
 
    $tmpdir=EDR::tmpdir();
    $nmp=$sys->cmd("_cmd_vxddladm get namingscheme 2>/dev/null | _cmd_tail -1 | _cmd_grep New | _cmd_wc -l");
 
    for my $disk (@disks) {
        my ($flag,$flags,@flags,$format,$invalid,$len,$off,$pub);

        Msg::log("Checking if there is 1TB issue for $disk");
        if ($nmp == 1) {
            $lpath="/dev/rdisk/$disk";
        } else {
            $lpath="/dev/vx/rdmp/$disk";
        }

        # Raw Geometry Mode Sense 3/4 Values
        $vxscsi=$sys->cmd("/etc/vx/diag.d/vxscsi -g $disk");
        next if (EDR::cmdexit() != 0);
 
        $nsect=0;
        $nhead=0;
        $ncyl=0;
        if ($vxscsi=~/nsect=(\d+)\s*/){
            $nsect=$1;
        }
        if ($vxscsi=~/nhead=(\d+)\s*/){
            $nhead=$1;
        }
        if ($vxscsi=~/ncyl=(\d+)\s*/){
            $ncyl=$1;
        }
        $cap=$nsect * $nhead * $ncyl;
        next if ($cap==0);

        $cyl=int($cap / $ncyl);
        $rsize=$cap - $cyl - $cyl;
        $ncyl_lmt=int($sol_disklmt / $cyl);
        $tb_mkr=$ncyl_lmt * $cyl;
 
        # Block 0 label values
        $tmpfile="$tmpdir/".$sys->{sys}."_$disk"."_blk0.out";
        $sys->cmd("_cmd_dd if=$lpath of=$tmpfile bs=512 count=2 >/dev/null 2>&1");
        if ($sys->exists($tmpfile)) {
            $sec_msb=$sys->cmd("od -An -v -j 438 -N 1 -tuC $tmpfile 2>/dev/null");
            $sec_lsb=$sys->cmd("od -An -v -j 439 -N 1 -tuC $tmpfile 2>/dev/null");
            $sec=$sec_msb * 256 + $sec_lsb;
            $hd_msb=$sys->cmd("od -An -v -j 436 -N 1 -tuC $tmpfile 2>/dev/null");
            $hd_lsb=$sys->cmd("od -An -v -j 437 -N 1 -tuC $tmpfile 2>/dev/null");
            $hd=$hd_msb * 256 + $hd_lsb;
            $cyl_msb=$sys->cmd("od -An -v -j 432 -N 1 -tuC $tmpfile 2>/dev/null");
            $cyl_lsb=$sys->cmd("od -An -v -j 433 -N 1 -tuC $tmpfile 2>/dev/null");
            $cyl=$cyl_msb * 256 + $cyl_lsb;
            $cylsize=$sec * $hd;
            $alt_cyl=$sys->cmd("od -An -v -j 435 -N 1 -tuC $tmpfile 2>/dev/null");
            $tcyl=$cyl + $alt_cyl;
 
            # Compare Raw Geo With Block 0
            if (($sec != $nsect ) || ( $hd != $nhead ) || ( $tcyl != $ncyl ) || ($cap > $tb_mkr)) { 
                $disk_size2=$cyl * $cylsize;
 
                # Check if disk is a cdsdisk and compare VXVM disk size with Block 0 size
                $vxdisk=$sys->cmd("_cmd_vxdisk list $disk 2>/dev/null");
 
                $format='';
                $invalid='';
                $flag='';
                if ($vxdisk=~/format=(\w+)/g) {
                    $format=$1;
                }
                if ($vxdisk=~/flags:.*invalid.*/g) {
                    $invalid=1;
                }
                if ($vxdisk=~/flags:(.+)/g) {
                    $flags=$1;
                    @flags=split(/\s+/m, $flags);
                    $flag=$flags[1];
                }
                if (( $format eq "cdsdisk" ) && ( $flag eq "online" ) && ( $invalid != 1 )) {
                    if ($vxdisk=~/public:(.+)/g) {
                        $pub=$1;
                    }
                    if ($pub=~/offset=(\d+)/) {
                        $off=$1;
                    }
                    if ($pub=~/len=(\d+)/) {
                        $len=$1;
                    }
                    $disk_s=$off + $len;
                    if ($sys->{plat} eq "HPUX") {
                        $disk_s2=$disk_s * 1024;
                        $disk_size1=int($disk_s2 / 512);
                    } else {
                        $disk_size1=$disk_s;
                    }
                } else {
                    $disk_size1=0;
                }
 
                if (( $disk_size1 > $disk_size2 ) || ( $disk_size1 == 0 ) || ( $disk_size1 > $rsize ) || ($cap > $tb_mkr)) {
                    # Ensure Block 0 is not empty and has a valid label
                    if (( $sec == 0 ) && ( $hd == 0 ) && ( $tcyl == 0 ) && ( $flag !~ "error" )) {
                        if ( $format eq "cdsdisk" ) {
                            $label=$sys->cmd("_cmd_strings $tmpfile");
                            if ( $label =~ /EFI PART/g ) {
                                Msg::log("Disk $disk Passed - Labeled EFI/GPT");
                            } else {
                                Msg::log("Disk $disk -- FAILED --");
                                Msg::log("Block 0 does not contain any SMI (disk label) data, Need run 'vxdisk resize $disk' to update Block 0\n");
                                push @disks_with_issue, $disk;
                            }
                        } else {
                            if ( $format eq "hpdisk" ) {
                                Msg::log("$disk format $format - Passed");
                            } else {
                                Msg::log("$disk is not under VxVM Control");
                            }
                        }
                    } else {
                        $tdisk_size=$tcyl * $sec * $hd;
                        $secoff=$sec - 1;
                        $skip_value=$tdisk_size - $secoff;

                        # Gather the dd from the backup label location base on Block 0 values
                        if ( $disk_size1 > $skip_value) {
                            Msg::log("Disk has Smaller Label than VxVM DMSIZE");
                            Msg::log("--------$disk Lun Size's ---------");
                            if ( $format eq "cdsdisk" ) {
                                Msg::log("VxVM DM(disk media) Size = $disk_size1");
                            } else {
                                Msg::log("$disk is not under VxVM Control");
                            }
                            
                            if ($vxscsi=~/cap=(\d+)/) {
                                $maxcap=$1;
                            }
                            Msg::log("Raw Geometry Size - Max Cap=$maxcap Available Size = $rsize");
                            Msg::log("Block 0 Label Size = $disk_size2");
                            Msg::log("    ---- CYL=$tcyl SEC=$sec HD=$hd ----");
 
                            Msg::log("Disk $disk -- FAILED --");

                            # Check is the backup label Block contain backup label data search for "cyl"
                            $tmpfile2="$tmpdir/".$sys->{sys}."_$disk"."_bklbl.out";
                            $sys->cmd("_cmd_dd if=$lpath of=$tmpfile2 bs=512 count=16 skip=$skip_value >/dev/null 2>&1");
                            $cnt=$sys->cmd("_cmd_strings $tmpfile2 | _cmd_grep cyl | _cmd_head -1 | _cmd_wc -l | _cmd_awk '{print \$1}'");
                            if  ( $cnt == 1 ) {
                                Msg::log("Script has detected that Disk $disk has Block 0 data in user data region.");
                                Msg::log("Block 0 data was located on device at block offset $skip_value");
                            } else {
                                Msg::log("No Block 0 label data was detected in user data blocks");
                            }
                            push @disks_with_issue, $disk;
                        } else {
                            if (( $flag eq "error" ) || ( $invalid == 1 )) {
                                Msg::log("Disk $disk ---- FAILED");
                                push @disks_with_issue, $disk;
                            } else {
                                if ( $cap > $tb_mkr ) {
                                    Msg::log("Disk $disk ---- FAILED");
                                    Msg::log("Lun $disk greater than 1TB with an SMI label\n");
                                    push @disks_with_issue, $disk;
                                } else {
                                    Msg::log("Size Discrepancy Noted 'Disk $disk Passed'");
                                }
                            }
                        }
                    }
                } else {
                    Msg::log("Disk $disk Passed");
                }
           } else {
               Msg::log("Disk $disk Passed ");
           }
        } else {
            Msg::log("Unable to run dd on Drive $disk");
        }
    }
 
    if ( @disks_with_issue ) {
        $sys->set_value('donot_restart_vxconfigd', 1);

        $disks=join ', ', @disks_with_issue;
        $msg=Msg::new("There are the following disks over terabyte on $sys->{sys}:\n\t$disks\nInstaller will not restart the vxconfigd due to a defect in Volume Manager of 5.1SP1 version that may potentially cause diskgroup import failures on upgrade and lead to potential data loss.\nPlease refer to the technical note for more details 'http://www.symantec.com/business/support/index?page=content&id=TECH155921'");
        $msg->log;
        return 0;
    } else {
        Msg::log("All disk Passed no action required");
        Msg::log("No User Data can get corrupted.");
    }
    return 1;
}


sub dmp_osn_stop_sys {
    my ($prod,$sys,$task) = @_;
    my ($msg,$out,$voldmode);
    my ($exitcode);

    return 0 if (Cfg::opt('rootpath'));
    return 0 if ($sys->exists("$prod->{mkdbfile}"));

    $voldmode = $prod->vold_status_sys($sys);
    if ($voldmode !~ /enabled/m) {
        if ($sys->{donot_restart_vxconfigd}) {
            return 0;
        } else {
            # If vold is down, attempt to start it
            $sys->cmd('_cmd_vxconfigd -k -r reset -x syslog 2> /dev/null');
            $voldmode = $prod->vold_status_sys($sys);
            if ($voldmode !~ /enabled/m ) {
                if (Cfg::opt('upgrade')) {
                    $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. You must make sure the tunable dmp_native_support is turned off on $sys->{sys} before upgrade. It is likely that vxdmp driver will fail to stop during upgrade if the tunable is on. If OS upgrade is involved during the upgrade process, you can ignore this warning.", 40, 4907, "$sys->{sys}", "$sys->{sys}");
                } else {
                    $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. You must affirm that dmp_native_support tunable is turned off or run 'vxdmpadm settune dmp_native_support=off' manually after restarting vxconfigd if you want to continue.", 40, 4496, "$sys->{sys}");
                }
                $sys->push_warning($msg);
                return 0;
            }
        }
    }

    $out = $sys->cmd('_cmd_vxdmpadm gettune dmp_native_support 2> /dev/null');
    if ($out =~ /dmp_native_support\s+on/mx) {
        Msg::log("Detected OS native stack support is enabled on $sys->{sys} before $task, turn it off...");
        $out = $sys->cmd('_cmd_vxdmpadm settune dmp_native_support=off');
        $exitcode = EDR::cmdexit();
        # e2851114: While making dmp_native_support off on Linux, it returns with value 215 (VE_REBOOT_REQUIRED 471). Should regard return value of 215 is successful on Linux.
        # check if $exitcode is -41 because in EDR $exitcode -= 256 if ($exitcode > 127);
        $exitcode = 0 if (($exitcode eq '-41') && $sys->linux());
        if ($exitcode) {
            if ($out =~ /\brootvg\b/m) {
                # error for AIX LVM
                $msg = Msg::new("Failed to turn off dmp_native_support tunable on $sys->{sys}. Refer to Dynamic Multi-Pathing Administrator's guide to determine the reason for the failure and take corrective action. Re-run bosboot manually.", 40, 4497, "$sys->{sys}");
            } else {
                # other error
                $msg = Msg::new("Failed to turn off dmp_native_support tunable on $sys->{sys}. Refer to Dynamic Multi-Pathing Administrator's guide to determine the reason for the failure and take corrective action.", 40, 4498, "$sys->{sys}");
            }
            $msg->{msg} .= "\n$out";
            $sys->push_error($msg);
        } else {
            if ($out =~ /\breboot\b/im) {
                if ($task eq 'upgrade') {
                    $msg = Msg::new("System $sys->{sys} needs reboot to disable dmp native support on $sys->{sys}. Re-run the upgrade task after reboot.", 40, 4499, "$sys->{sys}", "$sys->{sys}");
                } elsif ($task eq 'uninstall') {
                    $msg = Msg::new("System $sys->{sys} needs reboot to disable dmp native support on $sys->{sys}. Re-run the uninstall task after reboot.", 40, 4500, "$sys->{sys}", "$sys->{sys}");
                } elsif ($task eq 'stop') {
                    $msg = Msg::new("System $sys->{sys} needs reboot to disable dmp native support on $sys->{sys}. Re-run the stop task after reboot.", 40, 4501, "$sys->{sys}", "$sys->{sys}");
                } elsif ($task eq 'rollback') {
                    $msg = Msg::new("System $sys->{sys} needs reboot to disable dmp native support on $sys->{sys}. Re-run the rollback task after reboot.", 40, 4906, "$sys->{sys}", "$sys->{sys}");
                }
                $sys->push_warning($msg);
                $sys->set_value('reboot', 1);
                $sys->set_value('warning_exit', 1);
            }
        }
        # touch /etc/vx/.migrate_native to turn on dmp native support later
        $sys->cmd("_cmd_touch $prod->{migrate_native_file}") if (Cfg::opt(qw(upgrade patchupgrade stop)));
    }
    return 1;
}

sub vvr_upgrade_precheck_sys {
    my ($prod,$sys)= @_;
    my ($dg,@dgs,$msg,$rvg,$shared,$voldmode,$vvr);
    # check vxdctl mode
    # Follow upgrade procedure if VRTSvxvm is being upgraded
    # But just check whether the sys is being upgraded.
    return if ($sys->exists("$prod->{mkdbfile}"));
    $voldmode = $sys->cmd('_cmd_vxdctl mode 2> /dev/null');
    if ($voldmode !~ /mode: enabled/m ) {
        if ($sys->{donot_restart_vxconfigd}) {
            return '';
        } else {
            # If vold is down, atempt to start it
            $sys->cmd('_cmd_vxconfigd -k -r reset -x syslog 2> /dev/null');
            $voldmode = $sys->cmd('_cmd_vxdctl mode 2> /dev/null');
            if ($voldmode !~ /mode: enabled/m ) {
                $msg=Msg::new("vxconfigd could not be started. Upgrading VRTSvxvm in this scenario can result in configuration errors, or data loss for Veritas Volume Manager objects. Continue if you are sure that there are no Veritas Volume Manager objects on the host.", 40, 2652);
                $sys->push_warning($msg);
                return '';
            }
        }
    }

    # check dg version
    # Iterate twice
    $dg=$sys->cmd("_cmd_vxdg -q list 2>/dev/null | _cmd_awk '{ print \$1 }'");
    @dgs=split(/\s+/m,$dg);
    for my $dg (@dgs) {
        $rvg = $sys->cmd("_cmd_vxprint -g $dg -qQVF%name 2>/dev/null");
        if ($rvg) {
            $vvr = 1;
            $sys->set_value('rvg','1');
            last;
        }
    }
    return unless ($vvr);
    for my $dg (@dgs) {
        $shared = $sys->cmd("_cmd_vxdg list $dg 2> /dev/null | _cmd_grep 'flags:' | _cmd_grep shared");
        next if ($shared);
        $prod->vvr_check_dg_version_sys($sys, $dg);
    }
    return;
}

sub dg_conf_upgrade_precheck_sys {
    my ($prod,$sys) = @_;
    my ($cf_bkup_path,$cmd,$dg,$dgid,$dglist,$msg,$rootpath,$voldmode);
    return if ($sys->exists("$prod->{mkdbfile}"));
    return if ($prod->vol_is_vm_disabled_sys($sys));
    $voldmode = $prod->vold_status_sys($sys);
    if ($voldmode !~ /enabled/m ) {
        if ($sys->{donot_restart_vxconfigd}) {
            return '';
        } else {
            # If vold is down, atempt to start it
            # firstly try to start without '-r reset' since '-r reset' will increase dg seq no
            if ($voldmode =~ /not-running/m) {
                Msg::log("vxconfigd is not running on $sys->{sys}; restarting");
                $sys->cmd('_cmd_vxconfigd -k -x syslog 2> /dev/null');
            } else {
                # the mode could be disabled or booted
                Msg::log("vxconfigd is in $voldmode on $sys->{sys}; re-enabling");
                $sys->cmd('_cmd_vxdctl enable >/dev/null 2>&1');
            }
            $voldmode = $prod->vold_status_sys($sys);
            if ($voldmode !~ /enabled/m ) { 
                $sys->cmd('_cmd_vxconfigd -k -r reset -x syslog 2> /dev/null');
                $voldmode = $prod->vold_status_sys($sys);
                if ($voldmode !~ /enabled/m ) {
                    $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. Upgrading VRTSvxvm in this scenario can result in configuration errors, or data loss for Veritas Volume Manager objects. Continue if you are sure that there are no Veritas Volume Manager objects on the host.", 40, 5068, "$sys->{sys}");
                    $sys->push_warning($msg);
                    return '';
                }
            }
        }
    }
    $rootpath = Cfg::opt("rootpath");
    $cf_bkup_path = "$rootpath$prod->{cf_bkup_path}";
    $dglist = $sys->cmd("_cmd_vxdg -q list 2> /dev/null");
    for my $line(split(/\n/,$dglist)) {
        if ($line =~ /(\S+)\s+\S+\s+(\S+)/m) {
            $dg = $1;
            $dgid = $2;
        }
        if ($prod->is_dg_seqno_change_sys($sys,$dg,$dgid)) {
            $sys->set_value('dgs_need_backup','push',$dg);
        }

        # For 2808458: to check if any imported dg with mix of regular and clone disks
        if ($prod->is_dg_mix_clone_sys($sys,$dg)) {
            $sys->set_value('dgs_mix_clone','push',$dg);
        }
    }
    if ($sys->{dgs_need_backup}) {
        $dglist = join(" ", @{$sys->{dgs_need_backup}});
        $cmd = $prod->padv->{cmd}{vxconfigbackup};
        $msg = Msg::new("The following disk group(s) do not have the latest backup of configuration files in $cf_bkup_path on $sys->{sys} before upgrade:\n\t$dglist\nRun the command '$cmd -l [dir] [dgname|dgid]' to take the latest backup of them. The default backup directory $cf_bkup_path on $sys->{sys} may be removed during upgrade. Manually copy the directory to other places if you would like to keep the backup files.", 40, 5069, "$cf_bkup_path", "$sys->{sys}", "$dglist", "$cmd", "$cf_bkup_path", "$sys->{sys}");
        $sys->push_warning($msg);
    } elsif ($dglist) {
        $msg = Msg::new("The default backup directory $cf_bkup_path on $sys->{sys} may be removed during upgrade. Manually copy the directory to other places if you would like to keep the backup of the configration files for the disk groups before upgrade.", 40, 5070, "$cf_bkup_path", "$sys->{sys}");
        $sys->push_note($msg);
    }

    if ($sys->{dgs_mix_clone}) {
        $dglist = join(" ", @{$sys->{dgs_mix_clone}});
        $msg = Msg::new("The following disk group(s) have mix of regular and clone disk on $sys->{sys}:\n\t$dglist\nCheck the disk information and unset clone flag before upgrade.", 40, 5453, "$sys->{sys}", "$dglist");
        $sys->push_error($msg);
    }
    return;    
}

package Rel::UXRT601::Linux;

sub platvers_sys {
    my ($rel,$sys) = @_;
    my ($patchlevel,$msg,$rpm_dir,$kret,$ksles,$padv,$kstring,$cpic,@f,$distro,@pkg_files,$pkg,$prod);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return '' if ($sys->{stop_checks});
    return '' if (Cfg::opt('ignorechecks'));

    $cpic=Obj::cpic();
    $prod=$sys->prod($cpic->{prod});
    $padv=$sys->padv($sys->{padv});
    $distro=$padv->distro_sys($sys);
    $kstring=$sys->cmd('_cmd_uname -r');
    $kstring =~ /\-(\d+)/m;
    $kret = $1;
    $ksles=$1 if ($kstring =~ /\-\d+\.(\d+)/mx);
    @f=split(/\W/m,$kstring);

    if ($distro eq 'RHEL') {
        $padv->{cmd}{selinuxenabled}='/usr/sbin/selinuxenabled';
        if ($kstring =~ /el6/m) {
            if ($kret < 131) { # RHEL6u1 check
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.32-131.el6(RHEL6 U1) or higher is supported on RHEL6. Upgrade the OS to RHEL6 Update1 in order to install this product", 40, 4886, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }

            # 6.0.1 only support up to RHEL6U3.
            # RHEL6U4 kernel: 2.6.32-358.el6.x86_64
            # RHEL6U5 kernel: 2.6.32-431.el6.x86_64
            # RHEL6U6 kernel: 2.6.32-504.el6.x86_64
            if ($kret > 279) {
                $patches="SFHA 6.0.5 release for RHEL6: https://sort.symantec.com/patch/detail/8508";
                if ($kret == 504) {
                    $patches.="\n\tSFHA 6.0.5 patch for RHEL6 U6: https://sort.symantec.com/patch/detail/9326";
                }
                $msg=Msg::new("System $sys->{sys} is running Kernel Release $kstring. $rel->{reltitle} $rel->{titlevers} is not supported on Kernel Release $kstring without additional patches. Visit Symantec SORT web site to download the following required patches and follow the patch instructions to install the patches:\n\n\t$patches");
                $sys->push_error($msg);
            }
        } elsif ($kstring =~ /el5/m) {
            if ($kret < 194) { # RHEL5u5 or above check
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.18-194.el5(RHEL5 U5) or higher is supported on RHEL5. Upgrade the OS to RHEL5 Update5 or higher in order to install this product", 40, 4887, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
        }

    } elsif ($distro eq 'SLES') {
        $padv->{cmd}{selinuxenabled}='/usr/bin/selinuxenabld';

        #SLES 10 SP4 check
        #In 6.0, SLES 10 SP4 is supported,
        #SP3 kernel version is above 2.6.16.60-0.54.5
        #SP4 kernel version is above 2.6.16.60-0.85.1
        #For PATCHLEVEL is higher than SuSE 10 SP4 and SuSE11 SP2, maybe does not support

        $patchlevel=$sys->{patchlevel};

        if ($f[0]>2) {
            # sles11sp2 is using kernel 3.x
            if ((defined($patchlevel)) && ($patchlevel>2)){
                $msg=Msg::new("SuSE 11 SP1 and SP2 are the recommended platforms for the release on $sys->{sys}.", 40, 4963, "$sys->{sys}");
                $sys->push_warning($msg);
                return '';
            }
        } else {
            if (($f[2]==16) && ($f[3] < 60)) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.16.60-0.85(SLES10 SP4) or higher is supported on SuSE10. Upgrade the OS to SLES10 SP4 in order to install this product", 40, 4888, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
            if (($f[2]==16) && ($f[3]==60) && ($f[5] < 85)) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.16.60-0.85(SLES10 SP4) or higher is supported on SuSE10. Upgrade the OS to SLES10 SP4 in order to install this product", 40, 4888, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }

            #SLES 11 SP1 or above check
            if (($f[2]==27) || (($f[2]==32) && ($f[3] < 12))) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.32.12(SLES11 SP1) or higher is supported on SuSE11. Upgrade the OS to SLES11 SP1 in order to install this product", 40, 4889, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
            if (($f[2]==32) && (defined($patchlevel)) && ($patchlevel>2)){
                $msg=Msg::new("SuSE 11 SP1 and SP2 are the recommended platforms for the release on $sys->{sys}.", 40, 4963, "$sys->{sys}");
                $sys->push_warning($msg);
                return '';
            }
        }
    } elsif ($distro eq 'ESX') {
        # FIX: Figure this check out later for VMWARE. (Get versions of ESX supported and edit)
        if ($kret < 34) {
            $msg=Msg::new("Kernel Release $kstring is not supported on $sys->{sys}. Only Kernel Release 2.6.9-34.EL or higher is supported on RHEL.", 40, 3992, "$kstring", "$sys->{sys}");
            $sys->push_error($msg);
            return '';
        }
    } elsif ($distro eq 'OL') {
        # FIX: Figure this check out later for Oracle Linux.
        $padv->{cmd}{selinuxenabled}='/usr/sbin/selinuxenabled';
        if ($kstring =~ /el6/m) {
            # OL6 check
            if ($kstring =~ /uek/m || $kstring !~ /^2\.6\.32/m) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Red Hat compatible Kernel Release 2.6.32 is supported on Oracle Linux 6. you may configure the operating system to boot with this kernel instead.", 40, 4784, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
        } elsif ($kstring =~ /el5/m) {
            # OL5 check
            if ($kstring =~ /uek/m || $kstring !~ /^2\.6\.18/m) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Red Hat compatible Kernel Release 2.6.18 is supported on Oracle Linux 5. you may configure the operating system to boot with this kernel instead.", 40, 4785, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
        }
    } else {
        $msg=Msg::new("Distribution is not recognized as SuSE or Red Hat on $sys->{sys}", 40, 3993, "$sys->{sys}");
        $sys->push_error($msg);
        return '';
    }

    # XEN is not supported in 6.0
    if ($kstring =~ /xen/m) {
        $msg=Msg::new("Kernel Release on $sys->{sys} is $kstring. Xen kernel is not supported in this release.", 40, 4890, "$sys->{sys}", "$kstring");
        $sys->push_error($msg);
        return '';
    }

    # Distribution match on all systems removed as any to many support is added
    for my $pkgi (@{$rel->allpkgs}) {
        $pkg=$sys->pkg($pkgi);
        push(@pkg_files,$pkg->{file});
    }

    if (@pkg_files) {
       return 1 if ($cpic->{fromdisk});
       $rpm_dir=$rel->pkgs_patches_dir('pkgpath');;
       if ((($sys->{arch}=~/686/m)||($sys->{arch}=~/586/m)) && (grep {/86_64/m} @pkg_files)) {
            $msg=Msg::new("System $sys->{sys} is architecture $sys->{arch} and x86_64 rpms were found in $rpm_dir. Check that you are installing from the correct DVD. Some rpms will not install correctly on this system.", 40, 3626, "$sys->{sys}", "$sys->{arch}", "$rpm_dir");
            $sys->push_error($msg);
            return '';
        }

       if ((($distro eq 'RHEL') && (grep {/SLES/} @pkg_files))
            ||  (($distro eq 'SLES') && (grep {/RHEL/} @pkg_files))) {
            $msg=Msg::new("System $sys->{sys} is distribution $distro and incompatible rpms were found in $rpm_dir. Check that you are installing from the correct DVD. Some rpms will not install correctly on this system.", 40, 3627, "$sys->{sys}", "$distro", "$rpm_dir");
            $sys->push_error($msg);
            return '';
       }
    }

    # No need for selinux checks any more
    return 1;
}

1;