#HOTFIX_VERSION:cpi-6.0.3.900
package Padv::Linux;

sub detect_linux_by_dmidecode_sys {
    my ($padv, $sys) = @_;

    my $out = $sys->cmd("_cmd_dmidecode 2>/dev/null");

    my ($flag, $bios_vendor, $mfgr, $product) = ("", "", "", "");
    foreach (split(/\n+/, $out)) {
        if (/^\S/) {
            if (/^BIOS Information/) {
                $flag = "bios_info";
            } elsif (/^System Information/) {
                $flag = "system_info";
            } else {
                $flag = "other";
            }
            next;
        }

        if ($flag eq "bios_info") {
            $bios_vendor = $1 if (/Vendor:\s*(.*)/i);
        }
        if ($flag eq "system_info") {
            $mfgr = $1 if (/Manufacturer:\s*(.*)/i);
            $product = $1 if (/Product Name:\s*(.*)/i);
        }
    }

    $sys->{virtual_type} = $padv->VIRT_VMWARE if ($mfgr =~ /VMWare/i);
    $sys->{virtual_type} = $padv->VIRT_KVM . " or " . $padv->VIRT_QEMU if ($bios_vendor =~ /QEMU/i);
    $sys->{virtual_type} = $padv->VIRT_VIRTUALPC if ($mfgr =~ /microsoft/i && $product =~ /virtual machine/i);
    return;
}

package Prod::VM60::Common;

sub encap_upgrade_precheck_sys {
    my ($prod,$sys)=@_;
    my ($msg,$rel,$vxvm,$vers,$support_vers,$bootdg,$minor_num,$plex_num,$mode,$dg_blk,$dg_raw,$vol_raw,$vol_blk,$vol,$volumes,$minor,$major,$output);
    return 0 if (Cfg::opt('upgrade_nonkernelpkgs')); #For rolling upgrade phase2, doesn't check encapsulated bootdisk
    return 0 if (Cfg::opt('rootpath'));
    # Step 1. check for encapsulate bootdisk upgrade path support
    #$vxvm=$sys->pkg($prod->{mainpkg});
    $rel=$sys->rel();
    $vers=$prod->version_sys($sys);
    $support_vers = ($sys->linux()) ? '5.1' :
                    ($sys->sunos()) ? '4.1' : '0';
    # e2832803: need to unencapsulate first before upgrade to 6.0.1 on Redhat6 
    if ((EDRu::compvers($vers,$support_vers,2)==2) || ($sys->{padv} =~ /RHEL6/mx)) {
        $msg=Msg::new("Cannot proceed with the upgrade because the system $sys->{sys} may have an encapsulated bootdisk. Run $prod->{vxunroot} to unencapsulate the bootdisk.");
        $sys->push_error($msg);
        return 0;
    }

    # Step 2. check for vxconfigd running
    $mode=$sys->cmd('_cmd_vxdctl mode 2>/dev/null');
    if ($mode!~/mode: enabled/m) {
        $msg=Msg::new("vxconfigd must be running to proceed with encapsualted bootdisk upgrade");
        $sys->push_error($msg);
        return 0;
    }

    # Step 3. check for mirror state
    $plex_num=$sys->cmd("_cmd_vxprint -ptg bootdg | _cmd_grep 'pl .* rootvol' | _cmd_wc -l 2>/dev/null");
    if ($plex_num >1) {
        # vxrootadm integration is supported from 5.1SP1 onwards.
        # No vxrootadm support on HPUX. Supported platfroms: Solaris sparc/opteron and Linux
        # e2303520: solaris opteron, upgrade path is from 5.1SP1RP1 onwards to 6.0. Upgrade from 5.1SP1 is not supported.
        if (EDRu::compvers($vers,'5.1.100.0',4) < 2 && EDRu::compvers($vers,'5.1.101.0',4) == 2 && $sys->{arch}=~/i386/m ) {
            $msg=Msg::new("System $sys->{sys} has a boot disk that is mirrored and encapsulated. If you want to split the mirror during upgrade process, upgrade from current version $vers to 5.1SP1RP1 first. Then you can split the mirror during upgrade from 5.1SP1RP1 to $rel->{gavers}.");
            $sys->push_warning($msg);
        } elsif (EDRu::compvers($vers,'5.1.100.0',4) < 2 && !($sys->hpux())) {
            $sys->{mirror}=1;
            $sys->set_value('mirror', 1);
            $msg=Msg::new("System $sys->{sys} has a boot disk that is mirrored and encapsulated. It is highly recommended to split the mirrored disk group to get a backup copy of your OS environment.");
            $sys->push_note($msg);
        } else {
            $msg=Msg::new("System $sys->{sys} has a boot disk that is mirrored and encapsulated. If you do not want to upgrade the mirrored disk, refer to the Volume Manager admininstration guide on how to split the mirrored disk.");
            $sys->push_warning($msg);
            Msg::log("System $sys->{sys} has a boot disk that is mirrored and encapsulated. But the current version of vxrootadm is $vers which means it can not split the bootdg mirrors automatically during upgrade.");
        }
    }

    # Step 4. Sun specific check for reminor issue
    # refer to e1765860
    if ($sys->sunos()) {
        $bootdg=$sys->cmd('_cmd_vxdg bootdg 2>/dev/null');
        if (EDR::cmdexit()!=0) {
            $msg=Msg::new("Failed to find bootdg on system $sys->{sys}");
            $sys->push_error($msg);
            return 0;
        }
        $dg_blk="/dev/vx/dsk/$bootdg";
        $dg_raw="/dev/vx/rdsk/$bootdg";
        $minor_num=$sys->cmd("_cmd_vxprint -g $bootdg -F%base_minor $bootdg 2>/dev/null");
        # e2740143: reminor to 1000 unconditionally to keep consistent of VxVM preinstall script
        if ($minor_num != 1000) {
            $sys->cmd("_cmd_vxdg -f -g $bootdg reminor 1000 2>/dev/null");
            if (EDR::cmdexit()!=0) {
                $msg=Msg::new("Failed to reminor the bootdg on system $sys->{sys}");
                $sys->push_error($msg);
                return 0;
            }
        }

        # get the device major number
        $output=$sys->cmd("_cmd_ls -l $dg_blk/rootvol 2>/dev/null");
        if ($output=~/\s+(\d+),/m) {
            $major=$1;
        } else {
            $msg=Msg::new("Failed to get the bootdg device major number on system $sys->{sys}");
            $sys->push_error($msg);
            return 0;
        }

        $volumes=$sys->cmd("_cmd_ls $dg_blk");
        # e2740143: save bootdg info for later use in upgrade_postinstall_sys
        $sys->set_value('bootdg,dgname', $bootdg);
        $sys->set_value('bootdg,major',$major);
        for my $vol (split(/\s+/m,$volumes)) {
            next if ($vol eq 'rootvol');
            $vol_blk="$dg_blk/$vol";
            $vol_raw="$dg_raw/$vol";
            # get the new base minor number
            $minor=$sys->cmd("_cmd_vxprint -g $bootdg -F%minor -v $vol 2>/dev/null");
            # e3330485: try to get bootdg minor number for RVG if above cmd fails
            $minor=$sys->cmd("_cmd_vxprint -g $bootdg -F%minor -V $vol 2>/dev/null") unless (EDR::cmdexit()==0);
            if (EDR::cmdexit()==0) {
                # e2740143: save bootdg minor number since reminor is needed after upgrade
                $sys->set_value("bootdg,$vol,minor",$minor);
                # modify bootdg [r]dsk links with the new minor
                $sys->cmd("_cmd_rm -f $vol_blk 2>/dev/null");
                $sys->cmd("_cmd_rm -f $vol_raw 2>/dev/null");
                $sys->cmd("_cmd_mknod $vol_blk b $major $minor 2>/dev/null");
                $sys->cmd("_cmd_mknod $vol_raw c $major $minor 2>/dev/null");
            } else {
                $msg=Msg::new("Failed to get the bootdg device minor number on system $sys->{sys}");
                $sys->push_error($msg);
                return 0;
            }
        }
    }

    return 1;
}

sub enable_dmp_osn_sys {
    my ($prod,$sys) = @_;
    my ($cprod,$msg,$osn_enabled,$out,$voldmode);
    $cprod = CPIC::get('prod');
    if ($sys->{disable_dmp_native_support} == 1) {
        $msg=Msg::new("dmp_native_support is not enabled on $sys->{sys}. You need to run 'vxdmpadm settune dmp_native_support=on' manually to enable dmp_native_support in the future");
        $sys->push_warning($msg);
        return 1;
    }
    $osn_enabled = 1 if $sys->exists($prod->{migrate_native_file});
    if (($cprod =~ /DMP/m) || $osn_enabled) {
        if (Cfg::opt(qw(upgrade patchupgrade))) {
            if (!$osn_enabled) {
                Msg::log("dmp_native_support is not enabled before upgrade, do not turn it on after upgrade");
                return 1;
            }
        }
        $voldmode = $prod->vold_status_sys($sys);
        if ($voldmode !~ /enabled/m) {
            # If vold is down, attempt to start it
            $sys->cmd('_cmd_vxconfigd -k -r reset -x syslog 2> /dev/null');
            $voldmode = $prod->vold_status_sys($sys);
            if ($voldmode !~ /enabled/m ) {
                $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. You need to run 'vxdmpadm settune dmp_native_support=on' manually after restarting vxconfigd to enable dmp native support if it is not enabled yet.");
                $sys->push_warning($msg);
                return 0;
            }
        }
        $out = $sys->cmd('_cmd_vxdmpadm gettune dmp_native_support 2> /dev/null');
        if ($out =~ /dmp_native_support\s+on/mx) {
            Msg::log("Tunnable dmp_native_support is already turned on on $sys->{sys} ...");
            return 1;
        }
        Msg::log('Enabling OS native stack support ...');
        $out = $sys->cmd('_cmd_vxdmpadm settune dmp_native_support=on');
        if (EDR::cmdexit()) {
            if ($out =~ /\brootvg\b/m) {
                # error for AIX LVM
                $msg = Msg::new("Failed to enable root support on $sys->{sys}. Refer to Dynamic Multi-Pathing Administrator's guide to determine the reason for the failure and take corrective action. Re-run bosboot manually.");
            } else {
                # other error
                $msg = Msg::new("Failed to turn on dmp_native_support tunable on $sys->{sys}. Refer to Dynamic Multi-Pathing Administrator's guide to determine the reason for the failure and take corrective action.");
            }
            $msg->{msg} .= "\n$out";
            $sys->push_warning($msg);
        } elsif ($out =~ /\breboot\b/im) {
            $msg = Msg::new("System $sys->{sys} needs reboot to enable dmp native support on $sys->{sys}");
            $sys->push_note($msg);
            $sys->set_value('reboot', 1);
        } else {
            $msg = Msg::new("Tunable dmp_native_support is ENABLED on $sys->{sys}");
            $sys->push_note($msg);
        }
        $sys->cmd("_cmd_rmr $prod->{migrate_native_file}");
    }
    return 1;
}

sub ask_patchupgrade_configure {
    my ($prod) = @_;
    my ($msg,$syslist,$sys,$ayn);
    $syslist=CPIC::get('systems');

    # Ask user if he would like to enable_dmp_native_support after start.
    for my $sys (@$syslist) {
        if (Cfg::opt(qw(upgrade patchupgrade)) &&  # for upgrade
            $sys->{platvers} eq '5.10' &&
            $sys->exists($prod->{migrate_native_file})) { # previously dmp_native_support is enabled
            $msg=Msg::new("dmp_native_support is enabled before upgrade, do you want to turn it on after upgrade?");
            $ayn=$msg->aynn();
            if ($ayn eq 'N') {
                $sys->cmd("_cmd_rmr $prod->{migrate_native_file}");
                $sys->{disable_dmp_native_support}=1;
            }
        }
    }
}
package Prod;

sub ask_patchupgrade_configure {
    my ($prod,$sys)=@_;
    return $prod->perform_task('ask_patchupgrade_configure');
}

package CPIC;

sub ask_patchupgrade_configure {
    my ($cpic,$prod) = @_;
    my ($ayn,$cfg,$edr,$help,$msg);
    $edr=Obj::edr();
    $cfg=Obj::cfg();

    $prod->ask_patchupgrade_configure();
    if ($prod->check_config) {
        $help = Msg::new("If you are upgrading $prod->{abbr} and want to use the previous copy of the $prod->{abbr} configuration, enter 'y'. If you desire to do new configuration of $prod->{abbr}, enter 'n'.");
        $msg = Msg::new("Do you want to restore and reuse the previous $prod->{abbr} configuration on all systems");
        $ayn = $msg->ayny($help,0);
        if ($ayn eq 'Y') {
            $cpic->{reuse_config}=1;
            $cfg->{reuse_config}=1;
            return '';
        }
    }
    $cpic->completion() unless $cpic->ask_configure();
    return 1;
}

package Rel::UXRT601::SunOS;

sub init_plat {
    my ($rel) = @_;

    $rel->{driververs}='6.0.1';

    $rel->{prods}=EDRu::arrdel($rel->{prods}, 'SFCFSRAC60');
    $rel->{menuprods}=EDRu::arrdel($rel->{menuprods}, 'VCS60') if ($rel->{oem});
    $rel->{upgradeprods} = [ qw(SFRAC60 SVS60 SFCFSHA60 SFHA60 SF60 VCS60 VM60 DMP60 FS60) ];

    $rel->{obsoleted_but_still_support_pkgs} = [qw(VRTSat50)];
    $rel->{'commands_need_be_checked'}=[ qw(adddrv fstyp getconf hostid isainfo kstat ldd mdb ndd netstat nm nohup pkgadd pkgchk pkginfo pkgrm prtconf prtdiag rmdrv route swap) ];
    $rel->{'commands_need_be_checked_for_5.9'}=[ qw(patchadd patchrm showrev) ];
    $rel->{'commands_need_be_checked_for_5.10'}=[ qw(bootadm patchadd patchrm showrev zoneadm) ];
    $rel->{'commands_need_be_checked_for_patchupgrade'}=[ qw(gunzip) ];
    $rel->{'commands_need_be_checked_for_patchupgrade_5.10'}=[ qw(patchadd) ];

    return;
}

package EDR;

sub rsh_sys {
    my ($edr,$sys) = @_;
    my ($msg,$echo,$rsh_deny,$test,$ssh_deny,$rsh_failure,$mkdir,$ssh_failure);
    return '' if ($sys->{stop_checks});
    $sys->{rsh}=Cfg::opt('rsh') ? 'rsh' : 'ssh';
    $sys->{rcp}=Cfg::opt('rsh') ? 'rcp' : 'scp';
    $ssh_failure=$rsh_failure='';
    $sys->set_value('com_deny', 0);
    $sys->set_value('sshrsh_failure', 0);
    # When connecting to a host using ssh for the first time and the host name
    # doesn't exist in $HOME/.ssh/known_hosts, ssh will print an extra line
    # of warning message and add the host name
    # After the host name is added, this warning message will not appear again
    # Run $sys->cmd("echo") twice to ensure banner doesn't include extra words
    #to avoid ssh prompts for password when ssh is not configured
    unless (Cfg::opt('rsh')) {
        $test=$sys->cmd('echo');
        # chomp the last return character caused by "echo" command
        chomp ($test);
        if ($edr->{cmdexit}) {
            #print out the message when ssh is not configured.
            if ($test=~/Permission denied/m) {
                $ssh_failure .= Msg::new("ssh permission was denied on $sys->{sys}\n")->{msg};
                $ssh_deny = 1;
            } else {
                $ssh_failure .= Msg::new("ssh exited $edr->{cmdexit} on $sys->{sys}\n")->{msg};
            }
            Msg::log($ssh_failure);
            $sys->{rsh}='rsh';
            $sys->{rcp}='rcp';
        } else {
            $sys->{banner}=$sys->cmd('echo');
            chomp ($sys->{banner});
            # chomp the last return character caused by "echo" command
            $sys->set_value('banner', $sys->{banner});
        }
    }
    $sys->set_value('rsh', $sys->{rsh});
    $sys->set_value('rcp', $sys->{rcp});
    if ($sys->{rsh} eq 'rsh') {
        $test=$sys->cmd('echo');
        # chomp the last return character caused by "echo" command
        chomp ($test);
        Msg::log("Using $sys->{rsh} for communication with $sys->{sys}");
        if ($edr->{cmdexit}) {
            if ($test=~/permission denied|permission is denied|Host name for your address unknown|login incorrect/mi) {
                $rsh_failure .= Msg::new("rsh permission was denied on $sys->{sys}\n")->{msg};
                $rsh_deny = 1;
            } else {
                $rsh_failure .= Msg::new("rsh exited $edr->{cmdexit} on $sys->{sys}\n")->{msg};
            }
        }  else {
            $sys->{banner}=$sys->cmd('echo');
            chomp ($sys->{banner});
            $sys->set_value('banner', $sys->{banner});
        }
    }

    if ($rsh_failure) {
        if (Cfg::opt('rsh')) {
            $msg=Msg::new("${rsh_failure}rsh is needed to be setup between the local node and $sys->{sys} for communication");
        } else {
            $msg=Msg::new("${ssh_failure}${rsh_failure}either ssh or rsh is needed to be setup between the local node and $sys->{sys} for communication");
        }
        if ($rsh_deny || $ssh_deny) {
            $sys->set_value('com_deny', 1);
        } else {
            $sys->set_value('sshrsh_failure', 1);
        }
        $sys->push_stop_checks_error($msg);
        return '';
    }
    return 1;
}

package Padv::SunOS;

sub get_updatelevel_by_patchid {
    my ($arch, $osver, $patchid) = @_;
 
    # Get the following kernel patchid and update level matrix from:
    #   http://blogs.oracle.com/patch/entry/solaris_10_kernel_patchid_progression
   
    my $patch_matrix = << 'EOF';
# arch    os_version   update  patchid                 description
# ---------------------------------------------------------------------------------------------------
sparc,    5.10,        11,     147147-26,              Solaris 10 Update 11 Kernel PatchID
sparc,    5.10,        10,     144500-[0-9][0-9],      Solaris 10 Update 10 Kernel PatchID
sparc,    5.10,        9,      142909-17,              Solaris 10 9/10 (Update 9) Kernel PatchID
sparc,    5.10,        8,      141444-09,              Solaris 10 10/09 (Update 8) Kernel PatchID
sparc,    5.10,        7,      139555-08,              Solaris 10 5/09 (Update 7) Kernel PatchID
sparc,    5.10,        6,      137137-09,              Solaris 10 10/08 (Update 6) Kernel PatchID
sparc,    5.10,        5,      127127-11,              Solaris 10 5/08 (Update 5) Kernel PatchID
sparc,    5.10,        4,      120011-14,              Solaris 10 8/07 (Update 4) Kernel PatchID
sparc,    5.10,        3,      118833-33,              Solaris 10 11/06 (Update 3)
sparc,    5.10,        2,      118833-17,              Solaris 10 6/06 (Update 2)
sparc,    5.10,        1,      118822-25,              Solaris 10 1/06 (Update 1)

sparc,    5.10,        11,     148888-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 1/13 (Update 11)
sparc,    5.10,        10,     147440-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 8/11 (Update 10)
sparc,    5.10,        9,      144488-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 9/10 (Update 9)
sparc,    5.10,        8,      142900-(0[1-9]|1[0-5]), Kernel Bug Fixes post Solaris 10 10/09 (Update 8)
sparc,    5.10,        7,      141414-(0[1-9]|10),     Kernel Bug Fixes post Solaris 10 5/09 (Update 7)
sparc,    5.10,        6,      138888-0[1-8],          Kernel Bug Fixes post Solaris 10 10/08 (Update 6)
sparc,    5.10,        5,      137111-0[1-8],          Kernel Bug Fixes post Solaris 10 5/08 (Update 5)
sparc,    5.10,        4,      127111-(0[1-9]|1[0-1]), Kernel Bug Fixes post Solaris 10 8/07 (Update 4)
sparc,    5.10,        3,      125100-(0[4-9]|10),     Kernel Bug Fixes post Solaris 10 8/07 (Update 4)

i386,     5.10,        11,     147148-26,              Solaris 10 Update 11 Kernel PatchID
i386,     5.10,        10,     144501-[0-9][0-9],      Solaris 10 Update 10 Kernel PatchID
i386,     5.10,        9,      142910-17,              Solaris 10 9/10 (Update 9) Kernel PatchID
i386,     5.10,        8,      141445-09,              Solaris 10 10/09 (Update 8) Kernel PatchID
i386,     5.10,        7,      139556-08,              Solaris 10 5/09 (Update 7) Kernel PatchID
i386,     5.10,        6,      137138-09,              Solaris 10 10/08 (Update 6) Kernel PatchID
i386,     5.10,        5,      127128-11,              Solaris 10 5/08 (Update 5) Kernel PatchID
i386,     5.10,        4,      120012-14,              Solaris 10 8/07 (Update 4) Kernel PatchID
i386,     5.10,        3,      118855-33,              Solaris 10 11/06 (Update 3)
i386,     5.10,        2,      118855-14,              Solaris 10 6/06 (Update 2)
i386,     5.10,        1,      118844-26,              Solaris 10 1/06 (Update 1)

i386,     5.10,        11,     148889-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 1/13 (Update 11)
i386,     5.10,        10,     147441-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 8/11 (Update 10)
i386,     5.10,        9,      144489-[0-9][0-9],      Kernel Bug Fixes post Solaris 10 9/10 (Update 9)
i386,     5.10,        8,      142901-(0[1-9]|1[0-5]), Kernel Bug Fixes post Solaris 10 10/09 (Update 8)
i386,     5.10,        7,      141415-(0[1-9]|10),     Kernel Bug Fixes post Solaris 10 5/09 (Update 7)
i386,     5.10,        6,      138889-0[1-8],          Kernel Bug Fixes post Solaris 10 10/08 (Update 6)
i386,     5.10,        5,      137112-0[1-8],          Kernel Bug Fixes post Solaris 10 5/08 (Update 5)
i386,     5.10,        4,      127112-(0[1-9]|1[0-1]), Kernel Bug Fixes post Solaris 10 8/07 (Update 4)
i386,     5.10,        3,      125101-(0[1-9]|10),     Kernel Bug Fixes post Solaris 10 8/07 (Update 4)
 
EOF
 
    foreach my $line (split(/\n+/, $patch_matrix)) {
        next if ($line=~/^#/);
        my @fields = split(/,\s*/, $line);
        next if (@fields < 5);
 
        for my $i (0 .. $#fields) {
            $fields[$i] =~ s/^\s*//;
            $fields[$i] =~ s/\s*$//;
        }
 
        if ($arch eq $fields[0] && $osver eq $fields[1]) {
            if ($patchid =~ /^$fields[3]$/) {
                return $fields[2];
            }
        }
    }
    return "";
}

1;