#HOTFIX_VERSION:cpi-6.0.5.300
package Prod::VCS60::Common;

sub maincf_upgrade_precheck_sys {
    my ($prod,$sys) = @_;
    my ($rootpath,$msg,$unfreeze_sg,$grp,@grp_syslist,$cmd_hacf,@grps,$delete_cmc,$treeref_olduser,@extra_types,$attrs,$vxss_autostartlist,$upgradepath,$sfua_delete_SfuaBase,$line,$rtn,$delete_VCSweb,$grp_syslists,$vxss_add_autostartlist);

    $rootpath = Cfg::opt('rootpath') || '';
    $upgradepath= EDR::tmpdir().'/upgrade';
    $cmd_hacf = $prod->{cmd_hacf};

    # Use hacf on ABE only if the OS on PBE is same with ABE.
    if ($rootpath) {
        my @uname = split(/\s+/m,$sys->{uname});
        if ($sys->{platvers} eq $uname[2]) {
            $cmd_hacf= "$rootpath$prod->{bindir}/hacf";
            $prod->set_value('cmd_hacf',$cmd_hacf);
        } else {
            $sys->set_value('osupgraded',1);
        }
    }

    if ($sys->system1) {

        # dump vcs configuraton
        if (!Cfg::opt('rootpath') && !$prod->{upgrade_ignore_conf} && $sys->{prod_running}) {
            $prod->haconf_makerw();
            $prod->haconf_dumpmakero();
        }

        unless ($prod->dynupgrade_pre_upgrade_sys($sys)) {
            EDRu::create_flag('maincf_upgrade_precheck_done');
            return 0;
        }

        # create cmd list
        $treeref_olduser = dynupgrade_import_file2tree_sys($sys, "$rootpath$prod->{configdir}/main.cmd");

        maincf_check_TriggerResStateChange($sys,$treeref_olduser);

        # Update PreOnline attribute.
        maincf_preonline_precheck_sys($sys,$treeref_olduser);

        # vip will only bu updated for upgrade from less6.0 to 6.0 onwards
        $prod->maincf_precheck_if_update_vip_sys($sys);

        # Change sourcefile path for types/service groups to config dir
        modify_sourcefile_sys($sys,$treeref_olduser,'GRP');
        my $sourcefiles= modify_sourcefile_sys($sys,$treeref_olduser,'TYPE');
        dynupgrade_export_tree2file_sys( $sys, "$upgradepath/old_config/main.cmd", $treeref_olduser);
        $prod->translate_file_cmd2cf_sys( $sys, "$upgradepath/old_config");

        # Copy types for VRTSvcsea and VRTSdbac to old_conf;
        my ($extra_type,$extra_type_base,$pkg,$pkgi);
        for my $pkgi (qw(VRTSvcsea60 VRTSdbac60)) {
            $pkg = $prod->pkg($pkgi,1);
            next unless (defined $pkg);
            push (@extra_types,@{$pkg->{extra_types}});
        }
        for my $extra_type (@extra_types,@{$prod->{extra_types}}) {
            $extra_type_base = EDRu::basename($extra_type);
            if (EDRu::inarr($extra_type_base, @{$sourcefiles})) {
                $sys->cmd("_cmd_cp $upgradepath/old_config/$extra_type_base $upgradepath/old_conf")
                    unless ($sys->exists("$upgradepath/old_conf/$extra_type_base"));
                $sys->set_value("maincf_include,$extra_type_base", $extra_type_base);
            }
        }

        unless ($prod->check_obsolete_types_sys($sys,$treeref_olduser)) {
            EDRu::create_flag('maincf_upgrade_precheck_done');
            return 0;
        }

        my $panic_system_on_dg_loss_attr = dynupgrade_get_attrvalue_of_typename($treeref_olduser,'DiskGroup','PanicSystemOnDGLoss');
        if ($panic_system_on_dg_loss_attr eq '1') {
            my @reslist=dynupgrade_get_resname_list_from_typename($treeref_olduser, 'DiskGroup');
            if (@reslist) {
                $msg = Msg::new("The default value of the diskGroup agent attribute, PanicSystemOnDGLoss, will be changed from 1 to 0 after upgrade. Refer to the Veritas Cluster Server Bundled Agents Reference Guide for the implications of this change.", 40, 3383);
                $sys->push_warning($msg);
            }
        }

        $prod->upgrade_cvm_cfs_types_sys($sys,$treeref_olduser);

        @grps=dynupgrade_get_grplist($treeref_olduser);

        # Add AutoStartList to VxSS Service Group if it is not defined.
        if (EDRu::inarr('VxSS',@grps)) {
            $vxss_autostartlist = dynupgrade_get_attr_from_tree($treeref_olduser,'GRP','VxSS','AutoStartList');
            unless ($vxss_autostartlist) {
                @grp_syslist = dynupgrade_get_systemlist_of_grpname($treeref_olduser,'VxSS');
                $grp_syslists = join (' ', @grp_syslist);
                $vxss_add_autostartlist = "hagrp -modify VxSS AutoStartList $grp_syslists";
                $sys->set_value('maincf_upgrade,21_vxss_add_autostartlist', $vxss_add_autostartlist);
            }
        }

        # Update DNS ResRecord.
        my $dns_resrecord = dynupgrade_modify_attr_value_DNS($treeref_olduser);
        if ($dns_resrecord) {
            $sys->set_value('maincf_upgrade,30_dns_resrecord', $dns_resrecord);
        }

        #remove service group.
        #Add 99_ prefix to make sure the delete operation is added to the end.
        $sfua_delete_SfuaBase='hagrp -delete Sfua_Base';
        $sys->set_value('maincf_upgrade,99_sfua_delete_SfuaBase', $sfua_delete_SfuaBase);
        $delete_cmc='hagrp -delete CMC';
        $sys->set_value('maincf_upgrade,99_delete_CMC', $delete_cmc);

        $delete_VCSweb='hares -delete VCSweb';
        $sys->set_value('maincf_upgrade,99_delete_VCSweb', $delete_VCSweb);

        $prod->padv_maincf_upgrade_precheck_sys($sys,$treeref_olduser) if ($prod->can('padv_maincf_upgrade_precheck_sys'));
        $prod->padv_update_netmask($sys,$treeref_olduser);

        EDRu::create_flag('maincf_upgrade_precheck_done');
    } else {
        EDRu::wait_for_flag('maincf_upgrade_precheck_done');
    }
    return 1;
}

# update the Oracle and Netlsnr resource when upgrade
# for Oracle, if LevelTwoMonitorFreq exist, set it to 0 if DetailMonitor is 0
#             if LevelTwoMonitorFreq not exist, set it as the DetailMonitor
#             and remove type DetailMonitor(do not do this, only delete from resource)
# for Netlsnr, just override LevelTwoMonitorFreq attr and set it to 1.
# this applies to all upgrade path
sub maincf_result_upgrade_Oracle_sys {
    my ($prod,$sys,$treeref) = @_;
    my (@reslist,$res,$res_ref,$attr_ref,$attr_ref_in,$attr_value);
    my ($result_lines,$rtn);

    @reslist = dynupgrade_get_resname_list_from_typename($treeref,'Oracle');
    for my $res (@reslist) {
        $res_ref = dynupgrade_get_node_from_tree($treeref,'RES',$res);
        $attr_ref = dynupgrade_get_attr_from_node($res_ref,'DetailMonitor');
        $attr_ref_in = dynupgrade_get_attr_from_node($res_ref,'LevelTwoMonitorFreq');
        if ($attr_ref_in) {
            if ( $attr_ref ) {
                $attr_value = $attr_ref->{MODIFY}->{ATTRVAL}->[0];
                $result_lines .= "hares -modify $res LevelTwoMonitorFreq 0\n" if ($attr_value == 0);
                delete( $res_ref->{ATTR}->{DetailMonitor});
            }
        } else {
            if ( $attr_ref ) {
                $result_lines .= "hares -override $res LevelTwoMonitorFreq\n";
                $attr_value = $attr_ref->{MODIFY}->{ATTRVAL}->[0];
                $result_lines .= "hares -modify $res LevelTwoMonitorFreq $attr_value\n";
                delete( $res_ref->{ATTR}->{DetailMonitor});
            }
        }
    }
    @reslist = dynupgrade_get_resname_list_from_typename($treeref,'Netlsnr');
    for my $res (@reslist) {
        $res_ref = dynupgrade_get_node_from_tree($treeref,'RES',$res);
        $attr_ref = dynupgrade_get_attr_from_node($res_ref,'LevelTwoMonitorFreq');
        next if $attr_ref;
        $result_lines .= "hares -override $res LevelTwoMonitorFreq\n";
        $result_lines .= "hares -modify $res LevelTwoMonitorFreq 1\n";
    }

    # if os upgraded, CPI will use hacf of PBE, and following new attr will make the old hacf fail
    # so the cmds will be print to user
    if (Cfg::opt('rootpath') && $sys->{osupgraded}) {
        $sys->{dyn_manual_cmd_lu} .= $result_lines if $result_lines;
    } else {
        my @lines= split(/\n+/, $result_lines);
        $treeref= dynupgrade_import_lines2tree(\@lines,$treeref);
    }
    return $treeref;
}
    
package Padv::Sol11sparc;

sub media_patch_file {
    my ($padv,$patch,$patchdir)=@_;
    return '' unless ($patchdir);

    my $file="$patchdir/$patch->{patchname}.p5p";
    return $file if (-e $file);

    $file="$patchdir/VRTSpatches.p5p";
    if (-e $file) {
        $patch->{batchinstall}=1;
        return $file;
    }
    return $patchdir;
}

sub patch_install_sys {
    my ($padv,$sys,$patch)=@_;
    my ($rootpath,$iof,$path,$stat,$patchfile);

    $patchfile = $padv->installpath_sys($sys, $patch);
    $rootpath=Cfg::opt('rootpath')||'';
    $rootpath="-R $rootpath" if ($rootpath);
    $iof=EDRu::outputfile('install', $sys->{sys}, "patch.$patch->{patch_vers}");
    $path=EDRu::dirname($patchfile);
    $stat=$sys->filestat($path);
    $sys->cmd("_cmd_chmod a+x $path");

    #$sys->cmd("_cmd_pkg $rootpath install -g $archive $patches 2>$iof 1>&2");
    # 'pkg install -g VRTSpatches.p5p <patches>' do not work well when some zones in running state
    # According to 2707052: call set-publisher first
    $sys->cmd("_cmd_pkg $rootpath set-publisher -p $patchfile Symantec 2>>$iof 1>&2; _cmd_pkg $rootpath install --accept --no-backup-be $patch->{patchname} 2>>$iof 1>&2; _cmd_pkg $rootpath unset-publisher Symantec 2>>$iof 1>&2; _cmd_svcadm clear application/pkg/system-repository 2>/dev/null");
    $sys->change_filestat($path,$stat);
    return 1;
}


package Prod::VM60::Common;

sub upgrade_prestop_sys {
    my ($prod,$sys) = @_;
    my $syslist=CPIC::get('systems');
    # e2135884: roll back the change for dbms start script name.
    # $prod->tsub("disable_vrtsdbms3_sys",$sys);
    if ((Cfg::opt('vvr')) && !Cfg::opt('rootpath')) {
        if ($sys->system1) {
            for my $sysi (@$syslist) {
                $prod->vvr_upgrade_prestop_sys($sysi);
            }
            EDRu::create_flag('vvr_upgrade_prestop_done');
        } else {
            EDRu::wait_for_flag('vvr_upgrade_prestop_done');
        }
    }

    # check contents of /etc/vx/vxvm.exclude before upgrade if previous version <6.0
    $prod->check_exclude_file_sys($sys);

    if (Cfg::opt('rollback')) {
        # turn off dmp_native_support before rollback
        $prod->dmp_osn_stop_sys($sys,'rollback');
    } elsif (!Cfg::opt('upgrade_nonkernelpkgs')) {
        # turn off dmp native support before upgrade
        $prod->dmp_osn_stop_sys($sys,'upgrade');
    }

    return '';
}

package Pkg::VRTSvxvm60::RHEL6x8664;

sub init_padv {
    my $pkg=shift;
    $pkg->{hfvers}='6.0.500.100';
    $pkg->{oslibs}{all}={
        "glibc"  =>  "glibc-2.12-1.25.el6.i686 glibc-2.12-1.25.el6.x86_64",
        "libgcc"  =>  "libgcc-4.4.5-6.el6.i686 libgcc-4.4.5-6.el6.x86_64",
        "libstdc++"  =>  "libstdc++-4.4.5-6.el6.i686 libstdc++-4.4.5-6.el6.x86_64",
        "nss-softokn-freebl(x86-32)"  =>  "nss-softokn-freebl-3.12.9-3.el6.i686",
        "parted"  =>  "parted-2.1-13.el6.x86_64",
        "policycoreutils"  =>  "policycoreutils-2.0.83-19.8.el6_0.x86_64",
    };
    return;
}

package Pkg::VRTSaslapm60::Linux;

sub init_plat {
    my $pkg=shift;
    $pkg->{donotrmonupgrade}=1;
    $pkg->{hfvers}='6.0.500.300';
    return;
}

package Rel::UXRT601::Linux;

sub platvers_sys {
    my ($rel,$sys) = @_;
    my ($patchlevel,$msg,$rpm_dir,$kret,$ksles,$padv,$kstring,$cpic,@f,$distro,@pkg_files,$pkg,$prod);
    my ($localsys,$mediapath,$tmpdir,$vmware,$testVMware,@vmware_tool_scripts,$vmware_script,$state);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return '' if ($sys->{stop_checks});
    return '' if (Cfg::opt('ignorechecks'));

    $cpic=Obj::cpic();
    $prod=$sys->prod($cpic->{prod});
    $padv=$sys->padv($sys->{padv});
    $distro=$padv->distro_sys($sys);
    $kstring=$sys->cmd('_cmd_uname -r');
    $kstring =~ /\-(\d+)/m;
    $kret = $1;
    $ksles=$1 if ($kstring =~ /\-\d+\.(\d+)/mx);
    @f=split(/\W/m,$kstring);

    $localsys = Obj::localsys();
    $mediapath=EDR::get('mediapath');
    $tmpdir = EDR::tmpdir();
    $vmware = 0;
    $testVMware = "$mediapath/scripts/bin/testVMware";
        
    # Check ESX/KVM for AppHA, if vmware tools not installed or not running, quit installer 
    # For VCS in VMware environment, if vmware-tools not installed or not running, give warning
    if ($prod->{prod} =~ /ApplicationHA|VCS/) {
        $testVMware = "/opt/VRTSsfmh/bin/testVMware" unless ($localsys->exists($testVMware));
        if ($localsys->exists($testVMware)) {
            $localsys->copy_to_sys($sys,$testVMware,"$tmpdir/");
            $sys->cmd("$tmpdir/testVMware");
            if (EDR::cmdexit() == 0) {
                # it's VMware
                $sys->set_value('vmtype','vmware');
                $vmware = 1;

                @vmware_tool_scripts=('/etc/init.d/vmware-tools','/etc/vmware-tools/services.sh','/etc/init.d/vmware-tools-services');
                for my $script (@vmware_tool_scripts) {
                    if($sys->exists($script)) {
                        $vmware_script=$script;
                        last;
                    }
                }

                if ($vmware_script) {
                    $state = $sys->cmd("$vmware_script status 2> /dev/null");
                    # vmware-tools not running
                    if ($state =~ /not/i) {
                        # try to start it
                        $sys->cmd("$vmware_script start 2> /dev/null");
                        sleep 5;
                        $state = $sys->cmd("$vmware_script status 2> /dev/null");
                        if ($state =~ /not/i) {
                            $msg = Msg::new("vmware-tools is not running on $sys->{sys}, installer attempted to start it but failed. Please start the tool before installing $prod->{abbr}", 40, 5496, "$sys->{sys}", "$prod->{abbr}");
                            if ($prod->{prod} =~ /ApplicationHA/) {
                                $sys->push_error($msg);
                                return '';
                            } else {
                                $sys->push_warning($msg);
                            }
                        }
                    }
                } else {
                    $msg = Msg::new("The installer did not detect vmware-tools installed on $sys->{sys}, install vmware-tools and start it before you perform any task on $prod->{abbr}", 40, 5497, "$sys->{sys}", "$prod->{abbr}");
                    if ($prod->{prod} =~ /ApplicationHA/) {
                        $sys->push_error($msg);
                        return '';
                    } else {
                        $sys->push_warning($msg);
                    }
                }
            }
        } else {
            $msg = Msg::new("The installer can not detect if you are running under VMware environment on $sys->{sys}, it is required to install $prod->{prod} under the VMware or the KVM environment.", 40, 5498, "$sys->{sys}", "$prod->{prod}");
            $sys->push_warning($msg);
            # if installer do NOT have testVMware utility, print a warning but pass the check
            $vmware = 1;
        }

        # AppHA only support to be installed on VMware VM, and KVM on RHEL,
        # do not support to be installed on physical systems, or KVM on OEL/SuSE
        if (!$vmware && $prod->{prod} =~ /ApplicationHA/) {
            if ($sys->{virtual_type} eq Padv->VIRT_KVM) {
                if ($distro ne 'RHEL') {
                    $msg=Msg::new("$prod->{prod} for KVM does not support to be installed to OEL and SuSE.", 40, 5499, "$prod->{prod}");
                    $sys->push_error($msg);
                    return '';
                }
            } else {
                $msg=Msg::new("$prod->{prod} is only supported on systems running under VMware or KVM environment", 40, 5500, "$prod->{prod}");
                $sys->push_error($msg);
                return '';
            }
        }
    }

    if ($distro eq 'RHEL') {
        $padv->{cmd}{selinuxenabled}='/usr/sbin/selinuxenabled';
        #On RHEL6.x, install/upgrade VCS6.0.4 + VCS 6.0.5 is not allowed
        if ($cpic->{base_path_release} eq '6.0.4') {
            $msg = Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported if base release version is 6.0.4. Only Oracle Linux 6.4 with UEK kernel is supported.", 40, 5501, "$kstring", "$sys->{sys}");
            $sys->push_error($msg);
            return '';
        }
        if ($kstring =~ /el6/m) {
            if ($kret < 131) { # RHEL6u1 check
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.32-131.el6(RHEL6 U1) or higher is supported on RHEL6. Upgrade the OS to RHEL6 Update1 in order to install this product", 40, 4886, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }

            # 6.0.5 only support up to RHEL6U5.
            # RHEL6U6 kernel: 2.6.32-504.el6.x86_64
            if ($kret > 431) {
                $patches='';
                for my $pkgname (qw/VRTSaslapm60 VRTSvxvm60/) {
                    $pkg = $sys->pkg($pkgname);
                    $pkgvers=$pkg->version_sys($sys);
                    $hfvers = $pkg->{hfvers};
                    if (EDRu::compvers($pkgvers,$hfvers) == 2) {
                        if ($kret == 504) {
                            last if $prod->{prod} =~ /VCS/;
                            $patches.="SFHA patch for RHEL6 U6: https://sort.symantec.com/patch/detail/9326";
                        }
                        $msg=Msg::new("System $sys->{sys} is running Kernel Release $kstring. $rel->{reltitle} $rel->{titlevers} is not supported on Kernel Release $kstring without additional patches. Visit Symantec SORT web site to download the following required patches and follow the patch instructions to install the patches:\n\n\t$patches");
                        $sys->push_error($msg);
                        return '';
                    }
                }
            }
        } elsif ($kstring =~ /el5/m) {
            if ($kret < 194) { # RHEL5u5 or above check
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.18-194.el5(RHEL5 U5) or higher is supported on RHEL5. Upgrade the OS to RHEL5 Update5 or higher in order to install this product", 40, 4887, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
        }

    } elsif ($distro eq 'SLES') {
        $padv->{cmd}{selinuxenabled}='/usr/bin/selinuxenabld';

        #SLES 10 SP4 check
        #In 6.0, SLES 10 SP4 is supported,
        #SP3 kernel version is above 2.6.16.60-0.54.5
        #SP4 kernel version is above 2.6.16.60-0.85.1
        #For PATCHLEVEL is higher than SuSE 10 SP4 and SuSE11 SP2, maybe does not support

        $patchlevel=$sys->{patchlevel};

        if ($f[0]>2) {
            # sles11sp2 is using kernel 3.x
            if ((defined($patchlevel)) && ($patchlevel>3)){
                $msg=Msg::new("SuSE 11 SP1, SP2 and SP3 are the recommended platforms for the release on $sys->{sys}.", 40, 5502, "$sys->{sys}");
                $sys->push_warning($msg);
                return '';
            }
        } else {
            if (($f[2]==16) && ($f[3] < 60)) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.16.60-0.85(SLES10 SP4) or higher is supported on SuSE10. Upgrade the OS to SLES10 SP4 in order to install this product", 40, 4888, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
            if (($f[2]==16) && ($f[3]==60) && ($f[5] < 85)) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.16.60-0.85(SLES10 SP4) or higher is supported on SuSE10. Upgrade the OS to SLES10 SP4 in order to install this product", 40, 4888, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }

            #SLES 11 SP1 or above check
            if (($f[2]==27) || (($f[2]==32) && ($f[3] < 12))) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Kernel Release 2.6.32.12(SLES11 SP1) or higher is supported on SuSE11. Upgrade the OS to SLES11 SP1 in order to install this product", 40, 4889, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
            if (($f[2]==32) && (defined($patchlevel)) && ($patchlevel>2)){
                $msg=Msg::new("SuSE 11 SP1 and SP2 are the recommended platforms for the release on $sys->{sys}.", 40, 4963, "$sys->{sys}");
                $sys->push_warning($msg);
                return '';
            }
        }
    } elsif ($distro eq 'ESX') {
        # FIX: Figure this check out later for VMWARE. (Get versions of ESX supported and edit)
        if ($kret < 34) {
            $msg=Msg::new("Kernel Release $kstring is not supported on $sys->{sys}. Only Kernel Release 2.6.9-34.EL or higher is supported on RHEL.", 40, 3992, "$kstring", "$sys->{sys}");
            $sys->push_error($msg);
            return '';
        }
    } elsif ($distro eq 'OL') {
        # FIX: Figure this check out later for Oracle Linux.
        $padv->{cmd}{selinuxenabled}='/usr/sbin/selinuxenabled';
        if ($kstring =~ /el6/m) {
            # OL6 check
            #On OL6.4/6.5(uek2 kernel), install/upgrade non-VCS is not allowed
            if ($prod->{prod} ne 'VCS' && $kstring =~ /uek/m && $cpic->{base_path_release}) {
                $msg = Msg::new("$prod->{prod} is not supported on OL6.x UEK2 kernel", 40, 5503, "$prod->{prod}");
                $sys->push_error($msg);
                return '';
                # On OL6.4/6.5(Redhat Compatible Kernel), install/upgrade VCS6.0.4 + VCS 6.0.5 is not allowed
            } elsif ($kstring !~ /uek/m && $cpic->{base_path_release} eq '6.0.4' && $kstring =~ /^2\.6\.32-[34].*/m) {
                $msg = Msg::new("VCS 6.0.4 is not supported on $kstring kernel, please reboot into UEK2 kernel or use 6.0.1/6.0.2 base media then run installer again", 40, 5504, "$kstring");
                $sys->push_error($msg);
                return '';
            }
            if ($kstring =~ /uek/m && $kstring !~ /^2\.6\.39-400.*el6uek/m) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Unbreakable Enterprise Kernel 2.6.39-400 is supported on Oracle Linux 6.4 and Oracle Linux 6.5. you may configure the operating system to boot with this kernel instead.", 40, 5505, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
            if ($kstring !~ /uek/m && $kstring !~ /^2\.6\.32/m) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Red Hat compatible Kernel Release 2.6.32 is supported on Oracle Linux 6.4 and Oracle Linux 6.5. you may configure the operating system to boot with this kernel instead.", 40, 5506, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }


            if ($kstring !~ /uek/m && $cpic->{base_path_release}) {
                if ($kstring =~ /^2\.6\.32-[12].*/m) {
                    #On OL6.1~6.3(Redhat Compatible Kernel), install/upgrade 6.0.1/6.0.2+6.0.5 should print warnning message
                    if ($cpic->{base_path_release} ne '6.0.4') {
                        $msg = Msg::new("System should not be booted into UEK Kernel mode since $prod->{prod} is not supported on OL 6.1 to 6.3 UEK kernel", 40, 5507, "$prod->{prod}");
                        $sys->push_warning($msg);
                    } else {
                        #On OL6.1~6.3(Redhat Compatible Kernel), install/upgrade 6.0.4+6.0.5 should block 
                        $msg = Msg::new("VCS 6.0.4 is not supported on $kstring kernel, please use 6.0.1/6.0.2 base media then run installer again", 40, 5508, "$kstring");
                        $sys->push_error($msg);
                        return '';
                    }    
                    #On OL6.4/6.5(Redhat Compatible Kernel),install/upgrade non-VCS should print warning message
                } elsif ($prod->{prod} ne 'VCS' && $kstring =~ /^2\.6\.32-[34].*/m) {
                    $msg = Msg::new("System should not be booted into UEK Kernel mode since $prod->{prod} is not supported on UEK kernel", 40, 5509, "$prod->{prod}");
                    $sys->push_warning($msg);
                }    
            }

        } elsif ($kstring =~ /el5/m) {
            # OL5 check
            if ($kstring =~ /uek/m || $kstring !~ /^2\.6\.18/m) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Red Hat compatible Kernel Release 2.6.18 is supported on Oracle Linux 5. you may configure the operating system to boot with this kernel instead.", 40, 4785, "$kstring", "$sys->{sys}");
                $sys->push_error($msg);
                return '';
            }
        }
    } else {
        $msg=Msg::new("Distribution is not recognized as SuSE or Red Hat on $sys->{sys}", 40, 3993, "$sys->{sys}");
        $sys->push_error($msg);
        return '';
    }

    # XEN is not supported in 6.0
    if ($kstring =~ /xen/m) {
        $msg=Msg::new("Kernel Release on $sys->{sys} is $kstring. Xen kernel is not supported in this release.", 40, 4890, "$sys->{sys}", "$kstring");
        $sys->push_error($msg);
        return '';
    }

    # Distribution match on all systems removed as any to many support is added
    for my $pkgi (@{$rel->allpkgs}) {
        $pkg=$sys->pkg($pkgi);
        push(@pkg_files,$pkg->{file});
    }

    if (@pkg_files) {
       return 1 if ($cpic->{fromdisk});
       $rpm_dir=$rel->pkgs_patches_dir('pkgpath');;
       if ((($sys->{arch}=~/686/m)||($sys->{arch}=~/586/m)) && (grep {/86_64/m} @pkg_files)) {
            $msg=Msg::new("System $sys->{sys} is architecture $sys->{arch} and x86_64 rpms were found in $rpm_dir. Check that you are installing from the correct DVD. Some rpms will not install correctly on this system.", 40, 3626, "$sys->{sys}", "$sys->{arch}", "$rpm_dir");
            $sys->push_error($msg);
            return '';
        }

       if ((($distro eq 'RHEL') && (grep {/SLES/} @pkg_files))
            ||  (($distro eq 'SLES') && (grep {/RHEL/} @pkg_files))) {
            $msg=Msg::new("System $sys->{sys} is distribution $distro and incompatible rpms were found in $rpm_dir. Check that you are installing from the correct DVD. Some rpms will not install correctly on this system.", 40, 3627, "$sys->{sys}", "$distro", "$rpm_dir");
            $sys->push_error($msg);
            return '';
       }
    }

    # No need for selinux checks any more
    return 1;
}

1;
