#HOTFIX_VERSION:cpi-6.0P3
package Prod::SF60::HPUX;

sub init_plat {
    my $prod=shift;
    $prod->{allpkgs}=[ qw(VRTSob34 VRTSdbed60 VRTSodm60) ];
    $prod->{minpkgs}=[ qw() ];
    $prod->{recpkgs}=[ qw(VRTSob34 VRTSdbed60 VRTSodm60) ];
    $prod->{upgradevers}=[qw(3.5 4.1 5.0 5.1)];
    $prod->{obsoleted_bundles}=[ qw(Base-VxTools-50 Base-VxVM-50 B3929FB Base-VxFS-50 Base-VxVM Base-VxTools-501 Base-VxVM-501 B3929GB Base-VxFS-501) ];
    $prod->{obsoleted_bundled_pkgs}=[ qw(AVXTOOL AVXVM AONLINEJFS OnlineJFS01 AVXFS) ];
    $prod->{zru_releases}=[qw()];

    $prod->{obsoleted_previous_releases_pkgs} = [ qw(
        VRTSdbckp VRTSormap VRTScsocw VRTScfsdc VRTScpi VRTSap
        VRTStep VRTSvcsor VRTSgapms VRTSmapro VRTSvail VRTSorgui
        VRTSorweb VRTSodm60 VRTSvxmsa VRTSvrdev VRTSdbdoc VRTSdbed60
        VRTSdbcom VRTSfsmnd VRTSfssdk60 VRTSfsadv60 VRTSfsdoc VRTSfsman
        VRTSvrdoc VRTSvrw VRTSvrmcsg VRTSweb VRTSvcsvr VRTSvrpro
        VRTSddlpr VRTSvdid VRTSvsvc VRTSvmpro VRTSalloc VRTSdcli
        VRTSvmdoc VRTSvmman SYMClma VRTSspt60 VRTSaa VRTSmh
        VRTSccg VRTSobgui VRTSfspro VRTSdsa VRTSsfmh41 VRTSob34
        VRTSobc33 VRTSaslapm60 VRTSat50 VRTSsmf VRTSpbx VRTSicsco
        VRTSvxfs60 VRTSvxvm60 VRTSjre15 VRTSjre VRTSsfcpi60 VRTSperl512
        VRTSvlic32 VRTSwl
    ) ];
    return;
}

my $cpic=Obj::cpic();
if ($cpic->{padv}=~/HPUX/) {
    my $prod=Obj::prod('SF60','HPUX1131ia64');
    my @missing_bundles = ('Base-VxTools-501', 'Base-VxVM-501', 'B3929GB', 'Base-VxFS-501');
    foreach my $missing_bdl (@missing_bundles) {
        if (EDRu::arrpos($missing_bdl, @{$prod->{obsoleted_bundles}}) == -1 ){
            push @{$prod->{obsoleted_bundles}}, $missing_bdl;
        }
    }
}

package Prod::VM60::HPUX;

sub init_plat {
    my $prod=shift;
    my $padv=$prod->padv();
    if ($prod->{class} =~ /DMP/m) {
        $prod->{allpkgs}=[ qw(VRTSvxvm60 VRTSaslapm60 VRTSsfmh41) ];
        $prod->{minpkgs}=[ qw(VRTSvxvm60 VRTSaslapm60) ];
        $prod->{recpkgs}=[ qw(VRTSvxvm60 VRTSaslapm60 VRTSsfmh41) ];
        push @{$prod->{tunables}},
            {
                "name" => "dmp_evm_handling",
                "desc" => Msg::new("Whether EVM should be handled or not")->{msg},
                "define_object" => $prod,
                "reboot" => 0,
                "type" => "enum",
                "values" => [ "on", "off" ],
                "when_to_set" => 1,
            },
            ;
    } else {
        $prod->{allpkgs}=[ qw(VRTSvxvm60 VRTSaslapm60 VRTSob34 VRTSsfmh41) ];
        $prod->{minpkgs}=[ qw(VRTSvxvm60 VRTSaslapm60) ];
        $prod->{recpkgs}=[ qw(VRTSvxvm60 VRTSaslapm60 VRTSob34 VRTSsfmh41) ];

        # Some tunables have different value range on HP-UX
        push @{$prod->{tunables}},
            {
                "name" => "vol_max_nmpool_sz",
                "desc" => Msg::new("Maximum name pool size (bytes)")->{msg},
                "define_object" => $prod,
                "reboot" => 0,
                "type" => "range",
                "values" => [ 2097152, 4294967295, undef ],
                "when_to_set" => 2,
            },
            {
                "name" => "vol_max_rdback_sz",
                "desc" => Msg::new("Storage Record readback pool maximum (bytes)")->{msg},
                "define_object" => $prod,
                "reboot" => 0,
                "type" => "range",
                "values" => [ 2097152, 4294967295, undef ],
                "when_to_set" => 2,
            },
            {
                "name" => "vol_max_wrspool_sz",
                "desc" => Msg::new("Maximum memory used in clustered version of VERITAS Volume Replicator")->{msg},
                "define_object" => $prod,
                "reboot" => 0,
                "type" => "range",
                "values" => [ 2097152, 4294967295, undef ],
                "when_to_set" => 2,
            },
            {
                "name" => "vol_rvio_maxpool_sz",
                "desc" => Msg::new("Maximum memory requested by VERITAS Volume Replicator (bytes)")->{msg},
                "define_object" => $prod,
                "reboot" => 0,
                "type" => "range",
                "values" => [ 2097152, 4294967295, undef ],
                "when_to_set" => 2,
            },
            {
                "name" => "vol_maxspecialio",
                "desc" => Msg::new("Maximum size of a VxVM I/O operation issued by an ioctl call (kBytes)")->{msg},
                "define_object" => $prod,
                "reboot" => 1,
                "type" => "range",
                "values" => [ 512, 2048, undef ],
                "when_to_set" => 2,
            },
            {
                "name" => "voldrl_min_regionsz",
                "desc" => Msg::new("Minimum size of a VxVM Dirty Region Logging (DRL) region (kBytes)")->{msg},
                "define_object" => $prod,
                "reboot" => 1,
                "type" => "range",
                "values" => [ 512, 2097512, undef ],
                "when_to_set" => 2,
            },
            {
                "name" => "voliomem_chunk_size",
                "desc" => Msg::new("Size of VxVM memory allocation requests (bytes)")->{msg},
                "define_object" => $prod,
                "reboot" => 1,
                "type" => "range",
                "values" => [ 32768, 524288, undef ],
                "when_to_set" => 2,
            },
            ;
    }
    $prod->{upgradevers}=[qw(3.5 4.1 5.0 5.1)];
    $prod->{zru_releases}=[qw()];

    $prod->{obsoleted_previous_releases_pkgs} = [ qw(
        VRTSvrmcsg VRTStep VRTSap VRTSvrdoc VRTSvrw VRTSweb
        VRTSvcsvr VRTSvrpro VRTSvxmsa VRTSvrdev VRTSddlpr VRTSvsvc VRTSvdid
        VRTSalloc VRTSdcli VRTSvmdoc VRTSvmpro VRTSvmman VRTScpi
        SYMClma VRTSspt60 VRTSaa VRTSmh VRTSccg VRTSobgui VRTSfspro
        VRTSdsa VRTSsfmh41 VRTSob34 VRTSobc33 VRTSaslapm60
        VRTSat50 VRTSsmf VRTSpbx VRTSicsco VRTSvxvm60 VRTSjre15
        VRTSjre VRTSsfcpi60 VRTSperl512 VRTSvlic32 VRTSwl
    ) ];
    $prod->{obsoleted_bundles}=[ qw(Base-VxTools-50 Base-VxVM-50 B3929FB Base-VxVM Base-VxTools-501 Base-VxVM-501) ];
    $prod->{obsoleted_bundled_pkgs}=[ qw(AVXTOOL AVXVM) ];

    $prod->{vxdbms3scripts}='/sbin/rc?.d/*vxdbms3';
    $padv->{cmd}{is_vxvmroot}='/sbin/is_vxvmroot';
    $padv->{cmd}{vxdctl}='/usr/sbin/vxdctl';
    $padv->{cmd}{vxdisk}='/usr/sbin/vxdisk';
    $padv->{cmd}{vxddladm}='/usr/sbin/vxddladm';
    $padv->{cmd}{vxrelocd}='/usr/lib/vxvm/bin/vxrelocd';
    $padv->{cmd}{vxcached}='/usr/lib/vxvm/bin/vxcached';
    $padv->{cmd}{vxattachd}='/usr/lib/vxvm/bin/vxattachd';
    $padv->{cmd}{vxdg}='/usr/sbin/vxdg';
    $padv->{cmd}{vxprint}='/usr/sbin/vxprint';
    $padv->{cmd}{vxscriptlog}='/usr/sbin/vxscriptlog';
    $padv->{cmd}{nohup}='/usr/bin/nohup';
    $padv->{cmd}{vxdmpadm}='/sbin/vxdmpadm';
    $padv->{cmd}{vxedit}='/usr/sbin/vxedit';
    $padv->{cmd}{vxvol}='/usr/sbin/vxvol';
    $padv->{cmd}{vxassist}='/usr/sbin/vxassist';
    $padv->{cmd}{vxrvg}='/usr/sbin/vxrvg';
    $padv->{cmd}{vxrlink}='/usr/sbin/vxrlink';
    $padv->{cmd}{vradmin}='/usr/sbin/vradmin';
    $padv->{cmd}{vxtune}='/usr/sbin/vxtune';
    $padv->{cmd}{vxrecover}='/usr/sbin/vxrecover';
    $padv->{cmd}{vras_script}='/etc/init.d/vras-vradmind.sh';
    $padv->{cmd}{vxclustadm}='/opt/VRTS/bin/vxclustadm';
    $padv->{cmd}{vxconfigd}='/sbin/vxconfigd';
    $padv->{cmd}{vxconfigbackupd}='/etc/vx/bin/vxconfigbackupd';
    $padv->{cmd}{vxgetrootdisk}='/etc/vx/bin/vxgetrootdisk';
    $padv->{cmd}{vxdisksetup}='/usr/lib/vxvm/bin/vxdisksetup';
    return;
}

my $cpic=Obj::cpic();
if ($cpic->{padv}=~/HPUX/) {
    my $prod=Obj::prod('VM60','HPUX1131ia64');
    my @missing_bundles = ('Base-VxTools-501', 'Base-VxVM-501');
    foreach my $missing_bdl (@missing_bundles) {
        if (EDRu::arrpos($missing_bdl, @{$prod->{obsoleted_bundles}}) == -1 ){
            push @{$prod->{obsoleted_bundles}}, $missing_bdl;
        }
    }
}

package Prod::DMP60::HPUX;

my $cpic=Obj::cpic();
if ($cpic->{padv}=~/HPUX/) {
    my $prod=Obj::prod('DMP60','HPUX1131ia64');
    my @missing_bundles = ('Base-VxTools-501', 'Base-VxVM-501');
    foreach my $missing_bdl (@missing_bundles) {
        if (EDRu::arrpos($missing_bdl, @{$prod->{obsoleted_bundles}}) == -1 ){
            push @{$prod->{obsoleted_bundles}}, $missing_bdl;
        }
    }
}

package Prod::SFRAC60::HPUX;

sub init_plat {
    my $prod=shift;
    $prod->{allpkgs}=[ qw(VRTSob34 VRTSdbac60 VRTSgms60 VRTSodm60 VRTSdbed60 VRTSvcsea60) ];
    $prod->{minpkgs}=[ qw(VRTSdbac60 VRTSgms60 VRTSodm60 VRTSdbed60 VRTSvcsea60) ];
    $prod->{recpkgs}=[ qw(VRTSob34 VRTSdbac60 VRTSgms60 VRTSodm60 VRTSdbed60 VRTSvcsea60) ];
    $prod->{upgradevers}=[qw(3.5 4.1 5.0 5.1)];
    $prod->{zru_releases}=[qw()];

    # no need to check cpu speed for HPUX
    $prod->{minimal_cpu_speed_requirment} = undef;

    $prod->{obsoleted_previous_releases_pkgs} = [ qw(
        VRTSdbckp VRTSormap VRTScsocw VRTSdbac60 VRTSvcsea60 VRTScfsdc VRTSodm60
        VRTSgms60 VRTScavf60 VRTSglm60 VRTScpi VRTSd2doc VRTSordoc
        VRTSfsnbl VRTSfppm VRTSap VRTStep VRTSspc VRTSspcq
        VRTSfasdc VRTSfasag VRTSfas VRTSgapms VRTSmapro VRTSvail
        VRTSd2gui VRTSorgui VRTSvxmsa VRTSvrdev VRTSdbdoc VRTSdb2ed VRTSdbed60
        VRTSdbcom VRTSsydoc VRTSsybed VRTSvcsApache VRTScmc
        VRTSccacm VRTSvcsw VRTScspro VRTSvcsdb VRTSvcsor VRTSvcssy
        VRTScmccc VRTScmcs VRTScscm VRTScscw VRTScssim
        VRTScutil VRTSvcsdc VRTSvcsmn VRTSvcsmg VRTSvcsag60
        VRTScps60 VRTSvcs60 VRTSvxfen60 VRTSgab60 VRTSllt60
        VRTSfsmnd VRTSfssdk60 VRTSfsdoc VRTSfsman VRTSvrdoc
        VRTSvrw VRTSvrmcsg VRTSweb VRTSvcsvr VRTSvrpro VRTSddlpr VRTSvdid
        VRTSvsvc VRTSvmpro VRTSalloc VRTSdcli VRTSvmdoc VRTSvmman
        SYMClma VRTSspt60 VRTSaa VRTSmh VRTSccg VRTSobgui VRTSfspro
        VRTSdsa VRTSsfmh41 VRTSob34 VRTSobc33 VRTSaslapm60
        VRTSat50 VRTSsmf VRTSpbx VRTSicsco VRTSvxfs60 VRTSvxvm60
        VRTSjre15 VRTSjre VRTSperl512 VRTSvlic32 VRTSwl
    ) ];

    my $padv=$prod->padv();

    $padv->{cmd}{vxdctl}='/usr/sbin/vxdctl';
    $padv->{cmd}{vxdisk}='/usr/sbin/vxdisk';
    $padv->{cmd}{vxddladm}='/usr/sbin/vxddladm';
    $padv->{cmd}{vxrelocd}='/usr/lib/vxvm/bin/vxrelocd';
    $padv->{cmd}{vxcached}='/usr/lib/vxvm/bin/vxcached';
    $padv->{cmd}{vxdg}='/usr/sbin/vxdg';
    $padv->{cmd}{vxprint}='/usr/sbin/vxprint';
    $padv->{cmd}{vxscriptlog}='/usr/sbin/vxscriptlog';
    $padv->{cmd}{nohup}='/usr/bin/nohup';
    $padv->{cmd}{vxdmpadm}='/sbin/vxdmpadm';
    $padv->{cmd}{vxedit}='/usr/sbin/vxedit';
    $padv->{cmd}{vxvol}='/usr/sbin/vxvol';
    $padv->{cmd}{vxassist}='/usr/sbin/vxassist';
    $padv->{cmd}{mkfs}='/usr/sbin/mkfs';
    $padv->{cmd}{format}='/usr/sbin/format';
    $padv->{cmd}{psrinfo} = '/usr/sbin/psrinfo';
    $padv->{cmd}{prctl} = '/usr/bin/prctl';
    $padv->{cmd}{prtconf} = '/usr/sbin/prtconf';
    $padv->{cmd}{sysdef} = '/usr/sbin/sysdef';

    # Setting SF Oracle RAC/PADV specific variables here
    $prod->{lib_path} = '/opt/VRTSvcs/rac/lib/';
    $prod->{bin_path} = '/opt/VRTSvcs/rac/bin/';
    $prod->{initdir} = '/etc/init.d/';
    $prod->{ipc_utility} = '/opt/VRTSvcs/ops/bin/ipc_version_chk_shared';
    $prod->{hosts_file} = '/etc/hosts';
    $prod->{mount_points} = "/usr/sbin/mount | awk '{print \$1}'";
    $prod->{initdir} = '/sbin/init.d';
    $prod->{vendor_skgxn_path} = '/opt/nmapi/nmapi2/[pa20_64|hpux64]/lib';
    $prod->{xdpyinfo_path} = '/usr/contrib/bin/X11/xdpyinfo';
    return;
}

my $cpic=Obj::cpic();
if ($cpic->{padv}=~/HPUX/) {
    my $prod=Obj::prod('SFRAC60','HPUX1131ia64');
    my $missing_pkg = 'VRTSvcsea60';
    if (EDRu::arrpos($missing_pkg, @{$prod->{allpkgs}}) == -1 ){
        push @{$prod->{allpkgs}}, $missing_pkg;
    }
    if (EDRu::arrpos($missing_pkg, @{$prod->{minpkgs}}) == -1 ){
        push @{$prod->{minpkgs}}, $missing_pkg;
    }
    if (EDRu::arrpos($missing_pkg, @{$prod->{recpkgs}}) == -1 ){
        push @{$prod->{recpkgs}}, $missing_pkg;
    }
}

package Padv::HPUX;

sub bundle_pkg_sys {
    my ($padv,$sys,$bundle_pkg)= @_;
    my $rc=$sys->cmd("_cmd_swlist -l product -a revision -x verbose=0 $bundle_pkg 2>/dev/null | _cmd_grep $bundle_pkg | _cmd_awk '{print \$1}'");

    if ($rc) {
        if ($rc eq $bundle_pkg) {
            return 1;
        }else{
            return 0;
        }
    }
    return 0;
}

sub bundle_sys {
    my ($padv,$sys,$bundle)= @_;
    my $rc=$sys->cmd("_cmd_swlist -l bundle 2>/dev/null | _cmd_grep -w $bundle | _cmd_awk '{print \$1}'");

    # Another implementation
    #my $rc=$sys->cmd("_cmd_swlist -a fileset $bundle 2>/dev/null | _cmd_grep -v '^\#' | _cmd_grep -c '$bundle'");

    if ($rc) {
        if ($rc eq $bundle) {
            return 1;
        }else{
            return 0;
        }
    }
    return 0;
}

package Prod::VM60::Common;

sub upgrade_precheck_sys {
    my ($prod,$sys)= @_;
    my ($pkg,$cprod);

    $prod->large_lun_check_sys($sys);

    #fix e1518138 : uninstall AT pkgs when upgrade VM/VVR
    $cprod=CPIC::get('prod');
    if (EDRu::inarr($cprod,qw/VM60 DMP60/)) {
        $prod->obsoleted_bundles_sys($sys);
        $prod->obsoleted_bundled_pkgs_sys($sys);
    }
    $prod->obsoleted_thirdparty_pkgs_sys($sys);
    # check if dg has latest backup before upgrade
    $prod->dg_conf_upgrade_precheck_sys($sys) unless (Cfg::opt('upgrade_nonkernelpkgs'));
    if ($prod->is_encapsulated_bootdisk_sys($sys)){
        return '' unless ($prod->encap_upgrade_precheck_sys($sys));
        # set donotstop and donotremove flag here.
        $prod->set_flag_encap_upgrade_sys($sys);
    }
    $prod->open_volumes_sys($sys, 'upgrade') unless (Cfg::opt(qw(upgrade_kernelpkgs upgrade_nonkernelpkgs)));
    $prod->vvr_upgrade_precheck_sys($sys) if ($prod->vr_licensed_sys($sys));
    return;
}

sub dg_conf_upgrade_precheck_sys {
    my ($prod,$sys) = @_;
    my ($cf_bkup_path,$cmd,$dg,$dgid,$dglist,$msg,$rootpath,$voldmode);
    return if ($sys->exists("$prod->{mkdbfile}"));
    return if ($prod->vol_is_vm_disabled_sys($sys));
    $voldmode = $prod->vold_status_sys($sys);
    if ($voldmode !~ /enabled/m ) {
        if ($sys->{donot_restart_vxconfigd}) {
            return '';
        } else {
            # If vold is down, atempt to start it
            # firstly try to start without '-r reset' since '-r reset' will increase dg seq no
            if ($voldmode =~ /not-running/m) {
                Msg::log("vxconfigd is not running on $sys->{sys}; restarting");
                $sys->cmd('_cmd_vxconfigd -k -x syslog 2> /dev/null');
            } else {
                # the mode could be disabled or booted
                Msg::log("vxconfigd is in $voldmode on $sys->{sys}; re-enabling");
                $sys->cmd('_cmd_vxdctl enable >/dev/null 2>&1');
            }
            $voldmode = $prod->vold_status_sys($sys);
            if ($voldmode !~ /enabled/m ) { 
                $sys->cmd('_cmd_vxconfigd -k -r reset -x syslog 2> /dev/null');
                $voldmode = $prod->vold_status_sys($sys);
                if ($voldmode !~ /enabled/m ) {
                    $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. Upgrading VRTSvxvm in this scenario can result in configuration errors, or data loss for Veritas Volume Manager objects. Continue if you are sure that there are no Veritas Volume Manager objects on the host.");
                    $sys->push_warning($msg);
                    return '';
                }
            }
        }
    }
    $dglist = $sys->cmd("_cmd_vxdg -q list 2> /dev/null");
    for my $line(split(/\n/,$dglist)) {
        if ($line =~ /(\S+)\s+\S+\s+(\S+)/m) {
            $dg = $1;
        } else {
            next;
        }

        # For 2808458: to check if any imported dg with mix of regular and clone disks
        if ($prod->is_dg_mix_clone_sys($sys,$dg)) {
            $sys->set_value('dgs_mix_clone','push',$dg);
        }
    }

    if ($sys->{dgs_mix_clone}) {
        $dglist = join(" ", @{$sys->{dgs_mix_clone}});
        $msg = Msg::new("The following disk group(s) have mix of regular and clone disk on $sys->{sys}:\n\t$dglist\nplease verify disk information and unset clone flag before upgrade.");
        $sys->push_error($msg);
    }
    return;    
}

# To check if the dg is mix of regular and clone disks
sub is_dg_mix_clone_sys {
    my ($prod,$sys,$dg) = @_;
    my ($disks,$disknum,$clonedisknum);

    $disknum=0;
    $clonedisknum=0;
    $disks = $sys->cmd("_cmd_vxdisk list -q -g $dg 2>/dev/null");
    for my $disk (split(/\n/, $disks)) {
        $disknum++;
        $clonedisknum ++ if ($disk =~ /clone_disk/m)
    }

    return 1 if ($clonedisknum && $clonedisknum != $disknum);
    return 0;
}

sub large_lun_check_sys {
    my ($prod,$sys)=(@_);
    my ($tmpdir,$ivers,$disks,@disks,@disks_with_issue,$mode,$pkg,$msg);
    my ($cnt,$devcnt,$lpath,$nmp,$tmpfile,$tbnum,$tmpfile2,$vxscsi);
    my ($cap,$cyl,$ncyl,$ncyl_lmt,$nhead,$nsect,$rsize,$tb_mkr);
    my ($alt_cyl,$cyl_msb,$cyl_lsb,$cylsize,$hd,$hd_msb,$hd_lsb,$sec,$sec_msb,$sec_lsb,$tcyl);
    my ($disk_s,$disk_s2,$disk_size1,$disk_size2,$vxdisk);
    my ($label,$maxcap,$secoff,$skip_value,$tdisk_size);

    # VM have large LUN disk issue on 5.1SP1, 5.1SP1RP1, 5.1SP1RP2, 5.1SP1RP3
    # For more information, visit tech note:
    #   http://www.symantec.com/business/support/index?page=content&id=TECH155921
    $ivers=$prod->version_sys($sys);
    if ($ivers !~ /5\.1\.1\d[0123]/) {
        return 1;
    }

    # Need check large LUN if 5.1SP1, 5.1SP1RP1, 5.1SP1RP2, 5.1SP1RP3
    Msg::log("Checking large LUN: VM version: $ivers");

    # check vxvm is running
    $mode=$sys->cmd("_cmd_vxdctl mode 2>/dev/null");
    if ($mode!~/mode: enabled/) {
        # vxconfigd not running, installer will not re-start 5.1SP1 vxconfigd to gather data
        $sys->set_value('donot_restart_vxconfigd', 1);
        return 0;
    }
 
    # Check if any disk size > 1T, if yes, do not re-start 5.1SP1 vxconfigd.
    my $sol_disklmt=2147483647;
 
    $disks=$sys->cmd("_cmd_vxdisk -q list 2>/dev/null | _cmd_grep ':cdsdisk' | _cmd_grep -v error | _cmd_awk '{print \$1}' | _cmd_grep -v 'was:'");
    return 0 unless $disks;
 
    @disks=split(/\s+/,$disks);
    @disks_with_issue=();
 
    $tmpdir=EDR::tmpdir();
    $nmp=$sys->cmd("_cmd_vxddladm get namingscheme 2>/dev/null | _cmd_tail -1 | _cmd_grep New | _cmd_wc -l");
 
    for my $disk (@disks) {
        my ($flag,$flags,@flags,$format,$invalid,$len,$off,$pub);

        Msg::log("Checking if there is 1TB issue for $disk");
        if ($nmp == 1) {
            $lpath="/dev/rdisk/$disk";
        } else {
            $lpath="/dev/vx/rdmp/$disk";
        }

        # Raw Geometry Mode Sense 3/4 Values
        $vxscsi=$sys->cmd("/etc/vx/diag.d/vxscsi -g $disk");
        next if (EDR::cmdexit() != 0);
 
        $nsect=0;
        $nhead=0;
        $ncyl=0;
        if ($vxscsi=~/nsect=(\d+)\s*/){
            $nsect=$1;
        }
        if ($vxscsi=~/nhead=(\d+)\s*/){
            $nhead=$1;
        }
        if ($vxscsi=~/ncyl=(\d+)\s*/){
            $ncyl=$1;
        }
        $cap=$nsect * $nhead * $ncyl;
        next if ($cap==0);

        $cyl=int($cap / $ncyl);
        $rsize=$cap - $cyl - $cyl;
        $ncyl_lmt=int($sol_disklmt / $cyl);
        $tb_mkr=$ncyl_lmt * $cyl;
 
        # Block 0 label values
        $tmpfile="$tmpdir/".$sys->{sys}."_$disk"."_blk0.out";
        $sys->cmd("_cmd_dd if=$lpath of=$tmpfile bs=512 count=2 >/dev/null 2>&1");
        if ($sys->exists($tmpfile)) {
            $sec_msb=$sys->cmd("od -An -v -j 438 -N 1 -tuC $tmpfile 2>/dev/null");
            $sec_lsb=$sys->cmd("od -An -v -j 439 -N 1 -tuC $tmpfile 2>/dev/null");
            $sec=$sec_msb * 256 + $sec_lsb;
            $hd_msb=$sys->cmd("od -An -v -j 436 -N 1 -tuC $tmpfile 2>/dev/null");
            $hd_lsb=$sys->cmd("od -An -v -j 437 -N 1 -tuC $tmpfile 2>/dev/null");
            $hd=$hd_msb * 256 + $hd_lsb;
            $cyl_msb=$sys->cmd("od -An -v -j 432 -N 1 -tuC $tmpfile 2>/dev/null");
            $cyl_lsb=$sys->cmd("od -An -v -j 433 -N 1 -tuC $tmpfile 2>/dev/null");
            $cyl=$cyl_msb * 256 + $cyl_lsb;
            $cylsize=$sec * $hd;
            $alt_cyl=$sys->cmd("od -An -v -j 435 -N 1 -tuC $tmpfile 2>/dev/null");
            $tcyl=$cyl + $alt_cyl;
 
            # Compare Raw Geo With Block 0
            if (($sec != $nsect ) || ( $hd != $nhead ) || ( $tcyl != $ncyl ) || ($cap > $tb_mkr)) { 
                $disk_size2=$cyl * $cylsize;
 
                # Check if disk is a cdsdisk and compare VXVM disk size with Block 0 size
                $vxdisk=$sys->cmd("_cmd_vxdisk list $disk 2>/dev/null");
 
                $format='';
                $invalid='';
                $flag='';
                if ($vxdisk=~/format=(\w+)/g) {
                    $format=$1;
                }
                if ($vxdisk=~/flags:.*invalid.*/g) {
                    $invalid=1;
                }
                if ($vxdisk=~/flags:(.+)/g) {
                    $flags=$1;
                    @flags=split(/\s+/m, $flags);
                    $flag=$flags[1];
                }
                if (( $format eq "cdsdisk" ) && ( $flag eq "online" ) && ( $invalid != 1 )) {
                    if ($vxdisk=~/public:(.+)/g) {
                        $pub=$1;
                    }
                    if ($pub=~/offset=(\d+)/) {
                        $off=$1;
                    }
                    if ($pub=~/len=(\d+)/) {
                        $len=$1;
                    }
                    $disk_s=$off + $len;
                    if ($sys->{plat} eq "HPUX") {
                        $disk_s2=$disk_s * 1024;
                        $disk_size1=int($disk_s2 / 512);
                    } else {
                        $disk_size1=$disk_s;
                    }
                } else {
                    $disk_size1=0;
                }
 
                if (( $disk_size1 > $disk_size2 ) || ( $disk_size1 == 0 ) || ( $disk_size1 > $rsize ) || ($cap > $tb_mkr)) {
                    # Ensure Block 0 is not empty and has a valid label
                    if (( $sec == 0 ) && ( $hd == 0 ) && ( $tcyl == 0 ) && ( $flag !~ "error" )) {
                        if ( $format eq "cdsdisk" ) {
                            $label=$sys->cmd("_cmd_strings $tmpfile");
                            if ( $label =~ /EFI PART/g ) {
                                Msg::log("Disk $disk Passed - Labeled EFI/GPT");
                            } else {
                                Msg::log("Disk $disk -- FAILED --");
                                Msg::log("Block 0 does not contain any SMI (disk label) data, Need run 'vxdisk resize $disk' to update Block 0\n");
                                push @disks_with_issue, $disk;
                            }
                        } else {
                            if ( $format eq "hpdisk" ) {
                                Msg::log("$disk format $format - Passed");
                            } else {
                                Msg::log("$disk is not under VxVM Control");
                            }
                        }
                    } else {
                        $tdisk_size=$tcyl * $sec * $hd;
                        $secoff=$sec - 1;
                        $skip_value=$tdisk_size - $secoff;

                        # Gather the dd from the backup label location base on Block 0 values
                        if ( $disk_size1 > $skip_value) {
                            Msg::log("Disk has Smaller Label than VxVM DMSIZE");
                            Msg::log("--------$disk Lun Size's ---------");
                            if ( $format eq "cdsdisk" ) {
                                Msg::log("VxVM DM(disk media) Size = $disk_size1");
                            } else {
                                Msg::log("$disk is not under VxVM Control");
                            }
                            
                            if ($vxscsi=~/cap=(\d+)/) {
                                $maxcap=$1;
                            }
                            Msg::log("Raw Geometry Size - Max Cap=$maxcap Available Size = $rsize");
                            Msg::log("Block 0 Label Size = $disk_size2");
                            Msg::log("    ---- CYL=$tcyl SEC=$sec HD=$hd ----");
 
                            Msg::log("Disk $disk -- FAILED --");

                            # Check is the backup label Block contain backup label data search for "cyl"
                            $tmpfile2="$tmpdir/".$sys->{sys}."_$disk"."_bklbl.out";
                            $sys->cmd("_cmd_dd if=$lpath of=$tmpfile2 bs=512 count=16 skip=$skip_value >/dev/null 2>&1");
                            $cnt=$sys->cmd("_cmd_strings $tmpfile2 | _cmd_grep cyl | _cmd_head -1 | _cmd_wc -l | _cmd_awk '{print \$1}'");
                            if  ( $cnt == 1 ) {
                                Msg::log("Script has detected that Disk $disk has Block 0 data in user data region.");
                                Msg::log("Block 0 data was located on device at block offset $skip_value");
                            } else {
                                Msg::log("No Block 0 label data was detected in user data blocks");
                            }
                            push @disks_with_issue, $disk;
                        } else {
                            if (( $flag eq "error" ) || ( $invalid == 1 )) {
                                Msg::log("Disk $disk ---- FAILED");
                                push @disks_with_issue, $disk;
                            } else {
                                if ( $cap > $tb_mkr ) {
                                    Msg::log("Disk $disk ---- FAILED");
                                    Msg::log("Lun $disk greater than 1TB with an SMI label\n");
                                    push @disks_with_issue, $disk;
                                } else {
                                    Msg::log("Size Discrepancy Noted 'Disk $disk Passed'");
                                }
                            }
                        }
                    }
                } else {
                    Msg::log("Disk $disk Passed");
                }
           } else {
               Msg::log("Disk $disk Passed ");
           }
        } else {
            Msg::log("Unable to run dd on Drive $disk");
        }
    }
 
    if ( @disks_with_issue ) {
        $sys->set_value('donot_restart_vxconfigd', 1);

        $disks=join ', ', @disks_with_issue;
        $msg=Msg::new("There are the following disks over terabyte on $sys->{sys}:\n\t$disks\nInstaller will not restart the vxconfigd due to a defect in Volume Manager of 5.1SP1 version that may potentially cause diskgroup import failures on upgrade and lead to potential data loss.\nPlease refer to the technical note for more details 'http://www.symantec.com/business/support/index?page=content&id=TECH155921'");
        $msg->log;
        return 0;
    } else {
        Msg::log("All disk Passed no action required");
        Msg::log("No User Data can get corrupted.");
    }
    return 1;
}

sub dmp_osn_stop_sys {
    my ($prod,$sys,$task) = @_;
    my ($msg,$out,$voldmode);

    return 0 if (Cfg::opt('rootpath'));
    return 0 if ($sys->exists("$prod->{mkdbfile}"));

    $voldmode = $prod->vold_status_sys($sys);
    if ($voldmode !~ /enabled/m) {
        if ($sys->{donot_restart_vxconfigd}) {
            return '';
        } else {
            # If vold is down, attempt to start it
            $sys->cmd('_cmd_vxconfigd -k -r reset -x syslog 2> /dev/null');
            $voldmode = $prod->vold_status_sys($sys);
            if ($voldmode !~ /enabled/m ) {
                if (Cfg::opt('upgrade')) {
                    $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. You must make sure the tunable dmp_native_support is turned off on $sys->{sys} before upgrade. It is likely that vxdmp driver will fail to stop during upgrade if the tunable is on. If OS upgrade is involved during the upgrade process, you can ignore this warning.", 40, 4907, "$sys->{sys}", "$sys->{sys}");
                } else {
                    $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. You must affirm that dmp_native_support tunable is turned off or run 'vxdmpadm settune dmp_native_support=off' manually after restarting vxconfigd if you want to continue.", 40, 4496, "$sys->{sys}");
                }
                $sys->push_warning($msg);
                return 0;
            }
        }
    }

    $out = $sys->cmd('_cmd_vxdmpadm gettune dmp_native_support 2> /dev/null');
    if ($out =~ /dmp_native_support\s+on/mx) {
        Msg::log("Detected OS native stack support is enabled on $sys->{sys} before $task, turn it off...");
        $out = $sys->cmd('_cmd_vxdmpadm settune dmp_native_support=off');
        if (EDR::cmdexit()) {
            if ($out =~ /\brootvg\b/m) {
                # error for AIX LVM
                $msg = Msg::new("Failed to turn off dmp_native_support tunable on $sys->{sys}. Refer to Dynamic Multi-Pathing Administrator's guide to determine the reason for the failure and take corrective action. Re-run bosboot manually.", 40, 4497, "$sys->{sys}");
            } else {
                # other error
                $msg = Msg::new("Failed to turn off dmp_native_support tunable on $sys->{sys}. Refer to Dynamic Multi-Pathing Administrator's guide to determine the reason for the failure and take corrective action.", 40, 4498, "$sys->{sys}");
            }
            $msg->{msg} .= "\n$out";
            $sys->push_error($msg);
        } else {
            if ($out =~ /\breboot\b/im) {
                if ($task eq 'upgrade') {
                    $msg = Msg::new("System $sys->{sys} needs reboot to disable dmp native support on $sys->{sys}. Re-run the upgrade task after reboot.", 40, 4499, "$sys->{sys}", "$sys->{sys}");
                } elsif ($task eq 'uninstall') {
                    $msg = Msg::new("System $sys->{sys} needs reboot to disable dmp native support on $sys->{sys}. Re-run the uninstall task after reboot.", 40, 4500, "$sys->{sys}", "$sys->{sys}");
                } elsif ($task eq 'stop') {
                    $msg = Msg::new("System $sys->{sys} needs reboot to disable dmp native support on $sys->{sys}. Re-run the stop task after reboot.", 40, 4501, "$sys->{sys}", "$sys->{sys}");
                } elsif ($task eq 'rollback') {
                    $msg = Msg::new("System $sys->{sys} needs reboot to disable dmp native support on $sys->{sys}. Re-run the rollback task after reboot.", 40, 4906, "$sys->{sys}", "$sys->{sys}");
                }
                $sys->push_warning($msg);
                $sys->set_value('reboot', 1);
                $sys->set_value('warning_exit', 1);
            }
        }
        # touch /etc/vx/.migrate_native to turn on dmp native support later
        $sys->cmd("_cmd_touch $prod->{migrate_native_file}") if (Cfg::opt(qw(upgrade patchupgrade stop)));
    }
    return 1;
}

sub vvr_upgrade_precheck_sys {
    my ($prod,$sys)= @_;
    my ($dg,@dgs,$msg,$rvg,$shared,$voldmode,$vvr);
    # check vxdctl mode
    # Follow upgrade procedure if VRTSvxvm is being upgraded
    # But just check whether the sys is being upgraded.
    return if ($sys->exists("$prod->{mkdbfile}"));
    $voldmode = $sys->cmd('_cmd_vxdctl mode 2> /dev/null');
    if ($voldmode !~ /mode: enabled/m ) {
        if ($sys->{donot_restart_vxconfigd}) {
            return '';
        } else {
            # If vold is down, atempt to start it
            $sys->cmd('_cmd_vxconfigd -k -r reset -x syslog 2> /dev/null');
            $voldmode = $sys->cmd('_cmd_vxdctl mode 2> /dev/null');
            if ($voldmode !~ /mode: enabled/m ) {
                $msg=Msg::new("vxconfigd could not be started. Upgrading VRTSvxvm in this scenario can result in configuration errors, or data loss for Veritas Volume Manager objects. Continue if you are sure that there are no Veritas Volume Manager objects on the host.", 40, 2652);
                $sys->push_warning($msg);
                return '';
            }
        }
    }

    # check dg version
    # Iterate twice
    $dg=$sys->cmd("_cmd_vxdg -q list 2>/dev/null | _cmd_awk '{ print \$1 }'");
    @dgs=split(/\s+/m,$dg);
    for my $dg (@dgs) {
        $rvg = $sys->cmd("_cmd_vxprint -g $dg -qQVF%name 2>/dev/null");
        if ($rvg) {
            $vvr = 1;
            $sys->set_value('rvg','1');
            last;
        }
    }
    return unless ($vvr);
    for my $dg (@dgs) {
        $shared = $sys->cmd("_cmd_vxdg list $dg 2> /dev/null | _cmd_grep 'flags:' | _cmd_grep shared");
        next if ($shared);
        $prod->vvr_check_dg_version_sys($sys, $dg);
    }
    return;
}

1;