#HOTFIX_VERSION:cpi-6.0RP1P2
package CPIC;

sub prestart_config {
    my($ayn,$cfg,$cpic,$edr,$prod,$sys,$reboot_systems);
    $cpic=shift;
    $prod=$cpic->prod;
    $edr=Obj::edr();
    $cfg=Obj::cfg();

    if (Cfg::opt('install')) {
        $reboot_systems=$cpic->systems_need_reboot();
        if (Cfg::opt('rootpath') || @{$reboot_systems}) {
            $cpic->printmsg_skip_setting_tunables('reboot',$prod);
            $cpic->completion();
        }
        if ($prod->disableconfig()) {
            $cpic->printmsg_skip_setting_tunables('disableconfig',$prod);
            $cpic->completion();
        }
        if ($prod->hasconfig()) {
            if ($cpic->ask_configure()) {
                $cpic->set_task('configure');
            } else {
                $cpic->printmsg_skip_setting_tunables('skipconfig',$prod);
                $cpic->completion();
            }
        }
        $cpic->set_task('configure');
        $cpic->{installconfig}=1;
    }

    Cfg::set_opt('silent') if ($prod->{silent_installconfig});

    $cpic->validate_tunables();
    $cpic->set_tunables_any();
    $cpic->set_tunables_prestart();

    $cpic->{prod_running}=1;
    for my $sys (@{$cpic->{systems}}) {
        unless ($sys->{prod_running}) {
            $cpic->{prod_running}='';
            last;
        }
    }
    $cpic->{reuse_config}=$cpic->{prod_running};

    if (Cfg::opt('responsefile')) {
        $cpic->{reuse_config}=$cfg->{reuse_config};
        $prod->responsefile_prestart_config() unless (Cfg::opt(qw(patchupgrade rollback)) && $cfg->{reuse_config});
        if (Cfg::opt(qw(patchupgrade rollback))) {
            # e2784341: complete responsefile patch upgrade if no configuration task is set
            $cpic->completion() unless ($cpic->{reuse_config} || Cfg::opt('patchconfigure'));
        }
    }
    return '' if (Cfg::opt('responsefile') && ($edr->{exitfile} eq ''));

    if (Cfg::opt(qw(patchupgrade rollback))) {
        $cfg->{reuse_config} = $cpic->{prod_running};
        return '' if ($cpic->{prod_running});
        return '' unless (($prod->can('cli_prestart_config_questions')) || ($prod->can('web_prestart_config_questions')));
        ($cpic->ask_patchupgrade_configure($prod)) ? Cfg::set_opt('patchconfigure',1) : return '';
    }

    $prod->cli_prestart_config_questions() if
        ($prod->can('cli_prestart_config_questions'));
    $cpic->web_prestart_config($prod);
    return;
}

package Prod::VCS60::Common;
sub upgrade_preinstall_sys {
    my ($prod,$sys) = @_;
    my ($rootpath,$initfile_content);
    return unless (Cfg::opt(qw(patchupgrade)));
    $rootpath = Cfg::opt('rootpath');

    # backup init files for llt/gab/vxfen/vcs/amf
    if ( ! $rootpath ) {
        $rootpath = '';
    }
    $initfile_content = {};
    for my $initfile (keys %{$prod->{initfile}}) {
        if (!$sys->exists($rootpath.$prod->{initfile}{$initfile})) {
            Msg::log('No '.$rootpath.$prod->{initfile}{$initfile}.' to backup');
            next;
        }
        Msg::log('Found '.$rootpath.$prod->{initfile}{$initfile}.' to backup');
        for my $line (split(/\n/, $sys->readfile($rootpath.$prod->{initfile}{$initfile}))) {
            if ( $line =~ /^\s*([a-zA-Z0-9_]+)=(.*)$/mx ) {
                $initfile_content->{$initfile}{$1}=$2;

            }
        }
    }
    $sys->set_value('initfile_content', JSON::to_json($initfile_content));

    return;
}

package Prod::VM60::Common;
@Prod::VM60::Common::ISA = qw(Prod);

sub upgrade_precheck_sys {
    my ($prod,$sys)= @_;
    my ($pkg,$cprod);

    #fix e1518138 : uninstall AT pkgs when upgrade VM/VVR
    $cprod=CPIC::get('prod');
    if (EDRu::inarr($cprod,qw/VM60 DMP60/)) {
        $prod->obsoleted_bundles_sys($sys);
        $prod->obsoleted_bundled_pkgs_sys($sys);
    }
    $prod->obsoleted_thirdparty_pkgs_sys($sys);
    # check if dg has latest backup before upgrade
    $prod->dg_conf_upgrade_precheck_sys($sys) unless (Cfg::opt('upgrade_nonkernelpkgs'));
    if ($prod->is_encapsulated_bootdisk_sys($sys)){
        return '' unless ($prod->encap_upgrade_precheck_sys($sys));
        # set donotstop and donotremove flag here.
        $prod->set_flag_encap_upgrade_sys($sys);
    }
    $prod->open_volumes_sys($sys, 'upgrade') unless (Cfg::opt(qw(upgrade_kernelpkgs upgrade_nonkernelpkgs)));
    $prod->vvr_upgrade_precheck_sys($sys) if ($prod->vr_licensed_sys($sys));
    return;
}

sub patchupgrade_precheck_sys {
    my ($prod,$sys)= @_;
 
    $prod->obsoleted_bundles_sys($sys);
    $prod->obsoleted_bundled_pkgs_sys($sys);
 
    $prod->dg_conf_upgrade_precheck_sys($sys) unless (Cfg::opt('upgrade_nonkernelpkgs'));
    if ($prod->is_encapsulated_bootdisk_sys($sys)) {
        return '' unless ($prod->encap_upgrade_precheck_sys($sys));
        # set donotstop and donotremove flag here.
        $prod->set_flag_encap_upgrade_sys($sys);
    }
    $prod->open_volumes_sys($sys, 'upgrade') unless (Cfg::opt(qw(upgrade_kernelpkgs upgrade_nonkernelpkgs)));
    return;
}

sub dg_conf_upgrade_precheck_sys {
    my ($prod,$sys) = @_;
    my ($cf_bkup_path,$cmd,$dg,$dgid,$dglist,$msg,$rootpath,$voldmode);
    return if ($sys->exists("$prod->{mkdbfile}"));
    return if ($prod->vol_is_vm_disabled_sys($sys));
    $voldmode = $prod->vold_status_sys($sys);
    if ($voldmode !~ /enabled/m ) {
        # If vold is down, atempt to start it
        # firstly try to start without '-r reset' since '-r reset' will increase dg seq no
        if ($voldmode =~ /not-running/m) {
            Msg::log("vxconfigd is not running on $sys->{sys}; restarting");
            $sys->cmd('_cmd_vxconfigd -k -x syslog 2> /dev/null');
        } else {
            # the mode could be disabled or booted
            Msg::log("vxconfigd is in $voldmode on $sys->{sys}; re-enabling");
            $sys->cmd('_cmd_vxdctl enable >/dev/null 2>&1');
        }
        $voldmode = $prod->vold_status_sys($sys);
        if ($voldmode !~ /enabled/m ) { 
            $sys->cmd('_cmd_vxconfigd -k -r reset -x syslog 2> /dev/null');
            $voldmode = $prod->vold_status_sys($sys);
            if ($voldmode !~ /enabled/m ) {
                $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. Upgrading VRTSvxvm in this scenario can result in configuration errors, or data loss for Veritas Volume Manager objects. Continue if you are sure that there are no Veritas Volume Manager objects on the host.");
                $sys->push_warning($msg);
                return '';
            }
        }
    }
    $dglist = $sys->cmd("_cmd_vxdg -q list 2> /dev/null");
    for my $line(split(/\n/,$dglist)) {
        if ($line =~ /(\S+)\s+\S+\s+(\S+)/m) {
            $dg = $1;
        } else {
            next;
        }

        # For 2808458: to check if any imported dg with mix of regular and clone disks
        if ($prod->is_dg_mix_clone_sys($sys,$dg)) {
            $sys->set_value('dgs_mix_clone','push',$dg);
        }
    }

    if ($sys->{dgs_mix_clone}) {
        $dglist = join(" ", @{$sys->{dgs_mix_clone}});
        $msg = Msg::new("The following disk group(s) have mix of regular and clone disk on $sys->{sys}:\n\t$dglist\nplease verify disk information and unset clone flag before upgrade.");
        $sys->push_error($msg);
    }
    return;    
}

# To check if the dg is mix of regular and clone disks
sub is_dg_mix_clone_sys {
    my ($prod,$sys,$dg) = @_;
    my ($disks,$disknum,$clonedisknum);

    $disknum=0;
    $clonedisknum=0;
    $disks = $sys->cmd("_cmd_vxdisk list -q -g $dg 2>/dev/null");
    for my $disk (split(/\n/, $disks)) {
        $disknum++;
        $clonedisknum ++ if ($disk =~ /clone_disk/m)
    }

    return 1 if ($clonedisknum && $clonedisknum != $disknum);
    return 0;
}

1;