#HOTFIX_VERSION:cpi-Patch-6.2.1.400
package Prod;

sub cli_print_requirements {
    my ($prod) = @_;
    my ($otherreqs,$prodspace,$minspace,$rel,$msg,$platreqs,$platreqs_str,$ps,$ms,$recspace,@space,$cs,$prodname,$pkg,$pkgspace);
    $rel=$prod->rel;

    $rel->pre_platvers();
    $prodname = $prod->{name};
    $msg=Msg::new("The requirements for product '$prodname':\n");
    $msg->print;

    $platreqs = $rel->get_platreqs($prod);
    if (defined $platreqs) {
        $platreqs_str=join("\n",@{$platreqs});
        $msg=Msg::new("Required OS/Kernel Versions:\n$platreqs_str");
        $msg->print;
    }

    $prod->cli_print_os_dependency();

    if ($prod->{minimal_memory_requirment}) {
        $msg=Msg::new("\nMinimal Memory Required: $prod->{minimal_memory_requirment}");
        $msg->print;
    }

    # Determine the space requirement for the product
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$prod->pkg($pkgi);
        $pkg->{space} = $pkg->space();
        $pkg->{space}||=[100,100,100,100];
        @space=@{$pkg->{space}};
        $pkgspace=$space[0]+$space[1]+$space[2]+$space[3];
        $prodspace+=$pkgspace;
        if (EDRu::inarr($pkgi,@{$prod->minpkgs})) {
            $minspace+=$pkgspace;
        }
        if (EDRu::inarr($pkgi,@{$prod->recpkgs})) {
            $recspace+=$pkgspace;
        }
    }
    $ms=int($minspace/1024);
    $cs=int($recspace/1024);
    $ps=int($prodspace/1024);
    $msg=Msg::new("\nSpace Requirements:\n$prod->{abbr} Minimum Package Set - $ms MB\n$prod->{abbr} Recommended Package Set - $cs MB\n$prod->{abbr} All Package Set - $ps MB");
    $msg->print;

    $otherreqs=$prod->other_requirements||'';
    unless ($otherreqs){
        $otherreqs = Msg::new("None")->{msg};
    }

    $msg=Msg::new("\nOther Requirements:\n$otherreqs");
    $msg->print;

    return '';
}

package Prod::VCS62::Sol11sparc;

sub upgrade_preinstall_sys {
    my ($prod,$sys)=@_;

    return if (Cfg::opt('rootpath'));

    for my $proc (qw(vxfen gab llt)) {
        if ($sys->padv->driver_sys($sys,$proc)) {
            $sys->padv->unload_driver_sys($sys,$proc,1);
            sleep 2;
        }
    }

    return;
}

package Prod::VCS62::Common;

sub maincf_upgrade_cvm_cfs_types_sys {
    my ($prod,$sys,$treeref_olduser,$vcsvers) = @_;
    my ($attr_node,$cfsmount_attr,$type_node,$attr_name,$type_name,$rtn,$cvmvoldg_reglist,$cvmvoldg_arglist);
    my ($cvmcluster_reglist,$cvmcluster_attr,$cvmcluster_arglist);

    # Update CFSMount
    # Add AMFMountType as a attr to CFSMount in 5.1SP1
    $type_name = 'CFSMount';
    if ($type_node = dynupgrade_get_node_from_tree( $treeref_olduser, 'TYPE', $type_name)) {
        $attr_name = 'AMFMountType';
        $attr_node = dynupgrade_get_node_from_tree( $type_node, 'ATTR', $attr_name);
        # do nothing if AMFMountType already exist
        unless ($attr_node) {
            $cfsmount_attr = 'haattr -add -temp CFSMount AMFMountType -string';
            $sys->set_value('maincf_upgrade,10_cfsmount_add', $cfsmount_attr);
        }
    }

    # Update CFSMount
    # Add Primary to ArgList in CFSMount in 5.1
    # Add AMFMountType to ArgList in CFSMount in 5.1SP1
    $rtn = dynupgrade_get_attrvalue_of_typename($treeref_olduser,'CFSMount','ArgList');
    if ($rtn) {
        $rtn .= ' Primary' if ($rtn!~/Primary/m);
        $rtn .= ' AMFMountType' if ($rtn!~/AMFMountType/m);
        $cfsmount_attr="hatype -modify CFSMount ArgList $rtn";
        $sys->set_value('maincf_upgrade,10_cfsmount_attr', $cfsmount_attr);
    }

    # Update CVMVolDg
    # Add CVMVolume to RegList in CVMVolDg in 5.1
    $rtn = dynupgrade_get_attrvalue_of_typename($treeref_olduser,'CVMVolDg','RegList');
    if ($rtn) {
        $rtn .= ' CVMVolume' if ($rtn!~/CVMVolume/m);
        $cvmvoldg_reglist="hatype -modify CVMVolDg RegList $rtn";
        $sys->set_value('maincf_upgrade,11_cvmvoldg_reglist', $cvmvoldg_reglist);
    }
    # Add CVMVolumeIoTest, CVMDGAction to ArgList in CVMVolDg in 5.1
    # Add CVMDeportOnOffline to ArgList in CVMVolDg in 5.1SP1
    # Add CVMDeactivateOnOffline to ArgList in CVMVolDg in 6.0.1
    $rtn = dynupgrade_get_attrvalue_of_typename($treeref_olduser,'CVMVolDg','ArgList');
    if ($rtn) {
        $rtn .= ' CVMVolumeIoTest' if ($rtn!~/CVMVolumeIoTest/m);
        $rtn .= ' CVMDGAction' if ($rtn!~/CVMDGAction/m);
        $rtn .= ' CVMDeportOnOffline' if ($rtn!~/CVMDeportOnOffline/m);
        $rtn .= ' CVMDeactivateOnOffline' if ($rtn!~/CVMDeactivateOnOffline/m);
        $rtn .= ' State' if ($rtn!~/State/m);
        $rtn .= ' ClearClone' if ($rtn!~/ClearClone/m);
        $cvmvoldg_arglist="hatype -modify CVMVolDg ArgList $rtn\n";
        $cvmvoldg_arglist.="haattr -add CVMVolDg CVMVolumeIoTest -keylist\n";
        $cvmvoldg_arglist.="hatype -modify CVMVolDg OnlineRetryLimit 2\n";
        $cvmvoldg_arglist.="hatype -modify CVMVolDg OnlineTimeout 400\n";
        $cvmvoldg_arglist.="haattr -add CVMVolDg CVMDGAction -string\n";
        $cvmvoldg_arglist.="haattr -add CVMVolDg CVMDeportOnOffline -integer\n";
        $cvmvoldg_arglist.="haattr -add CVMVolDg CVMDeactivateOnOffline -integer\n";
        $cvmvoldg_arglist.="haattr -add CVMVolDg ClearClone -integer";
        $sys->set_value('maincf_upgrade,11_cvmvoldg_arglist', $cvmvoldg_arglist);
    }

    # Update CVMCluster
    $type_name = 'CVMCluster';
    if ($type_node = dynupgrade_get_node_from_tree( $treeref_olduser, 'TYPE', $type_name)) {
        # Add CVMNodePreference as an attr to CVMCluster in 6.0.1
        $attr_name = 'CVMNodePreference';
        $attr_node = dynupgrade_get_node_from_tree( $type_node, 'ATTR', $attr_name);
        # do nothing if CVMNodePreference already exist
        $cvmcluster_attr.="haattr -add CVMCluster CVMNodePreference -string\n" unless ($attr_node);

        # Add CVMClus_storage_connectivity as an attr to CVMCluster in 6.0.1
        $attr_name = 'CVMClus_storage_connectivity';
        $attr_node = dynupgrade_get_node_from_tree( $type_node, 'ATTR', $attr_name);
        # do nothing if CVMClus_storage_connectivity already exist
        $cvmcluster_attr.='haattr -add CVMCluster CVMClus_storage_connectivity -keylist' unless ($attr_node);
        $sys->set_value('maincf_upgrade,12_cvmcluster_add', $cvmcluster_attr) if ($cvmcluster_attr);

        # Add CVMNodePreference to RegList in 6.0.1
        # Add CVMClus_storage_connectivity to RegList and ArgList in 6.0.1
        $rtn = dynupgrade_get_attrvalue_of_typename($treeref_olduser,'CVMCluster','RegList');
        if ($rtn) {
            $rtn .= ' CVMNodePreference' if ($rtn!~/CVMNodePreference/m);
            $rtn .= ' CVMClus_storage_connectivity' if ($rtn!~/CVMClus_storage_connectivity/m);
            $cvmcluster_reglist="hatype -modify CVMCluster RegList $rtn";
            $sys->set_value('maincf_upgrade,12_cvmcluster_reglist', $cvmcluster_reglist);
        } else {
            # should not add this line if no CVMTypes
            $cvmcluster_reglist='haattr -add -static CVMCluster RegList  -keylist CVMNodePreference CVMClus_storage_connectivity';
            $sys->set_value('maincf_upgrade,12_cvmcluster_reglist', $cvmcluster_reglist);
        }
        $rtn = dynupgrade_get_attrvalue_of_typename($treeref_olduser,'CVMCluster','ArgList');
        if (($rtn) && ($rtn!~/CVMClus_storage_connectivity/m)) {
            $rtn .= ' CVMClus_storage_connectivity';
            $cvmcluster_arglist="hatype -modify CVMCluster ArgList $rtn";
            $sys->set_value('maincf_upgrade,12_cvmcluster_arglist', $cvmcluster_arglist);
        }
    }

    return;
}

package Prod::VM62::Common;

sub ask_upgrade_configure {
    my ($prod) = @_;
    my ($msg,$task,$ayn,$cfg,$syslist,@dmp_enabled_syslist,$out,$help,$sysobj,$sunos,$sys0);
    $cfg=Obj::cfg();
    return if ($cfg->{opt}{upgrade_nonkernelpkgs});
    return if ($cfg->{opt}{rootpath}); # for live upgrade, don't check dmp_native_support

    $syslist=CPIC::get('systems');
    $sys0=${$syslist}[0];
    # On AIX platform, we do NOT turn off dmp_native_support before rolling upgrade phase 1, no need to ask for retain
    return if(($sys0->{plat} eq 'AIX') && Cfg::opt("upgrade_kernelpkgs") && $sys0->{pkgvers}{VRTScavf});
    # for Solaris 11 on 6.2.1, we just perform in-place patch upgrade for VRTSvxvm,
    # so it's not necessary to turn off 'dmp_native_support' before patch upgrade, and turn on after upgrade
    return if (($sys0->{padv} =~ /^Sol11/m) && Cfg::opt('patchupgrade'));
    $sunos=1 if($sys0->sunos());
    if ($cfg->{opt}{makeresponsefile}) {
        for my $sys(@$syslist) {
            $out=$sys->cmd('_cmd_vxdmpadm gettune dmp_native_support 2> /dev/null');
            push (@dmp_enabled_syslist,$sys->{sys}) if($out =~ /dmp_native_support\s+on/mx);
        }
    } else {
        for my $sys(@$syslist) {
            push (@dmp_enabled_syslist,$sys->{sys}) if($sys->exists($prod->{migrate_native_file}));
        }
    }

    $syslist=join(',',@dmp_enabled_syslist);
    return unless $syslist;

    if ($cfg->{opt}{responsefile} || (defined $cfg->{opt}{disable_dmp_native_support})) {
        if ($cfg->{opt}{disable_dmp_native_support}==1) {
            for my $sys(@dmp_enabled_syslist) {
                $sysobj=Obj::sys($sys);
                $sysobj->cmd("_cmd_rmr $prod->{migrate_native_file} 2> /dev/null");
            }
        }
        return;
    }
    $task=Cfg::opt("rollback") ? Msg::new("rollback")->{msg} : Msg::new("upgrade")->{msg};
    # dmp_native_support is previously enabled
    $msg=Msg::new("Tunable dmp_native_support is turned 'on' on the following system(s): $syslist");
    $msg->print();
    if ($sunos) {
        $msg=Msg::new("Retaining this tunable setting may increase the $task time depending on the number of LUNs and native ZFS pools configured on the system.");
    } else {
        $msg=Msg::new("Retaining this tunable setting may increase the $task time depending on the number of LUNs and native LVM volume groups configured on the system.");
    }
    $msg->print();
    $msg=Msg::new("Would you like to retain the tunable setting?");
    $help=Msg::new("When dmp_native_support tunable is turned on, DMP enables support for LVM volume groups/ZFS pools on any device that does not have a VxVM label and is not in control of any third party multi-pathing (TPD) software. In addition, turning on the dmp_native_support tunable migrates any LVM volume groups/ZFS pools that are not in use onto DMP devices.");
    $ayn=$msg->aynn($help);
    if ($ayn eq 'Y') {
        $cfg->{opt}{disable_dmp_native_support}=0;
    } else {
        $cfg->{opt}{disable_dmp_native_support}=1;
        for my $sys(@dmp_enabled_syslist) {
            $sysobj=Obj::sys($sys);
            $sysobj->cmd("_cmd_rmr $prod->{migrate_native_file} 2> /dev/null");
        }
    }
    return;
}

sub patchupgrade_precheck_sys {
    my ($prod,$sys)= @_;

    if ($prod->is_encapsulated_bootdisk_sys($sys)) {
        return '' unless ($prod->encap_upgrade_precheck_sys($sys));
        # set donotstop and donotremove flag here.
        $prod->set_flag_encap_upgrade_sys($sys);
    }
    # check if any cache objects enabled on the system
    $prod->cache_areas_sys($sys) unless (Cfg::opt(qw(upgrade_nonkernelpkgs)));
    $prod->open_volumes_sys($sys, 'upgrade') unless (Cfg::opt(qw(upgrade_kernelpkgs upgrade_nonkernelpkgs)));

    # for Solaris 11 on 6.2.1, we just perform in-place patch upgrade for VRTSvxvm,
    # so it's not necessary to turn off 'dmp_native_support' before patch upgrade, and turn on after upgrade
    # here, skip stopping VxVM/VxFS processes.
    $prod->dmp_native_support_check_sys($sys) unless (Cfg::opt(qw(upgrade_nonkernelpkgs rootpath)));

    return;
}

sub dmp_native_support_check_sys {
    my ($prod,$sys)= @_;
    my ($out,$pkg,$msg);

    return unless (($sys->{padv} =~ /^Sol11/m) && Cfg::opt('patchupgrade'));

    $out=$sys->cmd('_cmd_vxdmpadm gettune dmp_native_support 2> /dev/null');
    return unless ($out =~ /dmp_native_support\s+on/mx);

    $pkg=$sys->pkg('VRTSvxvm62');
    $pkg->set_value('stopprocs',undef);
    $pkg=$sys->pkg('VRTSvxfs62');
    $pkg->set_value('stopprocs',undef);
    $sys->set_value('reboot', 1);

    $msg=Msg::new("Tunable dmp_native_support is turned 'on' on $sys->{sys}, patch upgrade can continue, but the processes and drivers for VRTSvxvm and VRTSvxfs packages will not be stopped or unloaded.");
    $sys->push_warning($msg);

    return;
}

sub dmp_osn_stop_sys {
    my ($prod,$sys,$task) = @_;
    my ($msg,$out,$voldmode);
    my ($exitcode);

    return 0 if (Cfg::opt('rootpath'));
    return 0 if ($sys->exists("$prod->{mkdbfile}"));
    return 0 if (($sys->{padv} =~ /^Sol11/m) && Cfg::opt('patchupgrade'));

    $voldmode = $prod->vold_status_sys($sys);
    if ($voldmode =~ /not-running/m) {
        if ($sys->{donot_restart_vxconfigd}) {
            return 0;
        } else {
            # If vold is down, attempt to start it
            $sys->cmd('_cmd_vxconfigd -k -r reset -x syslog 2> /dev/null');
            $voldmode = $prod->vold_status_sys($sys);
            if ($voldmode =~ /not-running/m ) {
                if (Cfg::opt('upgrade')) {
                    $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. You must make sure the tunable dmp_native_support is turned off on $sys->{sys} before upgrade. It is likely that vxdmp driver will fail to stop during upgrade if the tunable is on. If OS upgrade is involved during the upgrade process, you can ignore this warning.");
                } else {
                    $msg=Msg::new("vxconfigd could not be started on $sys->{sys}. You must affirm that dmp_native_support tunable is turned off or run 'vxdmpadm settune dmp_native_support=off' manually after restarting vxconfigd if you want to continue.");
                }
                $sys->push_warning($msg);
                return 0;
            }
        }
    }

    $out = $sys->cmd('_cmd_vxdmpadm gettune dmp_native_support 2> /dev/null');
    if ($out =~ /dmp_native_support\s+on/mx) {
        Msg::log("Detected OS native stack support is enabled on $sys->{sys} before $task, turn it off...");
        $out = $sys->cmd('_cmd_vxdmpadm settune dmp_native_support=off 2>&1');
        $exitcode = EDR::cmdexit();
        # e2851114: While making dmp_native_support off on Linux, it returns with value 215 (VE_REBOOT_REQUIRED 471). Should regard return value of 215 is successful on Linux.
        $exitcode = 0 if ($exitcode eq '215');
        if ($exitcode) {
            if ($out =~ /\brootvg\b/m) {
                # error for AIX LVM
                $msg = Msg::new("Failed to turn off dmp_native_support tunable on $sys->{sys}. Refer to Dynamic Multi-Pathing Administrator's guide to determine the reason for the failure and take corrective action. Re-run bosboot manually.");
            } else {
                # other error
                $msg = Msg::new("Failed to turn off dmp_native_support tunable on $sys->{sys}. Refer to Dynamic Multi-Pathing Administrator's guide to determine the reason for the failure and take corrective action.");
            }
            $msg->{msg} .= "\n$out";
            $sys->push_error($msg);
        } else {
            if ($out =~ /\breboot\b/im) {
                $msg = Msg::new("System $sys->{sys} needs reboot to disable dmp native support on $sys->{sys}. Re-run the installer after a reboot.");
                $sys->push_warning($msg);
                $sys->set_value('reboot', 1);
                $sys->set_value('warning_exit', 1);
            }
        }
        # touch /etc/vx/.migrate_native to turn on dmp native support later
        $sys->cmd("_cmd_touch $prod->{migrate_native_file}") if (Cfg::opt(qw(upgrade patchupgrade hotfixupgrade stop rollback)));
    } else {
        # It is possible that dmp_native_support tunnable is off
        # but rootvg is still under DMP control
        if ($sys->aix()) {
            $out = $sys->cmd('_cmd_vxdmpadm native list vgname=rootvg 2>&1');
            if (EDR::cmdexit() == 0 || $out !~ /ERROR/) {
                # rootvg is still under DMP control
                $out=$sys->cmd('_cmd_vxdmpadm native disable vgname=rootvg 2>&1');
                if($out =~ /\breboot\b/im) {
                    $msg=Msg::new("System $sys->{sys} needs reboot to disable dmp root support on $sys->{sys}. Re-run the installer after a reboot.");
                    $sys->push_warning($msg);
                    $sys->set_value('reboot', 1);
                    $sys->set_value('warning_exit', 1);
                }
            }
        }
    }

    return 1;
}

sub completion_messages {
    my $prod=shift;
    my ($cpic,$msg,$padv,$sys,$tmppath,$rootpath,$web,$alert_msg);
    my (@ssd_devices,$ssd_num,$ssd_list,$ssd_msg);
    my ($aslapm,$aslapmver,$out,@dmp_enabled_syslist,$syslist,$sys0);

    $cpic = Obj::cpic();
    $padv=$prod->padv();
    $tmppath=EDR::get('tmppath');
    $rootpath = Cfg::opt('rootpath');
    $web=Obj::web();
    $sys0=${CPIC::get('systems')}[0];
    if (Cfg::opt(qw/upgrade patchupgrade hotfixupgrade install configure/) && !Cfg::opt(qw(upgrade_kernelpkgs security securityonenode securitytrust fips fencing configcps))) {
        $msg=Msg::new("The updates to VRTSaslapm package are released via the Symantec SORT web page: https://sort.symantec.com/asl. To make sure you have the latest version of VRTSaslapm (for up to date ASLs and APMs), download and install the latest package from the SORT web page.");
        $alert_msg=$msg->{msg};

        # To check SSD devices on the system
        $ssd_msg='';
        for my $sys (@{CPIC::get('systems')}) {
            if ($prod->ssd_devices_sys($sys)) {
                next unless (defined $sys->{ssd_devices});

                # only show the first 10 SSD devices if number > 10.
                @ssd_devices=@{$sys->{ssd_devices}};
                $ssd_num=scalar @ssd_devices;
                if ($ssd_num > 10) {
                    $ssd_list=join ' ', @ssd_devices[0..9];
                    $ssd_list.=' ...';
                } else {
                    $ssd_list=join ' ', @ssd_devices;
                }
                $msg=Msg::new("The following $ssd_num SSD devices have been detected on system $sys->{sys}:\n\t$ssd_list");
                $ssd_msg.="\n" . $msg->{msg} . "\n";
            }
        }

        #don't prompt message for DMP
        if ($ssd_msg && $prod->{class} !~ /DMP/m) {
            $msg = Msg::new("It is strongly recommended to use the SmartIO feature to take advantage of these SSD devices to accelerate I/O performance. See the Storage Foundation and High Availability Solutions documentation for more details about this SmartIO feature.");
            $ssd_msg.="\n" . $msg->{msg};
            $alert_msg.="\n" . $ssd_msg;
        }
        Msg::printn($alert_msg);
        $alert_msg=~s/\n/\\n/g;
        $web->web_script_form('alert',$alert_msg) if (Obj::webui());
    }

    if (Cfg::opt("rollback")) {
        # Check on each system, if VRTSaslapm has version higher than/equal to 6.2.1, then print the message.
        # during rollback, inform user VRTSaslapm will not change
        for $sys(@{$cpic->{systems}}) {
            $aslapm=$sys->pkg('VRTSaslapm62');
            $aslapmver=$aslapm->version_sys($sys) if($aslapm);
            if ($aslapmver && EDRu::compvers($aslapmver,$aslapm->{vers})<2) {
                $msg = Msg::new("VRTSaslapm package will not revert to older version during rollback process, if you insist on using VRTSaslapm package with older version, uninstall and reinstall VRTSaslapm package after rollback.");
                $msg->printn();
                $web->web_script_form('alert',$msg) if (Obj::webui());
                last;
            }
        }
    }

    if (Cfg::opt('rootpath')) {
        for my $sys (@{CPIC::get('systems')}) {
            if ($sys->{rvg}) {
                $msg = Msg::new("\nInstaller has detected that VVR is configured on the systems. Follow the steps below to upgrade VVR:");
                $msg->print;
                if (EDRu::compvers($sys->{platvers},'5.11')<2) {
                    $msg = Msg::new("    Step 1 - Execute vvr_upgrade_lu_start on all nodes. If it is a CVM-VVR environment, execute this script from the CVM master node");
                    $msg->print;
                    $msg = Msg::new("    Step 2 - Execute '$padv->{cmd}{shutdown}' to properly restart your systems");
                    $msg->print;
                    $msg = Msg::new("    Step 3 - Execute vvr_upgrade_lu_finish on all nodes. If it is a CVM-VVR environment, execute this script from the CVM master node\n");
                    $msg->print;
                }
                else {
                    $msg = Msg::new("    Step 1 - Execute vxlufinish on all the systems");
                    $msg->print;
                    $msg = Msg::new("    Step 2 - Execute vvr_upgrade_lu_start on all nodes. If it is a CVM-VVR environment, execute this script from the CVM master node");
                    $msg->print;
                    $msg = Msg::new("    Step 3 - Execute '$padv->{cmd}{shutdown}' to properly restart your systems");
                    $msg->print;
                    $msg = Msg::new("    Step 4 - Execute vvr_upgrade_lu_finish on all nodes. If it is a CVM-VVR environment, execute this script from the CVM master node\n");
                    $msg->print;
                }
                last;
            }
        }
    }

    if (Cfg::opt('patchupgrade') && ($sys0->{padv} =~ /^Sol11/m) && !Cfg::opt(qw(upgrade_nonkernelpkgs rootpath))) {
        for my $sys (@{CPIC::get('systems')}) {
            $out=$sys->cmd('_cmd_vxdmpadm gettune dmp_native_support 2> /dev/null');
            push (@dmp_enabled_syslist,$sys->{sys}) if($out =~ /dmp_native_support\s+on/mx);
        }
        $syslist=join(',',@dmp_enabled_syslist);
        if ($syslist) {
            $msg = Msg::new("Tunable dmp_native_support is turned 'on' on the following system(s): $syslist.\nIt is required to reboot these machines to update the VRTSvxvm and VRTSvxfs modules.\n");
            $msg->bold;
        }
    }

    return '' unless (Cfg::opt('upgrade') && CPIC::get('reboot'));

    for my $sys (@{CPIC::get('systems')}) {
        if ($sys->{rvg}) {
            $msg=Msg::new("WARNING: Run install script with start option after the machine boots up to finish the upgrade processing.\nDo not delete $tmppath/vvr_upgrade* directories which contain the scripts to do the post upgrade processing.\n");
            $msg->bold;
            last;
        } elsif ($sys->{obsolete_vxvm_exclude_file}) {
            my $exclude_file = "$rootpath$prod->{vxvm_exclude_file}";
            $msg=Msg::new("The content of the file $exclude_file is not updated. Run the install script with start option after the machine boots up to finish the upgrade process\n");
            $msg->bold;
            last;
        } elsif ($sys->exists($prod->{migrate_native_file})) {
            $msg=Msg::new("The dmp_native_support tunable is not enabled. Run the install script with start option after the machine boots up to enable it.\n");
            $msg->bold;
            last;
        }
    }

    return '';
}

package Rel::UXRT62::Linux;

sub get_platreqs {
    my ($rel,$prod) = @_;
    my ($platreqs,$non_uekreqs);
    #e3535822:remove uek kernel from the product kernel list except vcs
    $rel->get_required_os_versions();
    $platreqs = $prod->{platreqs} || $rel->{platreqs};
    if ($prod->{prod} !~ /vcs/i) {
        for my $req (@{$platreqs}) {
            next if ($req =~ /uek/i);
        #OL not supported for sfsybasece
        next if ($req =~/ol/i && $prod->{prod}=~/sfsybase/i);
            push @{$non_uekreqs}, $req;
        }
        return $non_uekreqs;
    } else {
        return $platreqs;
    }
}

sub get_required_os_versions {
    my $rel = shift;
    my ($cpic,$padv,@array,$plat,$os_update_min,$os_update_max,$level,$os_version,@required_os);

    $cpic = Obj::cpic();
    $padv = $rel->{sort_padv};
    @array = split(/\_/,$padv);
    $plat = $array[0];
    $plat =~ tr/[a-z]/[A-Z]/;

    $os_update_min = $rel->{os_update_min};
    $os_update_max = $rel->{os_update_max};

    for my $key (sort compare_kernel_string keys %{$rel->{kernel_padv}{$padv}}) {
        $level = $rel->{kernel_padv}{$padv}->{$key};
        if ($level >= $os_update_min && $level <= $os_update_max) {
            $os_version = $key;
            # for RHEL, the format is 'RHEL6 U4 (2.6.32-358.el6)'
            # for SLES, the format if 'SLES11 SP3 (3.0.76)'
            if ($plat =~ /^RHEL(\d+)$/mx) {
                $os_version = "$plat U$level ($os_version.el$1)";
            } else {
                $os_version = "$plat SP$level ($os_version)";
            }
            push (@required_os,$os_version);
        }
    }

    # for RHEL, need to add OL uek kernel to array
    if (($plat =~ /^RHEL\d+/m) && $rel->{platreqs}) {
        for my $platreq (@{$rel->{platreqs}}) {
            push (@required_os,$platreq) unless ($platreq =~ /^RHEL\d+/m);
        }
    }

    # update it in order to diplay the latest required OS versions
    $rel->{platreqs} = \@required_os;

    return 1;
}

sub compare_kernel_string {
    my (@a,@b,$length,$i);
    @a=split(/-/,$a);
    @b=split(/-/,$b);
    $i=0;
    $length=scalar(@a) > scalar(@b)? scalar(@b) : scalar(@a);

    while ($i < $length) {
        if (EDRu::compvers($a[$i],$b[$i])==1) {
            return 1;
        } elsif (EDRu::compvers($a[$i],$b[$i])==2) {
            return -1;
        } else {
            $i++;
        }
    }
    return 1;
}

sub get_sort_kernel {
    my ($rel,$kstring,$sort_padv)=@_;  
    # on SLES platform, remove the tailing non digits after -
    if ($sort_padv=~/^sles/) {    
        $kstring =~ s/-\D.*$//;    
    }    
    for my $kernel (keys%{$rel->{kernel_padv}{$sort_padv}}) {  
        if ($kstring =~ /^$kernel/ || $kernel =~ /^$kstring/) {  
            return $kernel;  
        }  
    }  
    Msg::log("Failed to retrieve the OS minor version from kernel $kstring");
    return '';
}

sub must_have_patch_sys {
    my ($rel,$sys,$patchid)=@_;
    my ($sort_padv,$patch_hash,$pkg,$pkgname,$patch_arg,@pkglist,$cpic,@prods);

    # Uninstall will not need to download the must have patch
    # As suggested, we only allow the following tasks
    return 0 unless (Cfg::opt(qw/install configure upgrade patchupgrade hotfixupgrade precheck/));

    $cpic=Obj::cpic();
    $sort_padv=$rel->{sort_padv};
    $patch_hash=$rel->{release_matrix}{$sort_padv}{hotfix}{$patchid}{patch};
    $patch_arg=$rel->{patcharg};
    @prods=split(/,/,$rel->{release_matrix}{$sort_padv}{hotfix}{$patchid}{products});

    # Skip the patch if it's not for the product
    unless (EDRu::inarr(lc($cpic->prod->{prod}),@prods)) {
        Msg::log("Patch $patchid is not required by $cpic->prod->{prod}");
        return 0;
    }

    # If the patch(same or higher version) will be applied or already installed, skip this patch
    for my $key (keys %{$patch_hash}) {
        # Reset version
        my ($patch_version,$installed_version,$version);
        $pkgname = $patch_hash->{$key}{pkgs}[0];
        $patch_version = $patch_hash->{$key}{version};
        $installed_version = $sys->{pkgvers}{$pkgname};
        @pkglist=(grep {$_ =~ /$pkgname/} keys %$patch_arg);
        if (@pkglist) {
            # We don't consider the possibility that mutiple patch exists for the same package
            $pkg=$pkglist[0];
            if ($pkg =~ /\D+_(\w+)$/) {
                $version=$1;
                $version=~s/_/./g;
            }
        }
        # Choose the higher version to compare with $patch_version
        $version = (EDRu::compvers($installed_version,$version,4)==1) ? $installed_version : $version;
        if (EDRu::compvers($patch_version,$version,4)==1) {
            return 1;
        }
    }
    return 0;
}

sub platvers_sys {
    my ($rel,$sys) = @_;
    my ($patchlevel,$msg,$rpm_dir,$kret,$ksles,$padv,$kstring,$cpic,@f,$distro,@pkg_files,$pkg,$prod);
    my ($ivers,$localsys,$mediapath,$tmpdir,$vmware,$testVMware,$vmware_script,@vmware_tool_scripts,$state,$platvers);
    my ($service,@vmtoolsd_scripts,$check_vmware_tools_command,$start_vmware_tools_command,$not_running,$platreqs);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return '' if ($sys->{stop_checks});
    return '' if (Cfg::opt('ignorechecks'));

    $cpic=Obj::cpic();
    $prod=$sys->prod($cpic->{prod});
    $padv=$sys->padv($sys->{padv});
    $distro=$padv->distro_sys($sys);
    $kstring=$sys->cmd('_cmd_uname -r');
    $kstring =~ /\-(\d+)/m;
    $kret = $1;
    $ksles=$1 if ($kstring =~ /\-\d+\.(\d+)/mx);
    @f=split(/\W/m,$kstring);
    $platreqs = $rel->get_platreqs($prod);
    $platvers = join("\n\t",@{$platreqs});

    $localsys = Obj::localsys();
    $mediapath=EDR::get('mediapath');
    $tmpdir = EDR::tmpdir();
    $vmware = 0;
    $testVMware = "$mediapath/scripts/bin/testVMware";

    # Check ESX/KVM for AppHA, if vmware tools not installed or not running, quit installer
    # For VCS in VMware environment, if vmware-tools not installed or not running, give warning
    #e3515708:there is no need to check vmware-tools for uninstall
    if ($prod->{prod} =~ /ApplicationHA|VCS/ && !Cfg::opt('uninstall')) {
        $testVMware = "/opt/VRTSsfmh/bin/testVMware" unless ($localsys->exists($testVMware));
        if ($localsys->exists($testVMware)) {
            $localsys->copy_to_sys($sys,$testVMware,"$tmpdir/");
            $sys->cmd("$tmpdir/testVMware");
            if (EDR::cmdexit() == 0) {
                # it's VMware
                $sys->set_value('vmtype','vmware');
                $vmware = 1;
                @vmtoolsd_scripts=('/usr/lib/systemd/system/vmtoolsd.service');
                for my $script (@vmtoolsd_scripts) {
                    if ($sys->exists($script)) {
                        $service=EDRu::basename($script);
                        $check_vmware_tools_command="/usr/bin/systemctl is-active $service 2>/dev/null";
                        $start_vmware_tools_command="systemctl enable $service 2>/dev/null; systemctl start $service 2>/dev/null";
                        $not_running="inactive";
                        last;
                    }
                }
                # If vmtoolsd not exists
                if (!$service) {
                    @vmware_tool_scripts=('/etc/init.d/vmware-tools','/etc/vmware-tools/services.sh','/etc/init.d/vmware-tools-services');
                    for my $script (@vmware_tool_scripts) {
                        if($sys->exists($script)) {
                            $check_vmware_tools_command="$script status 2>/dev/null";
                            $start_vmware_tools_command="$script start 2>/dev/null";
                            $not_running="not";
                            last;
                        }
                    }
                }
                if ($check_vmware_tools_command && $start_vmware_tools_command && $not_running) {
                    $state = $sys->cmd("$check_vmware_tools_command");
                    # vmware-tools not running
                    if ($state =~ /$not_running/i) {
                        # try to start it
                        $sys->cmd("$start_vmware_tools_command");
                        sleep 5;
                        $state = $sys->cmd("$check_vmware_tools_command");
                        if ($state =~ /$not_running/i) {
                            $msg = Msg::new("vmware-tools is not running on $sys->{sys}, installer attempted to start it but failed. Please start the tool before installing $prod->{abbr}");
                            if ($prod->{prod} =~ /ApplicationHA/) {
                                $sys->push_error($msg);
                                return '';
                            } else {
                                $sys->push_warning($msg);
                            }
                        }
                    }
                } else {
                    $msg = Msg::new("The installer did not detect vmware-tools installed on $sys->{sys}, install vmware-tools and start it before you perform any task on $prod->{abbr}");
                    if ($prod->{prod} =~ /ApplicationHA/) {
                        $sys->push_error($msg);
                        return '';
                    } else {
                        $sys->push_warning($msg);
                    }
                }
            }
        } else {
            if ($prod->{prod} =~ /ApplicationHA/) {
                $msg = Msg::new("The installer cannot detect if you are running under VMware environment on $sys->{sys}, it is required to install $prod->{prod} under the VMware or the KVM environment.");
                $sys->push_warning($msg);
            }
            # if installer do NOT have testVMware utility, print a warning but pass the check
            $vmware = 1;
        }

        # AppHA only support to be installed on VMware VM, and KVM on RHEL,
        # do not support to be installed on physical systems, or KVM on OEL/SuSE
        if (!$vmware && $prod->{prod} =~ /ApplicationHA/) {
            if ($sys->{virtual_type} =~ /KVM/i) {
                if ($distro ne 'RHEL') {
                    $msg=Msg::new("$prod->{prod} for KVM does not support to be installed to OL and SuSE.");
                    $sys->push_error($msg);
                    return '';
                }
            } else {
                $msg=Msg::new("The installer cannot detect if you are running under KVM environment on $sys->{sys}, it is required to install $prod->{prod} under the VMware or the KVM environment.");
                $sys->push_warning($msg);
            }
        }
    }

    if ($distro eq 'RHEL') {
        $padv->{cmd}{selinuxenabled}='/usr/sbin/selinuxenabled';

        if ($kstring =~ /el7/m) {
            return '' unless $rel->verify_os_update_sys($sys, $kstring, $distro, $platvers);
        } elsif ($kstring =~ /el6/m) {
            return '' unless $rel->verify_os_update_sys($sys, $kstring, $distro, $platvers);
        } else {
            $sys->push_stop_checks_error($msg);
            return '';
        }
    } elsif ($distro eq 'SLES') {
        $padv->{cmd}{selinuxenabled}='/usr/bin/selinuxenabld';
        return '' unless $rel->verify_os_update_sys($sys, $kstring, $distro, $platvers);
    } elsif ($distro eq 'ESX') {
        # FIX: Figure this check out later for VMWARE. (Get versions of ESX supported and edit)
        if ($kret < 34) {
            $msg=Msg::new("Kernel Release $kstring is not supported on $sys->{sys}. Only Kernel Release 2.6.9-34.EL or higher is supported on RHEL.");
        $sys->push_stop_checks_error($msg);
            return '';
        }
    } elsif ($distro eq 'OL') {
        # FIX: Figure this check out later for Oracle Linux.
        $padv->{cmd}{selinuxenabled}='/usr/sbin/selinuxenabled';
        if ($kstring =~ /el6/m) {
            # OL6 check
            #On OL6.4/6.5(uek2 kernel), install/upgrade non-VCS is not allowed
            if ($prod->{prod} ne 'VCS' && $kstring =~ /uek/m) {
                $msg = Msg::new("$prod->{prod} is not supported on OL6.x UEK2 kernel");
                $sys->push_stop_checks_error($msg);
                return '';
            }

            if ($kstring =~ /uek/m) {
                if ($kstring !~ /^2\.6\.39-400.*el6uek/m) {
                    $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Unbreakable Enterprise Kernel 2.6.39-400 is supported on Oracle Linux 6.4 and Oracle Linux 6.5. you may configure the operating system to boot with this kernel instead.");
                    $sys->push_error($msg);
                    return '';
                }

                #Upgrade path check for UEK2
                #Only supported from 6.0.4/6.0.5 upgrade to 6.2
                $ivers = $prod->version_sys($sys);
                if ($ivers && Cfg::opt(qw(upgrade))
                    && $ivers !~ /^6\.0\.[45]/) {
                    $msg = Msg::new("The upgrade path from $ivers to 6.2 is not supported on OL6.x UEK2 kernel");
                    $sys->push_stop_checks_error($msg);
                    return '';
                }
            }

            if ($kstring !~ /uek/m && $kstring !~ /^2\.6\.32/m) {
                $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Red Hat compatible Kernel Release 2.6.32 is supported on Oracle Linux 6.4 and Oracle Linux 6.5. you may configure the operating system to boot with this kernel instead.");
                $sys->push_error($msg);
                return '';
            }

            # Block SFsybaseCE support for OL6
            if ($prod->{prod} eq 'SFSYBASECE') {
                $msg = Msg::new("Oracle Linux 6 on $sys->{sys} is not supported for $prod->{prod}");
                $sys->push_error($msg);
                return '';
            }

            #this branch won't work in 6.2 release, its existed for futher MR release.
            if ($kstring !~ /uek/m && $cpic->{base_path_release}) {
                if ($kstring =~ /^2\.6\.32-[12].*/m) {
                    #On OL6.1~6.3(Redhat Compatible Kernel), install/upgrade 6.0.1/6.0.2+6.0.5 should print warnning message
                    if ($cpic->{base_path_release} ne '6.0.4') {
                        $msg = Msg::new("System should not be booted into UEK Kernel mode since $prod->{prod} is not supported on OL 6.1 to 6.3 UEK kernel");
                        $sys->push_warning($msg);
                    }
                    #On OL6.4/6.5(Redhat Compatible Kernel),install/upgrade non-VCS should print warning message
                } elsif ($prod->{prod} ne 'VCS' && $kstring =~ /^2\.6\.32-[34].*/m) {
                    $msg = Msg::new("System should not be booted into UEK Kernel mode since $prod->{prod} is not supported on UEK kernel");
                    $sys->push_warning($msg);
                }
            }
        } elsif ($kstring =~ /el7/m) {
            # OL7 check
            if ($kstring =~ /uek/m) {
                if ($kstring !~ /^3\.8\.13-.*el7uek/m) {
                    $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported. Only Unbreakable Enterprise Kernel 3.8.13-16 or up is supported on Oracle Linux 7. You may configure the operating system to boot with this kernel instead.");
                    $sys->push_error($msg);
                    return '';
                }

                #On OL7(uek3 kernel), install/upgrade non-VCS is not allowed
                if ($prod->{prod} ne 'VCS') {
                   $msg = Msg::new("$prod->{prod} is not supported on OL7.x UEK3 kernel");
           $sys->push_stop_checks_error($msg);
                   return '';
                }
        # Only PR support for OL7 UEK in 6.2.1
        if ($rel->{type} =~ /M/ && $rel->{titlevers}== '6.2.1') {
                   $msg = Msg::new("This installation image is not supported on OL7.x UEK3 kernel");
           $sys->push_stop_checks_error($msg);
                   return '';
        }

                #Upgrade path check for UEK3
                #Only supported from 6.0.4/6.0.5/6.2 upgrade to 6.2.1
                $ivers = $prod->version_sys($sys);
                if ($ivers && Cfg::opt(qw(upgrade))
                    && $ivers !~ /^(6\.0\.[45]|6\.2)/) {
                    $msg = Msg::new("The upgrade path from $ivers to 6.2.1 is not supported on OL7.x kernel");
            $sys->push_stop_checks_error($msg);
                    return '';
                }
            } else {
        # No PR support for OL7 Redhat Compatible Mode in 6.2.1
                if ($rel->{type} !~ /M/ && $rel->{titlevers}== '6.2.1') {
                    $msg = Msg::new("This installation image is not supported on OL7 Redhat Compatible Kernel");
            $sys->push_stop_checks_error($msg);
                    return '';
                }
        }
        } else {
            $msg=Msg::new("Kernel Release $kstring found on $sys->{sys} is not supported.");
            $sys->push_error($msg);
            return '';
        }
    } else {
        $msg=Msg::new("Distribution is not recognized as SuSE or Red Hat on $sys->{sys}");
    $sys->push_stop_checks_error($msg);
        return '';
    }

    # XEN is not supported in 6.0
    if ($kstring =~ /xen/m) {
        $msg=Msg::new("Kernel Release on $sys->{sys} is $kstring. Xen kernel is not supported in this release.");
    $sys->push_stop_checks_error($msg);
        return '';
    }

    # Distribution match on all systems removed as any to many support is added
    for my $pkgi (@{$rel->allpkgs}) {
        $pkg=$sys->pkg($pkgi);
        push(@pkg_files,$pkg->{file});
    }

    if (@pkg_files) {
       return 1 if ($cpic->{fromdisk});
       $rpm_dir=$rel->pkgs_patches_dir('pkgpath');;
       if ((($sys->{arch}=~/686/m)||($sys->{arch}=~/586/m)) && (grep {/86_64/m} @pkg_files)) {
            $msg=Msg::new("System $sys->{sys} is architecture $sys->{arch} and x86_64 rpms were found in $rpm_dir. Check that you are installing from the correct DVD. Some rpms will not install correctly on this system.");
            $sys->push_error($msg);
            return '';
        }

       if ((($distro eq 'RHEL') && (grep {/SLES/} @pkg_files))
            ||  (($distro eq 'SLES') && (grep {/RHEL/} @pkg_files))) {
            $msg=Msg::new("System $sys->{sys} is distribution $distro and incompatible rpms were found in $rpm_dir. Check that you are installing from the correct DVD. Some rpms will not install correctly on this system.");
            $sys->push_error($msg);
            return '';
       }
    }

    # No need for selinux checks any more
    return 1;
}

sub verify_os_update_sys {
    my ($rel, $sys, $kstring, $distro, $platvers) = @_;
    my ($os_update, $os_update_min, $os_update_max, $os_update_err);
    my ($cpic, $sort_padv, $sort_kernel, $required_patch_id, $msg);

    $cpic=Obj::cpic();
    $os_update_min     = $rel->{os_update_min};
    $os_update_max     = $rel->{os_update_max};
    $os_update_err     = $rel->{os_update_err};
    $sort_padv         = $rel->{sort_padv};
    $sort_kernel       = $rel->get_sort_kernel($kstring, $sort_padv);
    $os_update         = $rel->{kernel_padv}{$sort_padv}{$sort_kernel};
    $required_patch_id = $rel->{os_update_patchid_mapping}{$os_update};

    Msg::log("OS update level of $sys->{sys}: $os_update");
    Msg::log("Required patch id: $required_patch_id") if ($required_patch_id);

    if ( defined($os_update_err) && ($os_update == $os_update_err) ) {
        $msg=Msg::new("Kernel Release $kstring is detected on $sys->{sys}, which is not supported for this product. The supported Kernel Releases are:\n\t$platvers");
        $sys->push_error($msg);
        return 0;
    } elsif ($os_update < $os_update_min) {
        $msg=Msg::new("Kernel Release $kstring is detected on $sys->{sys}, which is not supported for this product. The supported Kernel Releases are:\n\t$platvers");
        $sys->push_error($msg);
        return 0;
    } elsif ($os_update > $os_update_max) {
        if ($cpic->{sortready}) {
            $msg = Msg::new("Kernel Release $kstring is detected on $sys->{sys}, which is not supported for this product.");
        } else {
            $msg = Msg::new("Kernel Release $kstring is detected on $sys->{sys}. Additional patches may be required to deploy this product on the Kernel Release $kstring. It is strongly recommended to check it on SORT (https://sort.symantec.com) before continue.");
        }
        $sys->push_warning($msg);
    } else { # os_update_err is undefined and $os_update is between [$os_update_min..$os_update_max]
        # Check if there is patch required on this os update and not been included in this execution
        if ( $required_patch_id && $rel->must_have_patch_sys($sys,$required_patch_id) ) {
            $sys->set_value('must_have_patch',$required_patch_id);
        } else {
            # When prod_prerequisites has required OS lib, CPI will recommend to install the OS lib with YUM
            # After Yum installation, pre-check will be re-executed, need to re-set this value to avoid it download again
            $sys->set_value('must_have_patch',undef);
        }
    }

    return 1; # yeah! no errors!
}

package CPIC;

sub ask_install {
    my $cpic=shift;
    my ($ayn,$cfg,$msg,$prod,$systems,$continue,$rel);
    $prod=$cpic->prod;
    $cfg=Obj::cfg();
    $rel=$cpic->rel();
    $systems = join(' ',@{$cfg->{systems}});
    if (Cfg::opt('upgrade')) {
        $msg=Msg::new("Would you like to upgrade $prod->{abbr} on $systems?");
    } else {
        $msg=Msg::new("Would you like to install $prod->{abbr} on $systems?");
    }
    $ayn=$msg->aynn();

    if ($ayn eq 'Y') {
        $continue = 1;
        # down required path if users choose to go on
        if ($rel->{required_patch_id}) {
            $continue = $cpic->install_must_have_patch($rel->{required_patch_id});
        }
    }

    Msg::n();
    return $continue;
}

sub create_cpip_objects {
    my ($cpic,$padv) = @_;
    my ($rel,$patch,$pkglist,@pkgs,$pkg,$prod,$hotfix);
    if (!EDRu::inarr($padv, @{EDR::padvs()}, 'Common')) {
        # import anything to operate the rest of the commands OK
        # unsupported padvs will eventually die later
        Msg::log("create_cpip_objects: $padv is not a supported platform");
        return;
    } elsif ($Obj::pool{"Rel\::$cpic->{release}\::$padv"}) {
        #Msg::log("$cpic->{release} $padv CPIP objects already created");
        return $cpic->rel($padv);
    }
    #Msg::log("Creating $cpic->{release}::$padv CPIP objects");
    $rel=$cpic->{rel}{$padv}=init_cpip_object("Rel::$cpic->{release}",$padv);
    push(@pkgs, @{$rel->allpkgs});
    for my $prodv (@{$rel->{prods}}) {
        $prod=init_cpip_object("Prod::$prodv",$padv);
        $prod->{release}=$cpic->{release};
        $prod->set_pkgs;
        push(@pkgs, @{$prod->allpkgs_on_media});
        for my $subprodv(@{$prod->{subprods}}) {
            my $subprod=init_cpip_object("Prod::$subprodv",$padv);
            $subprod->{release}=$cpic->{release};
            push(@pkgs, @{$subprod->allpkgs_on_media});
        }
    }

    $pkglist=EDRu::arruniq(@pkgs);
    for my $pkgi (@{$pkglist}) {
        $pkg=init_cpip_object("Pkg::$pkgi",$padv);
        for my $proc (@{$pkg->allprocs}) { init_cpip_object("Proc::$proc",$padv); }
        for my $prevpkgname (@{$pkg->{previouspkgnames}}) {
            Pkg->new_pkg($prevpkgname, $padv);
        }
    }

    $cpic->add_mr();
    if ($cpic->{mr}) {
        my $mr = init_cpip_object("MR::$cpic->{mr}", $padv);
        $mr->define_patches();
        $cpic->{mrobj} = $mr;
    }

    $cpic->add_hotfix;
    for my $hfi(@{$cpic->{hotfixes}}) {
        $hotfix=init_cpip_object("Hotfix::$hfi",$padv);
        push(@{$cpic->{hotfixobj}}, $hotfix);
        # must correlate the hotfix to the arg used to import it
        # initialized patches receive this def in order to determine the correct mediapath
        $hotfix->{arg}=$cpic->{hotfix_arg}{$hfi} if (defined($cpic->{hotfix_arg}{$hfi}));
        $hotfix->define_patches();
    }

    $rel->define_patches();
    for my $pkgi (@{$pkglist}) {
        $pkg=Obj::pkg($pkgi, $padv);
        for my $patchi (@{$pkg->patches_allvers}) {
            # Patches are not built-in defined in CPIP, create it.
            $patch=new_object("Patch::$patchi",$padv);
            $patch->{pkg}=$pkgi;
            # was this patch initialized using base_path or a hotfix_path
            # if so we need to reference a different mediapath
            $patch->{arg}=$rel->{patcharg}{$patchi} if (defined($rel->{patcharg}{$patchi}));
        }
    }
    #$cpic->create_eeb_objects($rel,$padv) if ($rel->{reli}=~/^EEB/);
    $rel->adjust_pkglist();
    $cpic->rel->initiate_kernel_padv_hash();
    $cpic->rel->init_sort_platvers();
    return $rel;
}

sub setup_verify_systems_phase {
    my ($cpic, $check_com_only) = @_;
    my ($edr,$rel,$phase,$desc,$task,$task_name,%task_params,$pdfrs,$id);
    my ($skip_prerequisite,$skip_fsspace,$skip_license,$skip_platvers);

    $cpic->{falsefailures}=0;
    $phase= Phase->new('CPIC_prechecks');

    # Directly return $phase if it is already created.
    return $phase if (defined $phase->{description});

    $desc = Msg::new("Verifying systems");
    $phase->set_description($desc);
    $phase->set_fatal_error_key('stop_checks');
    $phase->initialize_tasks();

    $edr=Obj::edr();
    $rel=$cpic->rel();
    $pdfrs=Msg::get('pdfrs');

    # By default, skip product version/license checking,
    # skip patches checking, skip file system space checking
    $skip_prerequisite=1;
    $skip_fsspace=1;
    $skip_license=1;

    # if install/upgrade task specified, then run above checkings.
    if (install_check_task()) {
        $skip_prerequisite=0;
        $skip_fsspace=0;
        $skip_license=0;
    }
    if ($rel->rollback_check_task()) {
        $skip_fsspace=0;
    }
    $skip_fsspace=1 if (Cfg::opt('nospacecheck'));
    $skip_license=1 if (!$rel->can("read_licenses_sys"));
    $skip_platvers=1 if (Cfg::opt('uninstall'));

    $id=100;
    $task_name='edr_check_communication';
    %task_params= (
        'sequence_id' => $id,
        'description' => Msg::new("Checking system communication"),
        'description_sys' => Msg::new("Checking communication on #{SYS}"),
        'if_per_system' => 1,
        'handler' => \&EDR::transport_sys,
        'handler_object_arg' => $edr,
        'post_handler' => \&CPIC::post_transport,
        'post_handler_object_arg' => $cpic,
        'error_handler' => \&CPIC::transport_error_handler,
        'error_handler_object_arg' => $cpic,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    return $phase if ($check_com_only);

    $id+=100;
    $task_name='edr_check_release_compatibility';
    %task_params= (
        'sequence_id' => $id,
        'description' => Msg::new("Checking release compatibility"),
        'description_sys' => Msg::new("Checking release compatibility on #{SYS}"),
        'if_per_system' => 1,
        'handler' => \&EDR::supported_padv_sys,
        'handler_object_arg' => $edr,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    $id+=100;
    $task_name='cpic_check_installed_product';
    %task_params= (
        'sequence_id' => $id,
        'description' => Msg::new("Checking installed product"),
        'description_sys' => Msg::new("Checking installed product on #{SYS}"),
        'if_per_system' => 1,
        'handler' => \&CPIC::installed_prod_info_sys,
        'handler_object_arg' => $cpic,
        'post_handler' => \&CPIC::post_installed_prod_info,
        'post_handler_object_arg' => $cpic,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    $id+=100;
    $task_name='cpic_check_platform_version';
    %task_params= (
        'sequence_id' => $id,
        'description' => Msg::new("Checking platform version"),
        'description_sys' => Msg::new("Checking platform version on #{SYS}"),
        'if_per_system' => 1,
        'skip' => $skip_platvers,
        'pre_handler' => \&CPIC::pre_rel_platvers,
        'pre_handler_object_arg' => $cpic,
        'handler' => \&CPIC::rel_platvers_sys,
        'handler_object_arg' => $cpic,
        'post_handler' => \&CPIC::post_rel_platvers,
        'post_handler_object_arg' => $cpic,
        'error_handler' => \&CPIC::rel_platvers_error_handler,
        'error_handler_object_arg' => $cpic,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    $id+=100;
    $task_name='cpic_check_prerequisite_packages_patches';
    %task_params= (
        'sequence_id' => $id,
        'description' => Msg::new("Checking prerequisite patches and $pdfrs"),
        'description_sys' => Msg::new("Checking prerequisite patches and $pdfrs on #{SYS}"),
        'if_per_system' => 1,
        'skip' => $skip_prerequisite,
        'handler' => \&CPIC::prod_prerequisite_sys,
        'handler_object_arg' => $cpic,
        'post_handler' => \&CPIC::post_prod_prerequisite,
        'post_handler_object_arg' => $cpic,
        'error_handler' => \&CPIC::prod_prerequisite_error_handler,
        'error_handler_object_arg' => $cpic,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    $id+=100;
    $task_name='cpic_check_filesystem_free_space';
    %task_params= (
        'sequence_id' => $id,
        'description' => Msg::new("Checking file system free space"),
        'description_sys' => Msg::new("Checking file system free space on #{SYS}"),
        'if_per_system' => 1,
        'skip' => $skip_fsspace,
        'handler' => \&CPIC::fsspace_sys,
        'handler_object_arg' => $cpic,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    $id+=100;
    $task_name='cpic_check_product_license';
    %task_params= (
        'sequence_id' => $id,
        'description' => Msg::new("Checking product licensing"),
        'description_sys' => Msg::new("Checking product licensing on #{SYS}"),
        'if_per_system' => 1,
        'skip' => $skip_license,
        'handler' => \&CPIC::prod_licenses_sys,
        'handler_object_arg' => $cpic,
        'post_handler' => \&CPIC::post_prod_licenses,
        'post_handler_object_arg' => $cpic,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    $id+=100;
    $task_name='cpic_perform_product_prechecks';
    %task_params= (
        'sequence_id' => $id,
        'description' => Msg::new("Performing product prechecks"),
        'description_sys' => Msg::new("Performing product prechecks on #{SYS}"),
        'if_per_system' => 1,
        'pre_handler' => \&CPIC::pre_prod_precheck,
        'pre_handler_object_arg' => $cpic,
        'handler' => \&CPIC::prod_precheck_sys,
        'handler_object_arg' => $cpic,
        'post_handler' => \&CPIC::post_prod_precheck,
        'post_handler_object_arg' => $cpic,
        'error_handler' => \&CPIC::prod_precheck_error_handler,
        'error_handler_object_arg' => $cpic,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    return $phase;
}

sub pre_rel_platvers {
    my $cpic=shift;
    return $cpic->rel->pre_platvers();
}

sub rel_platvers_error_handler {
    my $cpic=shift;
    my ($rel,$syslist,$sys0,$padv,$msg,$patches,$rtn,$required_patch_id,$required_patch);

    $rel = $cpic->rel();
    $padv = $rel->{sort_padv};
    $syslist = CPIC::get('systems');
    $sys0 = $$syslist[0];

    for my $sys(@$syslist) {
        if ($sys->{must_have_patch}) {
            $required_patch_id = $sys->{must_have_patch} if ($sys->{must_have_patch} > $required_patch_id);
        }
    }
    return unless $required_patch_id;
    $rel->set_value('required_patch_id',$required_patch_id);

    Msg::n();
    $required_patch = $cpic->rel->{release_matrix}{$padv}{'hotfix'}{$required_patch_id}{'release_name'};
    if (Cfg::opt('precheck')) {
        $msg=Msg::new("The following patch for InfoScale product is required:\n\t$required_patch");
        $msg->print;
        $sys0->push_note($msg);
    } else {
        $msg=Msg::new("The following patch for InfoScale product will be downloaded and installed:\n\t$required_patch");
        $msg->print;
        $rtn=$cpic->install_must_have_patch($required_patch_id);
    }

    return 1;
}

sub install_must_have_patch {
    my ($cpic,$required_patch_id) =@_;
    my ($patch_path,$padv,$msg,$rtn);

    $patch_path = $cpic->download_must_have_patch($required_patch_id);
    return 0 if (!$patch_path);
    $rtn = $cpic->expand_must_have_patch($patch_path);
    return 0 if (!$rtn);
    $cpic->apply_must_have_patch($patch_path);

    return 1;
}

# to download must have patch
sub download_must_have_patch {
    my ($cpic,$required_patch_id) = @_;
    my ($url,$sys0,$dir,$filename,$release_name,$rtn,$file,$msg,$download_errors,$sort_hotfix,$padv);

    $padv = $cpic->rel->{sort_padv};
    $sort_hotfix = $cpic->rel->{release_matrix}{$padv}{'hotfix'}{$required_patch_id};
    $url = $sort_hotfix->{upload_location};
    return '' unless $url;

    $sys0 = ${CPIC::get('systems')}[0];
    $filename = $sort_hotfix->{filename};
    $release_name = $sort_hotfix->{release_name};
    $cpic->{required_patch_path}='/opt/VRTS/install/repository/required_patch';
    $dir = $cpic->{required_patch_path} . '/' . $release_name;
    $file = "$dir/$filename";
    # check if we can reuse the patch which has already been downloaded
    if (-f $file) {
        $rtn = $cpic->download_must_have_patch_cksum($required_patch_id,$dir);
        if ($rtn) {
            Msg::n();
            $msg = Msg::new("$release_name has already been downloaded.");
            $msg->print();
            return $dir;
        }
    }
    # sort isn't ready and we don't have it locally
    if (!$cpic->{sortready}) {
        $msg = Msg::new("Failed to connect to SORT (https://sort.symantec.com), the patch $release_name is required to deploy this product.");
        if (Cfg::opt('precheck')) {
            Msg::n();
            $msg->error;
            Msg::n();
        } else {
            $sys0->push_error($msg);
        }
        return ''
    }

    EDRu::mkdir_local_nosys($dir) || return '';
    Msg::n();
    $msg = Msg::new("Downloading required patch $release_name from SORT (https://sort.symantec.com)");
    $msg->printn;
    $rtn=EDRu::download_file($url, $file, 1, 2);
    if (!$rtn || ! -f $file) {
        $download_errors=EDR::get('download_errors');
        Msg::log("Could not download file $file!\n\n$download_errors\n");
        EDRu::rmdir_local_nosys($dir);
        return '';
    }

    # check sum after downloading the patch
    $rtn = $cpic->download_must_have_patch_cksum($required_patch_id,$dir);
    if ($rtn) {
        Msg::log("Download required patch $release_name from SORT (https://sort.symantec.com) successfully.\n");
        return $dir;
    } else {
        $msg = Msg::new("Failed to download the required patch $release_name.");
        $sys0->push_error($msg);
        return '';
    }
}

sub download_must_have_patch_cksum {
    my ($cpic,$required_patch_id,$dir) = @_;
    my ($padv,$filename,$file,$sort_hotfix,@cksum_size,$cksum,$size);

    $padv = $cpic->rel->{sort_padv};
    $sort_hotfix = $cpic->rel->{release_matrix}{$padv}{'hotfix'}{$required_patch_id};

    @cksum_size = ($sort_hotfix->{upload_cksum},$sort_hotfix->{upload_size});
    $filename = $sort_hotfix->{filename};
    $file = "$dir/$filename";

    if ((-f $file) && ($#cksum_size > 0)) {
        $cksum = EDRu::cksum($file);
        $size = EDR::cmd_local("_cmd_ls -l $file");
        $size=(split /\s+/, $size)[4];
        return 1 if (($cksum eq $cksum_size[0]) && ($size  eq $cksum_size[1]));
    }

    return 0;
}

sub apply_must_have_patch {
    my ($cpic,$patch_path) = @_;
    my ($msg,$pm,$install_script,$install_path,$cmd);
    my $edr=Obj::edr();
    my $rel=Obj::rel();

    $install_script= $cpic->{script};
    $install_path = $cpic->{scriptdir};
    if ($patch_path && ! -d $patch_path) {
        return 0;
    } else {
        if (!-d "$patch_path/scripts") {
            $msg = Msg::new("The downloaded patch is invalid. Contact our technical support for further troubleshooting.");
            $msg->error;
            return 0;
        }
    }

    # Add this hotfix
    my ($path1,$path2,$sub);
    for my $path (qw/patch_path patch2_path patch3_path patch4_path patch5_path/) {
        if (Cfg::opt("$path")) {
            $path2=$path;
            next;
        }
        $path1=$path;
        last;
    }

    Cfg::set_opt($path1,$patch_path);
    $sub="process_$path1\_arg";
    $sub=~s/patch/hotfix/;
    $cpic->$sub($path1,$path2);

    # create hotfix and patch object
    my ($hf,$hotfix,$pkg,$patch,$pkglist,$padv);
    $padv=$rel->{padv};
    $hf=$cpic->{hotfixes}[-1];
    $hotfix=init_cpip_object("Hotfix::$hf",$padv);
    push(@{$cpic->{hotfixobj}},$hotfix);

    $hotfix->{arg}=$cpic->{hotfix_arg}{$hf} if (defined($cpic->{hotfix_arg}{$hf}));
    $hotfix->define_patches();

    for my $patchi (@{$hotfix->{patches}}) {
        $patch=new_object("Patch::$patchi",$padv);
        if ($patch->{patch} =~ /^([a-zA-Z0-9]+)_(\d+)_(\d+)/mxg) {
            $patch->{pkg}="$1$2$3";
        }
        $patch->{arg}=$rel->{patcharg}{$patchi} if (defined($rel->{patcharg}{$patchi}));
    }
    # re-adjust package list
    $rel->adjust_pkglist();
    # re-verify media
    $cpic->verify_media($cpic->rel);
    return 1;
}

sub expand_must_have_patch {
    my ($cpic,$tar_path) = @_;
    my ($msg);

    EDR::cmd_local("cd $tar_path; _cmd_gunzip -c *.gz| _cmd_tar xf - 2>/dev/null");
    if (EDR::cmdexit()) {
        $msg = Msg::new("Failed to un-zip and un-tar the file under $tar_path.");
        $msg->error;
        return 0;
    }
    return 1;
}

package CPIP;

sub get_kernel_padv_sort_url {
    my $url;

    $url="http://sort.symantec.com/SORTServices/rest/matrix/1.0/padvmatrix?&source=cpi";

    return $url;
}

package Rel;

sub initiate_kernel_padv_hash {
    my ($rel) = @_;
    my ($response,$url,$kernel_padv_file,$jsonhash,$jsontext,$jsonfile);
    my ($rootpath,$cpic,$edr,$perl_matrix,$local_matrix,$image_matrix,$level);

    return 1 if ($rel->{kernel_padv} && %{$rel->{kernel_padv}});
    $rootpath=Cfg::opt("rootpath");
    $cpic=Obj::cpic();
    $edr=Obj::edr();

    $jsonfile="kernel_padv.json";
    $perl_matrix="$rootpath/$edr->{perlpath}/lib/site_perl/$cpic->{release}/CPIP/Rel/Matrix/$jsonfile";
    $local_matrix="$rootpath/opt/VRTS/install/.matrix/$jsonfile";
    $image_matrix="$rootpath$edr->{scriptdir}/scripts/CPIP/Rel/Matrix/$jsonfile";

    if ($cpic->{sortready}) {
        # with SORT connection, download the latest kernel-padv matrix
        $url=CPIP::get_kernel_padv_sort_url();
        Msg::log("Downloading kernel padv matrix from URL '$url'");
        $response = EDRu::download_page($url);
        if ($response) {
            $jsonhash = EDRu::eval_json($response);
            if ($jsonhash) {
                # Save the matrix to local
                $jsontext=JSON::to_json($jsonhash,{pretty=>1});
                EDRu::mkdir_local_nosys(EDRu::dirname($local_matrix));
                EDRu::writefile($jsontext,$local_matrix);
            }
        }
    }
    if (!$jsonhash) {
        # without SORT connection/matrix download failed, use the local kernel-padv matrix
        for my $file($local_matrix,$image_matrix,$perl_matrix) {
            if (-f $file) {
                $jsonhash=$rel->get_matrix_data($file);
            }
            if ($jsonhash) {
                Msg::log("Using local kernel padv matrix json: $file");
                last;
            }
        }
    }

    if (!$jsonhash) {
        # for 6.2.1, there's no kernel_padv.json in the build
        # if there's no sort connection, we will get it locally 
        $jsonhash = $rel->get_local_kernel_padv_hash();
    }

    # initiate the $jsonhash content into $rel->{kernel_padv}{$padv}{$kernel}
    for my $key (keys %{$jsonhash}) {
        $level=$jsonhash->{$key}{level};
        $level=~s/\D+(\d+)\D*/$1/mx;
        $rel->{kernel_padv}{$jsonhash->{$key}{padv}}{$key}=$level;
    }
    return 1;
}

sub get_local_kernel_padv_hash {
    my ($rel)=@_;
    my ($cpic,$response,$jsonhash,$rootpath,$local_jsonfile,$jsontext);

    $cpic=Obj::cpic();
    $rootpath = Cfg::opt("rootpath");
    $local_jsonfile = "$rootpath/opt/VRTS/install/.matrix/kernel_padv.json";
    $response = $rel->get_local_kernel_padv_response();

    $jsonhash = EDRu::eval_json($response);
    # if there's no SORT connnection and kernel_padv.json isn't there, write file to local matrix
    if (!-f $local_jsonfile && !$cpic->{sortready}) {
        $jsontext = JSON::to_json($jsonhash,{pretty=>1});
        EDRu::mkdir_local_nosys(EDRu::dirname($local_jsonfile));
        EDRu::writefile($jsontext,$local_jsonfile);
    }

    return $jsonhash;
}

# initiate the release matrix for target padv
sub init_sort_platvers {
    my ($rel)=@_;
    my ($response,$padv,$url,$jsontext,$jsonfile);
    my ($rootpath,$cpic,$edr,$perl_matrix_dir,$local_matrix_dir,$image_matrix_dir,$level);
    return 1 if (Padv::plat($rel->{padv}) ne 'Linux');
    return 1 if ($rel->{release_matrix} && %{$rel->{release_matrix}});
    $rootpath=Cfg::opt("rootpath");
    $cpic=Obj::cpic();
    $edr=Obj::edr();
    $padv=Padv::cpi_to_sort_padv($rel->{padv});
    $rel->set_value("sort_padv",$padv);

    $perl_matrix_dir="$rootpath/$edr->{perlpath}/lib/site_perl/$cpic->{release}/CPIP/Rel/Matrix/";
    $local_matrix_dir="$rootpath/opt/VRTS/install/.matrix/";
    $image_matrix_dir="$rootpath$edr->{scriptdir}/scripts/CPIP/Rel/Matrix/";

    # with SORT connection, download the latest $padv ga/patch/hotfix matrix
    for my $level (qw/ga patch hotfix/) {
        my $jsonhash;
        if ($cpic->{sortready}) {
            $url = CPIP::get_hotfix_sort_url();
            $url.= "?padv=$padv&level=$level&pretty=1&product=SFHA";
            Msg::log("Downloading release matrix data for $padv $level");
            $response = EDRu::download_page($url);
            if ($response) {
                $jsonhash = EDRu::eval_json($response);
                if ($jsonhash) {
                    # Save the matrix to local
                    $jsontext=JSON::to_json($jsonhash,{pretty=>1});
                    EDRu::mkdir_local_nosys($local_matrix_dir);
                    EDRu::writefile($jsontext,"$local_matrix_dir/$padv-$level.json");
                }
            }
        }
        if (!$jsonhash) {
            for my $dir ($local_matrix_dir,$image_matrix_dir,$perl_matrix_dir) {
                my $file="$dir/$padv-$level.json";
                if (-f $file) {
                    $jsonhash=$rel->get_matrix_data($file);
                }
                if ($jsonhash) {
                    Msg::log("Using local release matrix $padv $level json: $file");
                    last;
                }
            }
            if (!$jsonhash) {
                Msg::die("Cannot find release matrix json hash for $padv $level");
                return 0;
            }
        }
        $rel->{release_matrix}{$padv}{$level}=$jsonhash->{$padv};
    }
    return 1;
}

sub pre_platvers {
    my ($rel)=@_;
    my ($sort_padv,$matrix);
    my ($msg,$common_sys,$cpic);
    $cpic=Obj::cpic();
    
    return 1 if (Padv::plat($rel->{padv}) ne 'Linux');
    $sort_padv=$rel->{sort_padv};
    $matrix=$rel->{release_matrix}{$sort_padv}{'patch'};
    while ( my ($id, $val) = each %{$matrix} ) {
        if ($val->{release_version} == $rel->{vers}) {
            $rel->set_value('os_update_min', $val->{os_update_min});
            $rel->set_value('os_update_max', $val->{os_update_max});
            $rel->set_value('os_update_err', $val->{os_update_err});
            while ( my ($required_patch_update, $required_patch_id) = each %{$val->{required_patch}} ) {
                $rel->set_value("os_update_patchid_mapping,$required_patch_update", $required_patch_id);
            }
            last;
        }
    }
    # For 6.2.1 release, we define the os_update_min/os_update_max in code for rhel6/7, sles11/12
    unless (defined($rel->{os_update_min})) {
        if ($sort_padv =~ /^rhel6/) {
            $rel->set_value('os_update_min', 4);
        } elsif ($sort_padv =~ /^rhel7/) {
            $rel->set_value('os_update_min', 0);
        } elsif ($sort_padv =~ /^sles11/) {
            $rel->set_value('os_update_min', 2);
        } elsif ($sort_padv =~ /^sles12/) {
            $rel->set_value('os_update_min', 0);
        }
    }
    unless (defined($rel->{os_update_max})) {
        if ($sort_padv =~ /^rhel6/) {
            $rel->set_value('os_update_max', 6);
        } elsif ($sort_padv =~ /^rhel7/) {
            $rel->set_value('os_update_max', 1);
        } elsif ($sort_padv =~ /^sles11/) {
            $rel->set_value('os_update_max', 3);
        } elsif ($sort_padv =~ /^sles12/) {
            $rel->set_value('os_update_max', 0);
        }
    }
    Msg::log("Minimum OS update level supported: $rel->{os_update_min}");
    Msg::log("Maximum OS update level supported: $rel->{os_update_max}");
    Msg::log("Unsupported OS update level: $rel->{os_update_err}") if ($rel->{os_update_err});
    
    unless (defined($rel->{os_update_min}) && defined($rel->{os_update_max})) {
        $common_sys=@{$cpic->{systems}}[0];
        $msg=Msg::new("Failed to retrieve supported $rel->{padv} update level from the release matrix, contact our technical support for further troubleshooting.");
        $common_sys->push_error($msg);
        for my $sys (@{$cpic->{systems}}) {
            $sys->set_value('stop_checks',1);
        }
    }
    return 1;
}

sub get_local_kernel_padv_response {
    my ($rel)=@_;
    my ($response);

    $response = '{
   "2.6.16.21-0.15" : {
      "gaDate" : "2006-08-18",
      "level" : "",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.21-0.25" : {
      "gaDate" : "2006-09-26",
      "level" : "",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.21-0.8" : {
      "gaDate" : "2006-07-01",
      "level" : "",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.27-0.6" : {
      "gaDate" : "2006-12-21",
      "level" : "",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.27-0.9" : {
      "gaDate" : "2007-02-23",
      "level" : "",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.46-0.12" : {
      "gaDate" : "2007-06-01",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.46-0.14" : {
      "gaDate" : "2007-06-19",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.53-0.16" : {
      "gaDate" : "2007-10-11",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.53-0.18" : {
      "gaDate" : "2007-11-07",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.53-0.8" : {
      "gaDate" : "2007-09-04",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.54-0.2.10" : {
      "gaDate" : "2008-10-01",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.54-0.2.11" : {
      "gaDate" : "2008-10-01",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.54-0.2.12" : {
      "gaDate" : "2008-12-02",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.54-0.2.3" : {
      "gaDate" : "2007-12-03",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.54-0.2.5" : {
      "gaDate" : "2008-02-07",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.54-0.2.8" : {
      "gaDate" : "2008-07-04",
      "level" : "SP1",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.101.1" : {
      "gaDate" : "2013-04-12",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.103.1" : {
      "gaDate" : "2013-06-04",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.105.1" : {
      "gaDate" : "2014-04-16",
      "level" : "SP4 - LTSS",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.107.1" : {
      "gaDate" : "2014-06-10",
      "level" : "SP4 - LTSS",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.113.1" : {
      "gaDate" : "2013-12-09",
      "level" : "SP3 - LTSS",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.123.1" : {
      "gaDate" : "2014-06-23",
      "level" : "SP3 - LTSS",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.132.1" : {
      "gaDate" : "2015-04-30",
      "level" : "SP4 - LTSS",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.21" : {
      "gaDate" : "2008-05-01",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.23" : {
      "gaDate" : "2008-06-01",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.25" : {
      "gaDate" : "2008-07-18",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.27" : {
      "gaDate" : "2008-07-29",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.29" : {
      "gaDate" : "2008-10-01",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.30" : {
      "gaDate" : "2008-10-01",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.31" : {
      "gaDate" : "2008-10-20",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.33" : {
      "gaDate" : "2008-11-25",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.34" : {
      "gaDate" : "2009-01-29",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.35" : {
      "gaDate" : "2009-02-17",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.37" : {
      "gaDate" : "2009-04-03",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.39.3" : {
      "gaDate" : "2009-05-20",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.42.10" : {
      "gaDate" : "2010-05-06",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.42.11" : {
      "gaDate" : "2011-06-14",
      "level" : "SP2 - LTSS",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.42.4" : {
      "gaDate" : "2009-08-20",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.42.5" : {
      "gaDate" : "2009-08-27",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.42.54.1" : {
      "gaDate" : "2012-01-24",
      "level" : "SP2 - LTSS",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.42.7" : {
      "gaDate" : "2009-11-10",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.42.8" : {
      "gaDate" : "2009-12-22",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.42.9" : {
      "gaDate" : "2010-02-04",
      "level" : "SP2",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.54.5" : {
      "gaDate" : "2009-10-01",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.57.1" : {
      "gaDate" : "2009-11-12",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.58.1" : {
      "gaDate" : "2009-12-14",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.59.1" : {
      "gaDate" : "2010-01-22",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.60.1" : {
      "gaDate" : "2010-03-29",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.62.1" : {
      "gaDate" : "2010-04-26",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.66.1" : {
      "gaDate" : "2010-06-11",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.67.1" : {
      "gaDate" : "2010-08-17",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.68.1" : {
      "gaDate" : "2010-09-03",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.69.1" : {
      "gaDate" : "2010-09-22",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.74.7" : {
      "gaDate" : "2010-12-13",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.76.8" : {
      "gaDate" : "2011-01-25",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.77.1" : {
      "gaDate" : "2011-03-24",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.79.1" : {
      "gaDate" : "2011-06-29",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.81.2" : {
      "gaDate" : "2011-07-02",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.83.2" : {
      "gaDate" : "2011-09-20",
      "level" : "SP3",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.85.1" : {
      "gaDate" : "2011-04-01",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.87.1" : {
      "gaDate" : "2011-05-19",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.89.1" : {
      "gaDate" : "2011-08-12",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.91.1" : {
      "gaDate" : "2011-10-28",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.93.1" : {
      "gaDate" : "2012-01-23",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.95.1" : {
      "gaDate" : "2012-01-23",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.97.1" : {
      "gaDate" : "2012-06-14",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.16.60-0.99.1" : {
      "gaDate" : "2012-10-23",
      "level" : "SP4",
      "padv" : "sles10_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-128" : {
      "gaDate" : "2009-01-20",
      "level" : "Update 3",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-164" : {
      "gaDate" : "2009-09-02",
      "level" : "Update 4",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-194" : {
      "gaDate" : "2010-03-30",
      "level" : "Update 5",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-238" : {
      "gaDate" : "2011-01-13",
      "level" : "Update 6",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-274" : {
      "gaDate" : "2011-07-21",
      "level" : "Update 7",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-308" : {
      "gaDate" : "2012-02-20",
      "level" : "Update 8",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-348" : {
      "gaDate" : "2013-01-07",
      "level" : "Update 9",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-371" : {
      "gaDate" : "2013-10-01",
      "level" : "Update 10",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-398" : {
      "gaDate" : "2014-09-16",
      "level" : "Update 11",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-53" : {
      "gaDate" : "2007-11-07",
      "level" : "Update 1",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-8" : {
      "gaDate" : "2007-03-15",
      "level" : "",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.18-92" : {
      "gaDate" : "2008-05-21",
      "level" : "Update 2",
      "padv" : "rhel5_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.19-5.1" : {
      "gaDate" : "2009-03-01",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.21-0.1.2" : {
      "gaDate" : "2009-04-16",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.23-0.1.1" : {
      "gaDate" : "2009-06-09",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.25-0.1.1" : {
      "gaDate" : "2009-07-23",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.29-0.1.1" : {
      "gaDate" : "2009-08-20",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.37-0.1.1" : {
      "gaDate" : "2009-11-02",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.39-0.3.1" : {
      "gaDate" : "2009-12-02",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.42-0.1.1" : {
      "gaDate" : "2010-01-14",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.45-0.1.1" : {
      "gaDate" : "2010-03-02",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.48-0.1.1" : {
      "gaDate" : "2010-07-20",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.48-0.12.1" : {
      "gaDate" : "2010-09-22",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.48-0.6.1" : {
      "gaDate" : "2010-08-09",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.27.54-0.2.1" : {
      "gaDate" : "2010-10-26",
      "level" : "",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32-131.0.15" : {
      "gaDate" : "2011-05-19",
      "level" : "Update 1",
      "padv" : "rhel6_x86_64",
      "platform" : "Linux"
   },
   "2.6.32-220" : {
      "gaDate" : "2011-12-06",
      "level" : "Update 2",
      "padv" : "rhel6_x86_64",
      "platform" : "Linux"
   },
   "2.6.32-279" : {
      "gaDate" : "2012-06-20",
      "level" : "Update 3",
      "padv" : "rhel6_x86_64",
      "platform" : "Linux"
   },
   "2.6.32-358" : {
      "gaDate" : "2013-02-21",
      "level" : "Update 4",
      "padv" : "rhel6_x86_64",
      "platform" : "Linux"
   },
   "2.6.32-431" : {
      "gaDate" : "2013-11-21",
      "level" : "Update 5",
      "padv" : "rhel6_x86_64",
      "platform" : "Linux"
   },
   "2.6.32-504" : {
      "gaDate" : "2014-10-14",
      "level" : "Update 6",
      "padv" : "rhel6_x86_64",
      "platform" : "Linux"
   },
   "2.6.32-573" : {
      "gaDate" : "2015-07-22",
      "level" : "Update 7",
      "padv" : "rhel6_x86_64",
      "platform" : "Linux"
   },
   "2.6.32-71" : {
      "gaDate" : "2010-11-09",
      "level" : "",
      "padv" : "rhel6_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.12-0.7.1" : {
      "gaDate" : "2010-06-01",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.13-0.4.1" : {
      "gaDate" : "2010-07-01",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.13-0.5.1" : {
      "gaDate" : "2010-08-02",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.19-0.2.1" : {
      "gaDate" : "2010-09-13",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.19-0.3.1" : {
      "gaDate" : "2010-09-22",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.23-0.3.1" : {
      "gaDate" : "2010-10-13",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.24-0.2.1" : {
      "gaDate" : "2010-11-10",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.27-0.2.2" : {
      "gaDate" : "2011-01-14",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.29-0.3.1" : {
      "gaDate" : "2011-03-08",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.36-0.5.2" : {
      "gaDate" : "2011-04-27",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.43-0.4.1" : {
      "gaDate" : "2011-07-25",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.45-0.3.2" : {
      "gaDate" : "2011-09-01",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.46-0.3.1" : {
      "gaDate" : "2011-10-07",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.49-0.3.1" : {
      "gaDate" : "2011-12-13",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.54-0.3.1" : {
      "gaDate" : "2012-02-06",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.59-0.13.1" : {
      "gaDate" : "2014-06-17",
      "level" : "SP1 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.59-0.15.2" : {
      "gaDate" : "2014-09-16",
      "level" : "SP1 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.59-0.17.1" : {
      "gaDate" : "2014-12-24",
      "level" : "SP1 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.59-0.19.1" : {
      "gaDate" : "2015-04-02",
      "level" : "SP1 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.59-0.3.1" : {
      "gaDate" : "2012-05-14",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.59-0.7.1" : {
      "gaDate" : "2012-07-23",
      "level" : "SP1",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.32.59-0.9.1" : {
      "gaDate" : "2014-01-24",
      "level" : "SP1 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.104" : {
      "gaDate" : "2004-07-29",
      "level" : "",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.111" : {
      "gaDate" : null,
      "level" : "",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.111.19" : {
      "gaDate" : "2004-12-13",
      "level" : "",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.111.30" : {
      "gaDate" : "2005-01-17",
      "level" : "",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.111.5" : {
      "gaDate" : "2004-11-17",
      "level" : "",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.139" : {
      "gaDate" : "2005-01-01",
      "level" : "SP1",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.145" : {
      "gaDate" : "2005-01-28",
      "level" : "SP1",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.147" : {
      "gaDate" : "2005-02-24",
      "level" : "SP1",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.151" : {
      "gaDate" : "2005-03-17",
      "level" : "SP1",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.155.29" : {
      "gaDate" : "2005-06-03",
      "level" : "SP1",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.191" : {
      "gaDate" : "2005-07-01",
      "level" : "SP2",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.193" : {
      "gaDate" : "2005-07-29",
      "level" : "SP2",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.201" : {
      "gaDate" : "2005-08-28",
      "level" : "SP2",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.202.7" : {
      "gaDate" : "2005-12-02",
      "level" : "SP2",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.244" : {
      "gaDate" : "2005-12-01",
      "level" : "SP3",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.252" : {
      "gaDate" : "2006-02-16",
      "level" : "SP2",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.257" : {
      "gaDate" : "2006-05-15",
      "level" : "SP3",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.267" : {
      "gaDate" : "2006-06-23",
      "level" : "SP3",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.276" : {
      "gaDate" : "2006-07-25",
      "level" : "SP3",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.282" : {
      "gaDate" : "2006-08-30",
      "level" : "SP3",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.283" : {
      "gaDate" : "2006-12-04",
      "level" : "SP3",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.286" : {
      "gaDate" : "2007-06-04",
      "level" : "SP3",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.287.3" : {
      "gaDate" : "2007-10-09",
      "level" : "SP3",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.308" : {
      "gaDate" : "2007-12-01",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.311" : {
      "gaDate" : "2008-03-31",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.312" : {
      "gaDate" : "2008-06-24",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.314" : {
      "gaDate" : "2008-09-30",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.315" : {
      "gaDate" : "2008-12-04",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.316" : {
      "gaDate" : "2009-03-27",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.317" : {
      "gaDate" : "2009-06-16",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.318" : {
      "gaDate" : "2009-08-21",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.319" : {
      "gaDate" : "2009-08-28",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.321" : {
      "gaDate" : "2009-11-16",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.322" : {
      "gaDate" : "2010-02-17",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.323" : {
      "gaDate" : "2010-08-26",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.324" : {
      "gaDate" : "2010-09-22",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.325" : {
      "gaDate" : "2011-02-11",
      "level" : "SP4",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.5-7.97" : {
      "gaDate" : "2004-07-01",
      "level" : "",
      "padv" : "sles9_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-100" : {
      "gaDate" : "2011-02-16",
      "level" : "Update 9",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-11" : {
      "gaDate" : "2005-06-08",
      "level" : "Update 1",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-22" : {
      "gaDate" : "2005-10-05",
      "level" : "Update 2",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-34" : {
      "gaDate" : "2006-03-12",
      "level" : "Update 3",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-42" : {
      "gaDate" : "2006-08-10",
      "level" : "Update 4",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-5" : {
      "gaDate" : "2005-02-15",
      "level" : "",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-55" : {
      "gaDate" : "2007-05-01",
      "level" : "Update 5",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-67" : {
      "gaDate" : "2007-11-15",
      "level" : "Update 6",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-78" : {
      "gaDate" : "2008-07-29",
      "level" : "Update 7",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "2.6.9-89" : {
      "gaDate" : "2009-05-19",
      "level" : "Update 8",
      "padv" : "rhel4_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.15.1" : {
      "gaDate" : "2014-02-04",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.21.1" : {
      "gaDate" : "2014-04-15",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.29.1" : {
      "gaDate" : "2014-05-15",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.31.1" : {
      "gaDate" : "2014-06-11",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.35.1" : {
      "gaDate" : "2014-07-17",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.40.1" : {
      "gaDate" : "2014-10-22",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.42.1" : {
      "gaDate" : "2014-12-23",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.46.1" : {
      "gaDate" : "2014-12-23",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.47.52.1" : {
      "gaDate" : "2015-03-27",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.47.55-default" : {
      "gaDate" : "2015-07-02",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.47.55.1" : {
      "gaDate" : "2015-07-02",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.5.1" : {
      "gaDate" : "2013-11-21",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.63.1" : {
      "gaDate" : "2015-07-01",
      "level" : "SP4",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.7.15.1" : {
      "gaDate" : "2014-01-27",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.7.17.1" : {
      "gaDate" : "2014-02-12",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.7.19.1" : {
      "gaDate" : "2014-05-21",
      "level" : "SP2 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.7.21.1" : {
      "gaDate" : "2014-06-24",
      "level" : "SP2 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.7.23.1" : {
      "gaDate" : "2014-09-09",
      "level" : "SP2 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.7.27.1" : {
      "gaDate" : "2014-12-24",
      "level" : "SP2 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.7.29.1" : {
      "gaDate" : "2015-03-11",
      "level" : "SP2 - LTSS",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-0.8.1" : {
      "gaDate" : "2013-11-22",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.101-63-default" : {
      "gaDate" : "2015-07-01",
      "level" : "SP4",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.13-0.27-default" : {
      "gaDate" : "2012-02-01",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.13-0.27.1" : {
      "gaDate" : "2012-02-01",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.26-0.7.6" : {
      "gaDate" : "2012-04-23",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.31-0.9.1" : {
      "gaDate" : "2012-06-01",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.34-0.7.9" : {
      "gaDate" : "2012-06-25",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.38-0.5.1" : {
      "gaDate" : "2012-08-21",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.42-0.7.3" : {
      "gaDate" : "2012-10-15",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.51-0.7.9.1" : {
      "gaDate" : "2012-12-19",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.58-0.6.2.1" : {
      "gaDate" : "2013-02-07",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.58-0.6.6.1" : {
      "gaDate" : "2013-02-25",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.74-0.6.10.1" : {
      "gaDate" : "2013-05-31",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.74-0.6.6.2" : {
      "gaDate" : "2013-05-07",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.74-0.6.8.1" : {
      "gaDate" : "2013-05-22",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.76-0.11.1" : {
      "gaDate" : "2013-07-01",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.80-0.5.1" : {
      "gaDate" : "2013-06-17",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.80-0.7.1" : {
      "gaDate" : "2013-07-05",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.82-0.7.9" : {
      "gaDate" : "2013-07-11",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.93-0.5.1" : {
      "gaDate" : "2013-09-20",
      "level" : "SP2",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.0.93-0.8.2" : {
      "gaDate" : "2013-09-21",
      "level" : "SP3",
      "padv" : "sles11_x86_64",
      "platform" : "Linux"
   },
   "3.10.0-123" : {
      "gaDate" : "2014-06-09",
      "level" : "",
      "padv" : "rhel7_x86_64",
      "platform" : "Linux"
   },
   "3.10.0-229" : {
      "gaDate" : "2015-03-05",
      "level" : "Update 1",
      "padv" : "rhel7_x86_64",
      "platform" : "Linux"
   },
   "3.12.28-4.6" : {
      "gaDate" : "2014-11-01",
      "level" : "",
      "padv" : "sles12_x86_64",
      "platform" : "Linux"
   },
   "3.12.32-33.1" : {
      "gaDate" : "2015-01-16",
      "level" : "",
      "padv" : "sles12_x86_64",
      "platform" : "Linux"
   },
   "3.12.36-38.1" : {
      "gaDate" : "2015-01-30",
      "level" : "",
      "padv" : "sles12_x86_64",
      "platform" : "Linux"
   },
   "3.12.38-44.1" : {
      "gaDate" : "2015-03-18",
      "level" : "",
      "padv" : "sles12_x86_64",
      "platform" : "Linux"
   },
   "3.12.39-47.1" : {
      "gaDate" : "2015-04-02",
      "level" : "",
      "padv" : "sles12_x86_64",
      "platform" : "Linux"
   },
   "3.12.43-52.6.1" : {
      "gaDate" : "2015-06-16",
      "level" : "",
      "padv" : "sles12_x86_64",
      "platform" : "Linux"
   },
   "3.12.44-52.10.1" : {
      "gaDate" : "2015-07-31",
      "level" : "",
      "padv" : "sles12_x86_64",
      "platform" : "Linux"
   },
   "TBA" : {
      "gaDate" : null,
      "level" : "Update 2",
      "padv" : "rhel7_x86_64",
      "platform" : "Linux"
   }
}'
;

    return $response;
}

package Pkg::VRTScps62::Common;

sub check_nic_sys {
    my ($sys,$nic) = @_;
    my ($ret);

    # ([a-z]+(\d)+) -- pattern begins with lowercase chars and ends with digits
    # {1,2} -- above pattern can repeat exactly once('hme0') or twice ('e1000g0')
    if (EDRu::isnic($nic)) {
        # Valid NIC device
        # Check if NIC is configured on the system
        if ($sys->hpux()) {
            $sys->cmd("/usr/sbin/nwmgr | _cmd_grep '^$nic'");
        } else {
            $sys->cmd("_cmd_ifconfig -a | _cmd_grep '^$nic'");
        }
        (EDR::cmdexit()) ? return 0 : return 1;
    }
    return 0;
}

1;