#HOTFIX_VERSION:cpi-linux-6.0.2.300
package Padv::Linux;

sub detect_linux_by_dmidecode_sys {
    my ($padv, $sys) = @_;

    my $out = $sys->cmd("_cmd_dmidecode 2>/dev/null");

    my ($flag, $bios_vendor, $mfgr, $product) = ("", "", "", "");
    foreach (split(/\n+/, $out)) {
        if (/^\S/) {
            if (/^BIOS Information/) {
                $flag = "bios_info";
            } elsif (/^System Information/) {
                $flag = "system_info";
            } else {
                $flag = "other";
            }
            next;
        }

        if ($flag eq "bios_info") {
            $bios_vendor = $1 if (/Vendor:\s*(.*)/i);
        }
        if ($flag eq "system_info") {
            $mfgr = $1 if (/Manufacturer:\s*(.*)/i);
            $product = $1 if (/Product Name:\s*(.*)/i);
        }
    }

    $sys->{virtual_type} = $padv->VIRT_VMWARE if ($mfgr =~ /VMWare/i);
    $sys->{virtual_type} = $padv->VIRT_KVM . " or " . $padv->VIRT_QEMU if ($bios_vendor =~ /QEMU/i);
    $sys->{virtual_type} = $padv->VIRT_VIRTUALPC if ($mfgr =~ /microsoft/i && $product =~ /virtual machine/i);
    return;
}

package Prod::VCS60::Common;
sub is_ip_valid {
    my ($prod, $ip) = @_;
    my ($is_ipv4, $ipv4_addr, $ipv6_addr, @octets, $ocnt);
    my (@ip_parts, $ip_uniq, $cnt, $bcnt, $tcnt);

    $is_ipv4 = EDRu::ip_is_ipv4($ip);
    if ($is_ipv4) {
        @octets = split(/\./m, $ip);
        $ocnt = @octets;
        if ($ocnt == 4) {
            return 0 if ($ip =~ /^0+\./m);
            return 1;
        } else {
            return 0;
        }
    }

    if (!($ip =~ /:/m)) {
         return 0;
    }

    if ($ip =~ /\//m) {
        if (!($ip =~ /^.+\/\d+$/m)) {
            return 0;
        }

        @ip_parts = split(/\//m, $ip);
        $cnt = @ip_parts;
        if ($cnt > 2) {
            return 0;
        }

        $ip_uniq = $ip_parts[0];

        $bcnt = $ip_parts[1] + 0;
        if (!($bcnt <= 128)) {
            return 0;
        }

    } else {
        $ip_uniq = $ip;
    }


    if (!EDRu::ip_is_ipv6($ip_uniq)) {
        return 0;
    }

    if ($ip_uniq =~ /\./m) {
        @ip_parts = split(/\:/m, $ip_uniq);
        $cnt = @ip_parts;
        $cnt--;
        $ipv4_addr = $ip_parts[$cnt];
        @octets = split(/\./m, $ipv4_addr);
        $ocnt = @octets;
        if ($ocnt != 4) {
            return 0;
        }

        $ipv6_addr = '';
        my $i =0;
        while ($i < $cnt) {
            $ipv6_addr = $ipv6_addr."$ip_parts[$i]:";
            $i++;
        }

        $tcnt = 6;
    } else {
        $ipv6_addr = $ip_uniq;
        $tcnt = 8;
    }

    if (!($ipv6_addr =~ /::/m)) {
        @ip_parts = split(/:/m, $ipv6_addr);
        $cnt = @ip_parts;
        if ($cnt == $tcnt) {
            return 1;
        } else {
            return 0;
        }
    }

    return 1;
}

# Validate volume name and size
# Return 0 means things are okay
# Return 1 means volume name already exists
#     or invalid diskgroup specified
# Return 2 means volume size specified cannot be accommodated
# Return 3 means unfathomable error
sub validate_vol {
    my ($prod, $volname, $volsize, $unavailsize, $dgname, $master, $enable_mirroring) = @_;
    my ($msg, $status, $ret, $temp, $availsize);
    my $web = Obj::web();
    $status = 0;

    # Check if the volume name is non-existent, if so check size available
    $ret = $master->cmd("_cmd_vxprint -g $dgname $volname");
    if (!EDR::cmdexit()) {
        $msg = Msg::new("(Volume with the name $volname already exists)");
        $msg->print;
#        $web->web_script_form("alert", $msg->{msg}) if (Obj::webui());
        return 1;
    } elsif ($ret =~ /Record $volname not found/m) { # What's the EDR::cmdexit() here, 11 or 2816?
        $ret = $master->cmd("_cmd_vxassist -g $dgname maxsize layout=mirror") if ($enable_mirroring == 1);
        $ret = $master->cmd("_cmd_vxassist -g $dgname maxsize ") if ($enable_mirroring == 0);
        if (EDR::cmdexit()) {
            if ($ret =~ /"Diskgroup $dgname not found"/m) {
                $msg = Msg::new("(Diskgroup $dgname doesn't seem to exist anymore)");
                $msg->print;
                $web->web_script_form("alert", $msg->{msg}) if (Obj::webui());
                return 1;
            } else {
                $msg = Msg::new("Inconsistency error with 'vxassist' command on $master->{sys}. Exiting validating volume.");
                $msg->log;
                $web->web_script_form("alert", $msg->{msg}) if (Obj::webui());
                return 3;
            }
        } else {
# Do the size check here
            ($temp, $temp, $temp, $temp, $availsize) = split(/\s+/m, $ret);
            $availsize =~ s/\D+//mg;
            $msg = Msg::new("Availavle volume size: $availsize MB");
            $msg->log;
            if (((0+$availsize) - (0+$unavailsize)) >= (0+$volsize)) {
                $msg = Msg::new("Volume with size $volsize MB can be successfully created");
                $msg->log;
                return 0;
            } else {
                $msg = Msg::new("(Volume with the size $volsize MB cannot be successfully created due to insufficient space)");
                $msg->print;
                $web->web_script_form("alert", $msg->{msg}) if (Obj::webui());
                return 2;
            }
        }
    } else {
        $msg = Msg::new("Some error with 'vxprint' command on $master->{sys}. Exiting validating volume.");
        $msg->log;
        $web->web_script_form("alert", $msg->{msg}) if (Obj::webui());
        return 2;
    }
}

package Pkg::VRTScps60::Common;
sub verify_responsefile {
    my $pkg = shift;
    my ($cfg,$msg,$num,$sys0,$vxfen_pkg);
    my (@nics,@nw_hosts,@ports,@systems,@vips);
    my ($nic_prefix,$vip_prefix,$vcs);
    $cfg = Obj::cfg();
    @systems = @{CPIC::get('systems')};
    $sys0 = $systems[0];
    $vcs = $pkg->prod('VCS60');
    $vxfen_pkg = $pkg->pkg('VRTSvxfen60');
    $nic_prefix = 'cpsnic';
    $vip_prefix = 'cpsvip';
    
    if (!$cfg->{opt}{configcps}) {
        $msg = Msg::new("\$CFG{opt}{configcps} must be set in responsefile");
        $msg->die;
    }
    if (!$cfg->{cps_sfha_config} && !$cfg->{cps_singlenode_config} && !$cfg->{cps_unconfig}) {
        $msg = Msg::new("\$CFG{cps_sfha_config}, \$CFG{cps_singlenode_config} or \$CFG{cps_unconfig} must be set in responsefile");
        $msg->die;
    }
    if (($cfg->{cps_sfha_config} && $cfg->{cps_singlenode_config}) ||
        ($cfg->{cps_sfha_config} && $cfg->{cps_unconfig}) ||
        ($cfg->{cps_singlenode_config} && $cfg->{cps_unconfig})) {
        $msg = Msg::new("\$CFG{cps_sfha_config}, \$CFG{cps_singlenode_config} or \$CFG{cps_unconfig} cannot be set in responsefile at the same time");
        $msg->die;
    }
    if ($cfg->{cps_singlenode_config} || $cfg->{cps_sfha_config}) {
        if ((!$cfg->{cps_db_dir}) || (!$cfg->{cpsname}) || (!defined($cfg->{cps_security})) || (!$cfg->{cps_vips}) ||
             (!$cfg->{cps_ports}) || (!$cfg->{cps_nic_list}) || (!$cfg->{cps_vip2nicres_map})) {
            $msg = Msg::new("\$CFG{cpsname}, \$CFG{cps_db_dir}, \$CFG{cps_vips}, \$CFG{cps_ports}, \$CFG{cps_nic_list}, \$CFG{cps_vip2nicres_map}, \$CFG{cps_security} must be set in responsefile");
            $msg->die;
        }
        if (!$pkg->validate_cps_db_dir_name($cfg->{cps_db_dir})) {
            $msg = Msg::new("\$CFG{cps_db_dir} in responsefile has an invalid value");
            $msg->die;
        }
        if (!check_systemname($cfg->{cpsname})) {
            $msg = Msg::new("\$CFG{cpsname} in responsefile has an invalid value");
            $msg->die;
        }
        if (($cfg->{cps_security} != 0) && ($cfg->{cps_security} != 1)) {
            $msg = Msg::new("\$CFG{cps_security} in responsefile has an invalid value");
            $msg->die;
        }

        if (ref($cfg->{cps_vips}) ne "ARRAY") {
            $msg = Msg::new("\$CFG{cps_vips} in responsefile is not a reference to an array");
            $msg->die;
        }
        @vips = @{$cfg->{cps_vips}};
        if (!EDRu::arr_isuniq(@vips)) {
            $msg = Msg::new("\$CFG{cps_vips} in responsefile has duplicate value");
            $msg->die;
        }
        for my $vip(@vips) {
            if (!$vcs->is_ip_valid($vip)) {
                $msg = Msg::new("\$CFG{cps_vips} in responsefile has an invalid value");
                $msg->die;
            }
        }
        if (ref($cfg->{cps_ports}) ne "ARRAY") {
            $msg = Msg::new("\$CFG{cps_ports} in responsefile is not a reference to an array");
            $msg->die;
        }
        @ports = @{$cfg->{cps_ports}};
        for my $port(@ports) {
            if (!Pkg::VRTSvxfen60::Common::validate_cps_port($port,$pkg->{defport})) {
                $msg = Msg::new("\$CFG{cps_ports} in responsefile has an invalid value");
                $msg->die;
            }
        }
        # verify cps_nic_list, cps_network_hosts and cps_vip2nicres_map
        for my $vipkey(keys %{$cfg->{cps_nic_list}}) {
            if ($vipkey =~ /^$vip_prefix(\d+)$/m) {
                $num = $1;
                if (($num < 1) || ($num > (scalar @vips))) {
                    $msg = Msg::new("\$CFG{cps_nic_list} in responsefile has an invalid value");
                    $msg->die;
                }
                if (ref($cfg->{cps_nic_list}{"$vipkey"}) ne "ARRAY") {
                    $msg = Msg::new("\$CFG{cps_nic_list} in responsefile has an invalid value");
                    $msg->die;
                }

                @nics = @{$cfg->{cps_nic_list}{"$vipkey"}};
                if ($#nics != $#systems) {
                    $msg = Msg::new("\$CFG{cps_nic_list} in responsefile has an invalid value");
                    $msg->die;
                }
            } else {
                $msg = Msg::new("\$CFG{cps_nic_list} in responsefile has an invalid value");
                $msg->die;
            }
        }
        for my $vipkey(keys %{$cfg->{cps_vip2nicres_map}}) {
            if (!EDRu::inarr($vipkey,@vips)) {
                $msg = Msg::new("\$CFG{cps_vip2nicres_map} in responsefile has an invalid value");
                $msg->die;
            }
            $num = $cfg->{cps_vip2nicres_map}{"$vipkey"};
            if (($num < 1) || ($num > (scalar @vips))) {
                $msg = Msg::new("\$CFG{cps_vip2nicres_map} in responsefile has an invalid value");
                $msg->die;
            }
        }
        if (!$cfg->{cps_netmasks} && !$cfg->{cps_prefix_length}) {
            $msg = Msg::new("\$CFG{cps_netmasks} or \$CFG{cps_prefix_length} must be set in responsefile");
            $msg->die;
        } elsif ($cfg->{cps_netmasks}) {
           if (ref($cfg->{cps_netmasks}) ne "ARRAY") {
                $msg = Msg::new("\$CFG{cps_netmasks} in responsefile is not a reference to an array");
                $msg->die;
           }
            for my $netmask(@{$cfg->{cps_netmasks}}) {
                if (!EDRu::ip_is_ipv4($netmask) || ($netmask =~ /^0+\./m)) {
                    $msg = Msg::new("\$CFG{cps_netmasks} in responsefile has an invalid value");
                    $msg->die;
                }
            }
        } elsif ($cfg->{cps_prefix_length}) {
           if (ref($cfg->{cps_prefix_length}) ne "ARRAY") {
                $msg = Msg::new("\$CFG{cps_prefix_length} in responsefile has an invalid value");
                $msg->die;
           }
            for my $prefix_len(@{$cfg->{cps_prefix_length}}) {
                if (!$pkg->validate_prefix_length($prefix_len)) {
                    $msg = Msg::new("\$CFG{cps_prefix_length} in responsefile has an invalid value");
                    $msg->die;
                }
            }
        }
        for my $nickey(keys %{$cfg->{cps_network_hosts}}) {
            if ($nickey =~ /^$nic_prefix(\d+)/m) {
                $num = $1;
                if ($num < 1) {
                    $msg = Msg::new("\$CFG{cps_network_hosts} in responsefile has an invalid value");
                    $msg->die;
                }
                if (ref($cfg->{cps_network_hosts}{"$nickey"}) ne "ARRAY") {
                    $msg = Msg::new("\$CFG{cps_network_hosts} in responsefile has an invalid value");
                    $msg->die;
                }
                @nw_hosts = @{$cfg->{cps_network_hosts}{"$nickey"}};
                for my $nw_host(@nw_hosts) {
                    if (!$vcs->is_ip_valid($nw_host)) {
                        $msg = Msg::new("\$CFG{cps_network_hosts} in responsefile has an invalid value");
                        $msg->die;
                    }
                }
            } else {
                $msg = Msg::new("\$CFG{cps_network_hosts} in responsefile has an invalid value");
                $msg->die;
            }
        }
        if ($cfg->{cps_fips_mode} && ($cfg->{cps_fips_mode} != 0) && ($cfg->{cps_fips_mode} != 1)) {
            $msg = Msg::new("\$CFG{cps_fips_mode} in responsefile has an invalid value");
            $msg->die;
        }
        if ($cfg->{cps_sfha_config}) {
            if ((!$cfg->{cps_diskgroup}) || (!$cfg->{cps_volume})) {
                $msg = Msg::new("\$CFG{cps_diskgroup}, \$CFG{cps_volume} must be set in responsefile");
                $msg->die;
            }
            # $cfg->{cps_diskgroup} $cfg->{cps_volume} $cfg->{cps_newdg_disks} $cfg->{cps_newvol_volsize}
            if (!$vxfen_pkg->validate_dgname($cfg->{cps_diskgroup})) {
                $msg = Msg::new("\$CFG{cps_diskgroup} in responsefile has an invalid value");
                $msg->die;
            }
            if (!$pkg->validate_dg_vol_name($cfg->{cps_volume})) {
                $msg = Msg::new("\$CFG{cps_volume} in responsefile has an invalid value");
                $msg->die;
            }
            if ($cfg->{cps_newvol_volsize}) {
                if (!$pkg->validate_vol_size($pkg->{cpsvol_min_size},$cfg->{cps_newvol_volsize})) {
                    $msg = Msg::new("\$CFG{cps_newvol_volsize} in responsefile has an invalid value");
                    $msg->die;
                }
            }
            if (!$cfg->{cps_newdg_disks}) {
                # using existing dg
                if (!$vxfen_pkg->validate_diskgroup($cfg->{cps_diskgroup})) {
                    $msg = Msg::new("$cfg->{cps_diskgroup} does not exist on all the systems");
                    $msg->die;
                }
                # using existing diskgroup
                $sys0->cmd("_cmd_vxdg -q list 2> /dev/null | _cmd_grep -w $cfg->{cps_diskgroup}");
                if (EDR::cmdexit()) {
                    $sys0->cmd("_cmd_vxdg -t import $cfg->{cps_diskgroup}");
                }
                if (EDR::cmdexit()) {
                    $msg = Msg::new("Failed to import $cfg->{cps_diskgroup} on $sys0->{sys}");
                    $msg->die;
                }
                if ($cfg->{cps_newvol_volsize}) {
                    # creating new vol on existing dg
                    if ($vcs->validate_vol($cfg->{cps_volume}, $cfg->{cps_newvol_volsize}, 0, $cfg->{cps_diskgroup},$sys0,$cfg->{cps_enable_mirroring})) {
                        $msg = Msg::new("Failed to check $cfg->{cps_volume} on $sys0->{sys}");
                        $msg->die;
                    }
                } else {
                    # using existing vol on existing dg
                    $sys0->cmd("_cmd_vxprint -g $cfg->{cps_diskgroup} $cfg->{cps_volume}");
                    if (EDR::cmdexit()) {
                        $msg = Msg::new("Failed to check $cfg->{cps_volume} on $sys0->{sys}");
                        $msg->die;
                    }
                }
            } else {
                # creating new dg and vol
                if (!$cfg->{cps_newvol_volsize}) {
                    $msg = Msg::new("\$CFG{cps_newvol_volsize} must be set in responsefile");
                    $msg->die;
                }
                if (!$vxfen_pkg->validate_diskgroup($cfg->{cps_diskgroup},1)) {
                    $msg = Msg::new("$cfg->{cps_diskgroup} already exists on the systems");
                    $msg->die;
                }
            }
        }
    }
    return;
}



sub ask_cps_info {
    my ($pkg,$config_flag) = @_;
    my (@ports,@vips,$answer,$backopt,$conf,$cps_db_dir,$cps_security,$cpsname,$defport,$invalid,$msg,$num,$cps_fips_mode);
    my ($defopt,$sys0,$ret,$web,$result,$vcs,$sec_or_fips);
    $sys0 = ${CPIC::get('systems')}[0];
    $vcs = $pkg->prod("VCS60");
    $backopt = 1;
    $defport = $pkg->{defport};
    $cps_security = 0;
    $web = Obj::web();
    if(Obj::webui()) {
        $result = $web->web_script_form('configcps',$pkg,$config_flag);
        return "__back__" if ($result eq '__back__');
        $cpsname = $result->{cpsname};
        @vips = @{$result->{vips}};
        @ports = @{$result->{ports}};
        $cps_db_dir = $result->{cps_db_dir};
        $msg = Msg::new("Symantec recommends secure communication between the CP server and application clusters. Enabling security requires Symantec Product Authentication Service to be installed and configured on the cluster. Do you want to enable Security for the communications?");
        Msg::n();
        $answer = $msg->ayny('',$backopt);
        return $answer if (EDR::getmsgkey($answer,'back'));
        $cps_security = ($answer eq 'Y') ?  1: 0 ;
        # Check if the AT service resource is online
        if ($cps_security) {
            unless ($vcs->is_secure_cluster($sys0)) {
                $msg = Msg::new("Security is not configured for this cluster.");
                $web->web_script_form('alert',$msg->{msg});
                $web->{complete_failed} = 1;
                $ret = 1;
                return ($ret,$conf);
            }
            $sys0->cmd("_cmd_ps -ef | _cmd_grep -w 'vcsauthserver' | _cmd_grep -v grep");
            if (EDR::cmdexit()) {
                $msg = Msg::new("vcsauthserver is not running on the system. Check if security is enabled for this cluster and retry");
                $web->web_script_form('alert',$msg->{msg});
                $web->{complete_failed} = 1;
                $ret = 1;
                return ($ret,$conf);
            }

            if($vcs->is_fips_cluster($sys0)) {
                $sec_or_fips = Msg::new("security with fips")->{msg};
                $cps_fips_mode = 1;
            } else {
                $sec_or_fips = Msg::new("secure")->{msg};
                $cps_fips_mode = 0;
            }
            $msg=Msg::new("CP Server will be configured in $sec_or_fips mode, since the cluster is configured in this mode.");
            $web->web_script_form('alert',$msg->{msg});
        }
    } else {
        Msg::n();
        # Getting CPS Server Name
        while (1) {
            $msg = Msg::new("Enter the name of the CP Server:");
            $answer = $msg->ask('','',$backopt);
            next if (!check_systemname($answer));
            return $answer if (EDR::getmsgkey($answer,'back'));
            $cpsname = $answer;
            last;
        }
        Msg::n();

        # Get Virtual IP from the user
        while (1) {
            $invalid = 0;
            $msg = Msg::new("Enter valid IP addresses for Virtual IPs for the CP Server, separated by space:");
            $answer = $msg->ask('','',$backopt);
            return $answer if (EDR::getmsgkey($answer,'back'));
            #$answer = EDRu::despace($answer);
            @vips = split(/\s+/, $answer);
            if (!EDRu::arr_isuniq(@vips)) {
                $msg = Msg::new("Duplicate inputs. Input again");
                $msg->print;
                next;
            }
            for my $vip(@vips) {
                if (!$vcs->is_ip_valid($vip)) {
                    $msg = Msg::new("$vip is not a valid IP address. Input again");
                    $msg->print;
                    $invalid = 1;
                }
            }
            next if ($invalid);
            last;
        }
        Msg::n();
        # Get port number for CP Server 
        while (1) {
            $invalid = 0;
            $msg = Msg::new("Enter corresponding port number for each Virtual IP address in the range [49152, 65535], separated by space, or simply accept the default port suggested:");
            $answer = $msg->ask($defport,'',$backopt);
            return $answer if (EDR::getmsgkey($answer,'back'));
            #$answer = EDRu::despace($answer);
            @ports = split(/\s+/, $answer);
            for my $port(@ports) {
                if (!Pkg::VRTSvxfen60::Common::validate_cps_port($port,$defport)) {
                    $invalid = 1;
                }
            }
            next if ($invalid);
            if (@ports > @vips) {
                $msg = Msg::new("The number of ports is larger than the number of Virtual IPs. Input again");
                $msg->print;
                next;
            } elsif (@ports != @vips) {
                $num = @vips - @ports;
                foreach(1..$num) {
                    push(@ports,$defport);
                }
            }
            last;
        }
        # Get security information
        $msg = Msg::new("Symantec recommends secure communication between the CP server and application clusters. Enabling security requires Symantec Product Authentication Service to be installed and configured on the cluster. Do you want to enable Security for the communications?");
        Msg::n();
        $answer = $msg->ayny('',$backopt);
        return $answer if (EDR::getmsgkey($answer,'back'));
        $cps_security = ($answer eq 'Y') ?  1: 0 ;
        # Check if the AT service resource is online
        if ($cps_security) {
            unless ($vcs->is_secure_cluster($sys0)) {
                $msg = Msg::new("Security is not configured for this cluster.");
                $msg->error();
                $ret = 1;
                return ($ret,$conf);
            }
            $sys0->cmd("_cmd_ps -ef | _cmd_grep -w 'vcsauthserver' | _cmd_grep -v grep");
            if (EDR::cmdexit()) {
                $msg = Msg::new("vcsauthserver is not running on the system. Check if security is enabled for this cluster and retry");
                $msg->error();
                $ret = 1;
                return ($ret,$conf);
            }

            Msg::n();
            if($vcs->is_fips_cluster($sys0)) {
                $sec_or_fips = Msg::new("security with fips")->{msg};
                $cps_fips_mode = 1;
            } else {
                $sec_or_fips = Msg::new("secure")->{msg};
                $cps_fips_mode = 0;
            }
            $msg=Msg::new("CP Server will be configured in $sec_or_fips mode, since the cluster is configured in this mode.");
            $msg->print;
            sleep(1);
        }

        Msg::n();
        # Get location of CPS database
        $msg = Msg::new("CP Server uses an internal database to store the client information.");
        $msg->print;
        if ($config_flag->{sfha_config}) {
            $msg = Msg::new("As the CP Server is being configured on SFHA cluster, the database should reside on shared storage with vxfs file system. Please refer to documentation for information on setting up of shared storage for CP server database.");
            $msg->printn;
        }
        if ($config_flag->{singlenode_config}) {
            $msg = Msg::new("As the CP Server is being configured on a single node VCS, database can reside on local file system.");
            $msg->printn;
        }
        while (1) {
            $defopt = ($config_flag->{sfha_config}) ? $pkg->{default_ha_db_dir} : $pkg->{default_db_dir};
            $msg = Msg::new("Enter absolute path of the database:");
            $answer = $msg->ask($defopt,'',$backopt);
            return $answer if (EDR::getmsgkey($answer,'back'));
            next if (!$pkg->validate_cps_db_dir_name($answer));
            $cps_db_dir= $answer;
            last;        
        }
    }
    $conf->{cpsname} = $cpsname;    
    $conf->{vips} = \@vips;
    $conf->{ports} = \@ports;
    $conf->{cps_db_dir} = $cps_db_dir;
    $conf->{cps_security} = $cps_security;
    $conf->{cps_fips_mode} = $cps_fips_mode;
    $ret = 0;
    return ($ret,$conf);
}



sub ask_vol_sys {
    my ($pkg,$sys,$dgname) = @_;
    my (@vollist,$volname,$volsize);
    my ($answer,$backopt,$cfg,$choice,$menuopt,$msg,$ret,$vcs);
    my (@disks,$enable_mirror,$disklist,$fs,$min_volsize,$mkret);
    $cfg = Obj::cfg();
    $fs = $pkg->prod('FS60');
    $vcs = $pkg->prod('VCS60');
    $min_volsize = $pkg->{cpsvol_min_size};
    @vollist = @{$pkg->get_vol_sys($sys,$dgname)};
    $msg = Msg::new("Select one of the options below for CP Server database volume:");
    $msg->print;
    $msg = Msg::new("Create a new volume on disk group $dgname");
    $menuopt = [$msg->{msg}];
    $msg = Msg::new("Using an existing volume on disk group $dgname");
    push (@{$menuopt},$msg->{msg}) if (@vollist>0);
    $msg = Msg::new("Enter the choice for a volume:");
    $choice = $msg->menu($menuopt);
    Msg::n();
    if($choice == 1) {
        # enable mirroring if more than one disks in selected disk group
        $disklist = $sys->cmd("_cmd_vxdisk -g $dgname list 2> /dev/null | _cmd_grep $dgname | _cmd_awk '{print \$1}'");
        if ($disklist) {
            @disks = split(/\n/,$disklist);
            $enable_mirror = 1 if (@disks > 1);
        }
        # ask volume name and size to create new volume
        while (1) {
            $msg = Msg::new("Enter the name of volume to be created on diskgroup $dgname:");
            $answer = $msg->ask();
            next if (!$pkg->validate_dg_vol_name($answer));
            $volname = $answer;
            $msg=Msg::new("Enter the volume size (in MB):");
            $answer = $msg->ask($min_volsize);
            next if (!$pkg->validate_vol_size($min_volsize,$answer));
            $volsize = $answer;
            next if ($vcs->validate_vol($volname, $volsize, 0, $dgname,$sys,$enable_mirror)); 
            last;
        }
        $msg = Msg::new("Creating volume '$volname' for CP Server database on $sys->{sys}");
        $msg->left;
        if ($enable_mirror) {
            $sys->cmd("_cmd_vxassist -g $dgname make $volname ${volsize}m layout=mirror");
        } else {
            $sys->cmd("_cmd_vxassist -g $dgname make $volname ${volsize}m");
        }
        if (EDR::cmdexit()) {
            $msg->right_failed();
            return 1;
        } else {
            $msg->right_done();
        }
        # make vxfs file system on the volume
        $mkret = $fs->make_vxfs_sys($sys,$dgname,$volname);
        Msg::log("Make fs return $mkret");
        $cfg->{cps_newvol_volsize} = $volsize;
        $cfg->{cps_enable_mirroring} = $enable_mirror;
    } else {
        $menuopt = [ @vollist ];
        $msg = Msg::new("Select one volume as CP Server database volume:");
        $choice = $msg->menu($menuopt,'','',$backopt,0,1);
        return $choice if (EDR::getmsgkey($choice,'back'));
        $volname = $vollist[$choice-1];
    }
    return ($ret, $volname);
}

sub addto_vcs_config {
    my ($pkg,$conf,$config_flag) = @_;
    my (@nodeid_list,@system_list,@systems,$answer,$count,$cps_blockdevice,$cps_diskgroup,$cps_volume,$msg,$out,$sys0,$validated);
    my (@cps_virtual_ip,@cps_netmasks,@cps_prefix_length,@is_ipv4_ipv6,$cps_netmask,$cps_prefix_len,$i,$j,$nic_count,$nic_same,$numnics,$numvips,$ret);
    my (%vip2nicres_map,%nics_taken_care_of,@nic_list,@nic_list_for_vip,@nic_set,@nic_set_for_system,@nicres_list,@network_hosts,@network_hosts_for_nic);
    my ($cfg,$defopt,$fs,$line,$nwhosts_same,$vxfen_pkg,$vcs);
    @systems = @{CPIC::get('systems')};
    $sys0 = $systems[0];
    $fs = $pkg->prod('FS60');
    $vcs = $pkg->prod('VCS60');
    $vxfen_pkg = $pkg->pkg('VRTSvxfen60');
    $cfg = Obj::cfg();
    my $web = Obj::web();
    Msg::title();
    $msg = Msg::new("Configuring CP Server Service Group (CPSSG) for this cluster");
    $msg->print;
    $out = $sys0->cmd('_cmd_hasys -list -clus localclus'); 
    if (EDR::cmdexit()) {
        Msg::n();
        $msg = Msg::new("Failed to list nodes of the cluster");
        $msg->error();
        return 1;
    }
    chomp($out);
    @system_list = split(/\n/,$out);
    $out = $sys0->cmd("_cmd_hasys -display | grep -i LLTNodeId | awk '{print \$3}'");
    chomp($out);
    @nodeid_list = split(/\n/,$out);
    $numvips = @{$conf->{vips}};
    $i = 0;
    foreach (@{$conf->{vips}}) {
        if (EDRu::ip_is_ipv4(${$conf->{vips}}[$i])) {
            $is_ipv4_ipv6[$i] = 0; 
        } else {
            $is_ipv4_ipv6[$i] = 1; 
        }
        $i++;
    }
    $conf->{is_ipv4_ipv6} = \@is_ipv4_ipv6;
    $conf->{system_list} = [ @system_list ];
    $conf->{nodeid_list} = [ @nodeid_list ];
    if (Cfg::opt('responsefile')) {
        $conf->{prefix_length} = $cfg->{cps_prefix_length} if ($cfg->{cps_prefix_length});
        $conf->{netmasks} = $cfg->{cps_netmasks} if ($cfg->{cps_netmasks});
        $conf->{vip2nicres_map} = $cfg->{cps_vip2nicres_map};
        $conf->{nicres_list} = [ @nicres_list ];
        for my $vipkey (keys %{$cfg->{cps_nic_list}}) {
            if ($vipkey =~ /cpsvip(\d+)/m) {
                $i = $1;
                $nic_list[$i-1] = $cfg->{cps_nic_list}{"$vipkey"};
            }
        }
        for my $nickey (keys %{$cfg->{cps_network_hosts}}) {
            if ($nickey =~ /cpsnic(\d+)/m) {
                $i = $1;
                $network_hosts[$i-1] = $cfg->{cps_network_hosts}{"$nickey"}; 
            }
        }
        $conf->{nic_list} = [ @nic_list ];
        $conf->{network_hosts} = [ @network_hosts ];
        $conf->{cps_diskgroup} = $cfg->{cps_diskgroup};
        $conf->{cps_volume} = $cfg->{cps_volume};
        $conf->{cps_blockdevice} = "$pkg->{device_path}/$conf->{cps_diskgroup}/$conf->{cps_volume}";
        if ($cfg->{cps_newdg_disks}) {
            # create new diskgroup
            $line = join(' ', @{$cfg->{cps_newdg_disks}});
            return 1 if (!$vxfen_pkg->create_new_dg_sys($sys0,$conf->{cps_diskgroup},$line));
        }
        if ($cfg->{cps_newvol_volsize}) {
            $msg = Msg::new("Creating volume '$conf->{cps_volume}' for CP Server database on $sys0->{sys}");
            $msg->left;
            if ($cfg->{cps_enable_mirroring}) {
                $sys0->cmd("_cmd_vxassist -g $conf->{cps_diskgroup} make $conf->{cps_volume} $cfg->{cps_newvol_volsize}m layout=mirror");
            } else {
                $sys0->cmd("_cmd_vxassist -g $conf->{cps_diskgroup} make $conf->{cps_volume} $cfg->{cps_newvol_volsize}m");
            }
            if (EDR::cmdexit()) {
                $msg->right_failed();
                return 1;
            } else {
                $msg->right_done();
            }
            $ret = $fs->make_vxfs_sys($sys0,$conf->{cps_diskgroup},$conf->{cps_volume});
            Msg::log("Make fs return $ret");
        }
        $ret = $pkg->update_maincf($conf,$config_flag);
        return $ret;
    }
    if (Obj::webui()) {
        my $result1 = $web->web_script_form('cps_sg1',$conf,$config_flag);
        @nic_list = @{$result1->{nic_list}};
        %vip2nicres_map = $result1->{vip2nicres_map};
        @nicres_list = @{$result1->{nicres_list}};
        
        my $result2 = $web->web_script_form('cps_sg2',$conf,$result1->{nic_list},$pkg);
        @cps_netmasks = @{$result2->{cps_netmasks}};
        @network_hosts = @{$result2->{network_hosts}};
        @cps_prefix_length = @{$result2->{cps_prefix_length}};
        
        if ($config_flag->{multinode_vcs}) {
            
            my $vxfen_pkg = $pkg->pkg('VRTSvxfen60');
            my $dglist = $vxfen_pkg->get_dglist_sys($sys0);            
            my @disks = @{$dglist->{disks}} if (defined($dglist->{disks}));
            my ($ret,$vxvmdisks);
            
            if(@disks<1) {
                 my $numdisks = $#disks + 1;
                 $msg=Msg::new("A new disk group cannot be created since no free VxVM CDS disks are available. If there are disks available which are not under VxVM control, use the command vxdisksetup or use the installer to initialize them as VxVM disks.");
                 $web->web_script_form('alert',$msg->{msg});
                 ($ret,$vxvmdisks) = $vxfen_pkg->init_vxvm_disks();
                 if (@{$vxvmdisks}) {
                     $dglist->{disks}=\@{EDRu::arruniq(@disks,@{$vxvmdisks})};
                 }
            }
            
            my $result3 = $web->web_script_form('cps_dg',$conf,$pkg,$sys0,$dglist);
            $cps_diskgroup = $result3->{cps_diskgroup};
            my $result4 = $web->web_script_form('cps_vol',$conf,$pkg,$sys0,$cps_diskgroup);
            $cps_volume = $result4->{cps_volume};
            $cps_blockdevice = "$pkg->{device_path}/$cps_diskgroup/$cps_volume";
        }
        
    } else {
        if ($numvips > 1) {
            Msg::n();
            do {
                $msg = Msg::new("Enter how many NIC resources you want to configure (1 to $numvips):");
                $nic_count = $msg->ask();
            } while ("$nic_count" eq "" || $nic_count =~ /\D/ || $nic_count > $numvips || $nic_count < 1);
        } else {
            $nic_count = $numvips;
        } 
        $i = 1;
        for (my $count = 0; $count < $nic_count; $count++) {
            if ($config_flag->{multinode_vcs}) {
                Msg::n();
                $msg = Msg::new("Is the name of network interfaces for NIC resource - $i same on all the systems?");                
                $answer = $msg->ayn();
                $nic_same = ($answer eq 'Y') ? 1 : 0;
            }
            @nic_set_for_system = ();
            if ($nic_same) {
                Msg::n();
                while (1) {
                    $msg = Msg::new("Enter a valid network interface for NIC resource - $i:");
                    $answer = $msg->ask();
                    for my $sys(@systems) {
                        if (!check_nic_sys($sys,$answer)) {
                            $msg = Msg::new("Network interface validation failed on $sys->{sys}");
                            $msg->print;
                            $validated = 0;
                            last;
                        } else {
                            $validated = 1;
                        }
                    }
                    last if ($validated);
                }
                foreach (@systems) {
                    push (@nic_set_for_system, $answer);
                }
            } else {
                Msg::n();
                for my $sys(@systems) {
                    while(1) {
                        $msg = Msg::new("Enter a valid network interface on $sys->{sys} for NIC resource - $i:");
                        $answer = $msg->ask();
                        if (!check_nic_sys($sys,$answer)) {
                            $msg = Msg::new("Network interface validation failed on $sys->{sys}");
                            $msg->print;
                            next;
                        }
                        last;
                    }
                    push (@nic_set_for_system, $answer);
                }
            }
            $nic_set[$count] = [ @nic_set_for_system ];
            $i++;
        }

        $count = 0;
        @cps_virtual_ip = @{$conf->{vips}};
        foreach(@cps_virtual_ip) {
            @nic_list_for_vip = ();
            if ($nic_count > 1) {
                Msg::n();
                do {
                    $msg = Msg::new("Enter the NIC resource you want to associate with the virtual IP $cps_virtual_ip[$count] (1 to $nic_count):");
                    $answer = $msg->ask();
                } while ($answer =~ /\D/ || $answer > $nic_count || $answer < 1);
            } else {
                $answer = 1;
            }

            $vip2nicres_map{$cps_virtual_ip[$count]} = $answer;
            $nicres_list[$answer-1] = 0;
            $i = 0;
            foreach(@systems) {
                push(@nic_list_for_vip, $nic_set[$answer-1][$i]);
                $i++;
            }
            $nic_list[$count] = [ @nic_list_for_vip ];
            $count++;
        }

        ##  Check if Network Hosts attribute can be added to NIC resource
        Msg::n();
        $msg = Msg::new("Symantec recommends configuring NetworkHosts attribute to ensure NIC resource to be always online");
        $msg->print;
        $i = 0;
        $numnics = 0;
        foreach (@nic_list) {
            $j = 0;
            $nwhosts_same = 0;
            while ($nic_list[$i][$j]) {
                if ($nwhosts_same) {
                    $network_hosts[$numnics] = [ @network_hosts_for_nic ];
                    $nics_taken_care_of{$nic_list[$i][$j].'@'.$system_list[$j]}  = '1';
                    $numnics++;
                    $j++;
                    next;
                }
                @network_hosts_for_nic = ();
                # We should ask the question for NetworkHosts attribute for every combination of (node, nic).
                # If a (node, nic) combination has already been taken care of, then we can skip that pair safely.
                if ($nics_taken_care_of{$nic_list[$i][$j].'@'.$system_list[$j]} eq '1') {
                    $j++;
                    next;
                }
                $msg = Msg::new("Do you want to add NetworkHosts attribute for the NIC device $nic_list[$i][$j] on system $systems[$j]->{sys}?");
                $answer = $msg->ayn;
                $nics_taken_care_of{$nic_list[$i][$j].'@'.$system_list[$j]}  = '1';
                if ($answer eq 'Y') {
                    do {
                        while (1) {
                            Msg::n();
                            $msg = Msg::new("Enter a valid IP address to configure NetworkHosts for NIC $nic_list[$i][$j] on system $systems[$j]->{sys}:");
                            $answer = $msg->ask();
                            if (!$vcs->is_ip_valid($answer)) {
                                $msg = Msg::new("Invalid NetworkHosts value for the network interface/NIC $nic_list[$i][$j]");
                                $msg->print;
                                next;
                            }
                            last;
                        }
                        push (@network_hosts_for_nic, $answer);
                        Msg::n();
                        $msg = Msg::new("Do you want to add another Network Host?");
                        $answer = $msg->ayn();
                    } while ($answer eq 'Y');
                    if ($j == 0 && $config_flag->{multinode_vcs}) {
                        $msg = Msg::new("Do you want to apply the same NetworkHosts for all systems?");
                        $answer = $msg->ayny();
                        $nwhosts_same = 1 if ($answer eq 'Y');
                    }
                }
                $network_hosts[$numnics] = [ @network_hosts_for_nic ];
                $numnics++;
                $j++;
            }
            $i++;
        }
        # Get the netmask
        $i = 0;
        foreach(@cps_virtual_ip) {
            $defopt = '';
            Msg::n();
            if ($is_ipv4_ipv6[$i] == 1) {
                $defopt = ($sys0->linux()) ? '[0, 128]' : '[1, 128]';
                $msg = Msg::new("Enter the prefix length for virtual IP ${cps_virtual_ip[$i]}:");
                $answer = $msg->ask($defopt);
                redo if (!$pkg->validate_prefix_length($answer));
                $cps_prefix_len = $answer;
                push (@cps_prefix_length, $cps_prefix_len);

            } else {
                $defopt = $sys0->defaultnetmask(${cps_virtual_ip[$i]},$nic_list[$i][0]);
                $msg = Msg::new("Enter the netmask for virtual IP ${cps_virtual_ip[$i]}:");
                $answer = $msg->ask($defopt);
                if (!EDRu::ip_is_ipv4($answer) || ($answer =~ /^0+\./m)) {
                    $msg = Msg::new("$answer is not a valid Netmask. Input again");
                    $msg->print;
                    redo;
                }
                $cps_netmask = $answer;
                push (@cps_netmasks, $cps_netmask);
            }
            $i++;
        }

        if ($config_flag->{multinode_vcs}) {
            Msg::n();
            ($ret,$answer) = $pkg->ask_dg_sys($sys0);
            return $ret if ($ret);
            $cps_diskgroup = $answer;
            Msg::n();
            ($ret,$answer) = $pkg->ask_vol_sys($sys0,$cps_diskgroup);
            return $ret if ($ret);
            $cps_volume = $answer;
            $cps_blockdevice = "$pkg->{device_path}/$cps_diskgroup/$cps_volume";
        }
    }

    $conf->{prefix_length} = [ @cps_prefix_length ];
    $conf->{netmasks} = [ @cps_netmasks ];
    $conf->{nic_list} = [ @nic_list ];
    $conf->{network_hosts} = [ @network_hosts ];
    $conf->{vip2nicres_map} = \%vip2nicres_map;
    $conf->{nicres_list} = [ @nicres_list ];
    $conf->{cps_blockdevice} = $cps_blockdevice;
    $conf->{cps_diskgroup} = $cps_diskgroup;
    $conf->{cps_volume} = $cps_volume;
    # Save cfg variables for responsefile
    $cfg->{cpsname} = $conf->{cpsname};
    $cfg->{cps_vips} = $conf->{vips};
    $cfg->{cps_ports} = $conf->{ports};
    $cfg->{cps_db_dir} = $conf->{cps_db_dir};
    $cfg->{cps_prefix_length} = $conf->{prefix_length} if ($conf->{prefix_length});
    $cfg->{cps_netmasks} = $conf->{netmasks} if ($conf->{netmasks});
    for $i(0..$#nic_list) {
        $j = $i + 1;
        $cfg->{cps_nic_list}{"cpsvip$j"} = $nic_list[$i];
    }
    for $i(0..$#network_hosts) {
        $j = $i + 1;
        $cfg->{cps_network_hosts}{"cpsnic$j"} = $network_hosts[$i];
    }
    $cfg->{cps_vip2nicres_map} = $conf->{vip2nicres_map};
    $cfg->{cps_diskgroup} = $conf->{cps_diskgroup};
    $cfg->{cps_volume} = $conf->{cps_volume};

    $ret = $pkg->update_maincf($conf,$config_flag);
    return $ret;
}

1;